/*
 * Copyright (c) 2025 EdgeImpulse Inc.
 *
 * Generated by Edge Impulse and licensed under the applicable Edge Impulse
 * Terms of Service. Community and Professional Terms of Service
 * (https://edgeimpulse.com/legal/terms-of-service) or Enterprise Terms of
 * Service (https://edgeimpulse.com/legal/enterprise-terms-of-service),
 * according to your product plan subscription (the “License”).
 *
 * This software, documentation and other associated files (collectively referred
 * to as the “Software”) is a single SDK variation generated by the Edge Impulse
 * platform and requires an active paid Edge Impulse subscription to use this
 * Software for any purpose.
 *
 * You may NOT use this Software unless you have an active Edge Impulse subscription
 * that meets the eligibility requirements for the applicable License, subject to
 * your full and continued compliance with the terms and conditions of the License,
 * including without limitation any usage restrictions under the applicable License.
 *
 * If you do not have an active Edge Impulse product plan subscription, or if use
 * of this Software exceeds the usage limitations of your Edge Impulse product plan
 * subscription, you are not permitted to use this Software and must immediately
 * delete and erase all copies of this Software within your control or possession.
 * Edge Impulse reserves all rights and remedies available to enforce its rights.
 *
 * Unless required by applicable law or agreed to in writing, the Software is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language governing
 * permissions, disclaimers and limitations under the License.
 */
// Generated on: 10.04.2025 20:10:42

#include <stdio.h>
#include <stdlib.h>
#include "edge-impulse-sdk/tensorflow/lite/c/builtin_op_data.h"
#include "edge-impulse-sdk/tensorflow/lite/c/common.h"
#include "edge-impulse-sdk/tensorflow/lite/micro/micro_mutable_op_resolver.h"
#include "edge-impulse-sdk/porting/ei_classifier_porting.h"

#if EI_CLASSIFIER_PRINT_STATE
#if defined(__cplusplus) && EI_C_LINKAGE == 1
extern "C" {
    extern void ei_printf(const char *format, ...);
}
#else
extern void ei_printf(const char *format, ...);
#endif
#endif

#define STRINGIZE(x) #x
#define STRINGIZE_VALUE_OF(x) STRINGIZE(x)

#if defined (__GNUC__)  /* GNU compiler */
#define ALIGN(X) __attribute__((aligned(X)))
#define DEFINE_SECTION(x) __attribute__((section(x)))
#elif defined (_MSC_VER)
#define ALIGN(X) __declspec(align(X))
#elif defined (__TASKING__) /* TASKING Compiler */
#define ALIGN(X) __align(X)
#define DEFINE_SECTION(x) __attribute__((section(x)))
#elif defined (__ARMCC_VERSION) /* Arm Compiler */
#define ALIGN(X) __ALIGNED(x)
#define DEFINE_SECTION(x) __attribute__((section(x)))
#elif defined (__ICCARM__) /* IAR Compiler */
#define ALIGN(x) __attribute__((aligned(x)))
#define DEFINE_SECTION(x) __attribute__((section(x)))
#elif defined (__clang__) /* LLVM/Clang Compiler */
#define ALIGN(X) __ALIGNED(x)
#define DEFINE_SECTION(x) __attribute__((section(x)))
#endif

#if defined(EI_MODEL_SECTION) && (defined(__GNUC__) || defined(__clang__))
#define MODEL_SECTION(X) __attribute__((section(STRINGIZE_VALUE_OF(X))))
#else
#define MODEL_SECTION(X)
#endif

#ifndef EI_MAX_SCRATCH_BUFFER_COUNT
#ifndef CONFIG_IDF_TARGET_ESP32S3
#define EI_MAX_SCRATCH_BUFFER_COUNT 14
#else
#define EI_MAX_SCRATCH_BUFFER_COUNT 28
#endif // CONFIG_IDF_TARGET_ESP32S3
#endif // EI_MAX_SCRATCH_BUFFER_COUNT

#ifndef EI_MAX_OVERFLOW_BUFFER_COUNT
#define EI_MAX_OVERFLOW_BUFFER_COUNT 10
#endif // EI_MAX_OVERFLOW_BUFFER_COUNT

using namespace tflite;
using namespace tflite::ops;
using namespace tflite::ops::micro;

namespace {

#if defined(EI_CLASSIFIER_ALLOCATION_STATIC_HIMAX) || defined(EI_CLASSIFIER_ALLOCATION_STATIC_HIMAX_GNU)
constexpr int kTensorArenaSize = 242656;
#else
constexpr int kTensorArenaSize = 241632;
#endif

#if defined(EI_CLASSIFIER_ALLOCATION_STATIC)
#if defined (EI_TENSOR_ARENA_LOCATION)
uint8_t tensor_arena[kTensorArenaSize] ALIGN(16) DEFINE_SECTION(STRINGIZE_VALUE_OF(EI_TENSOR_ARENA_LOCATION));
#else
uint8_t tensor_arena[kTensorArenaSize] ALIGN(16);
#endif
#elif defined(EI_CLASSIFIER_ALLOCATION_STATIC_HIMAX)
#pragma Bss(".tensor_arena")
uint8_t tensor_arena[kTensorArenaSize] ALIGN(16);
#pragma Bss()
#elif defined(EI_CLASSIFIER_ALLOCATION_STATIC_HIMAX_GNU)
uint8_t tensor_arena[kTensorArenaSize] ALIGN(16) __attribute__((section(".tensor_arena")));
#else
#define EI_CLASSIFIER_ALLOCATION_HEAP 1
uint8_t* tensor_arena = NULL;
#endif

static uint8_t* tensor_boundary;
static uint8_t* current_location;

template <int SZ, class T> struct TfArray {
  int sz; T elem[SZ];
};

enum used_operators_e {
  OP_CONV_2D, OP_DEPTHWISE_CONV_2D, OP_PAD, OP_ADD, OP_SOFTMAX,  OP_LAST
};

struct TensorInfo_t { // subset of TfLiteTensor used for initialization from constant memory
  TfLiteAllocationType allocation_type;
  TfLiteType type;
  void* data;
  TfLiteIntArray* dims;
  size_t bytes;
  TfLiteQuantization quantization;
};

typedef struct {
  TfLiteTensor tensor;
  int16_t index;
} TfLiteTensorWithIndex;

typedef struct {
  TfLiteEvalTensor tensor;
  int16_t index;
} TfLiteEvalTensorWithIndex;

TfLiteContext ctx{};
static const int MAX_TFL_TENSOR_COUNT = 4;
static TfLiteTensorWithIndex tflTensors[MAX_TFL_TENSOR_COUNT];
static const int MAX_TFL_EVAL_COUNT = 4;
static TfLiteEvalTensorWithIndex tflEvalTensors[MAX_TFL_EVAL_COUNT];
TfLiteRegistration registrations[OP_LAST];

namespace g0 {
const TfArray<4, int> tensor_dimension0 = { 4, { 1,96,96,3 } };
const TfArray<1, float> quant0_scale = { 1, { 0.0039215688593685627, } };
const TfArray<1, int> quant0_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant0 = { (TfLiteFloatArray*)&quant0_scale, (TfLiteIntArray*)&quant0_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data1[4*2] = { 
  0, 0, 
  0, 1, 
  0, 1, 
  0, 0, 
};
const TfArray<2, int> tensor_dimension1 = { 2, { 4,2 } };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(8) int32_t tensor_data2[3] = { 26260, -32122, -34523, };
const TfArray<1, int> tensor_dimension2 = { 1, { 3 } };
const TfArray<3, float> quant2_scale = { 3, { 0.00019343454914633185, 0.00017067490261979401, 0.0001673264050623402, } };
const TfArray<3, int> quant2_zero = { 3, { 0,0,0 } };
const TfLiteAffineQuantization quant2 = { (TfLiteFloatArray*)&quant2_scale, (TfLiteIntArray*)&quant2_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data3[3*1*1*32] = { 
  /* [0][0][][] */ 88,86,127,-58,-35,12,-16,44,-41,94,-5,17,-34,-68,97,-12,-22,47,119,117,34,-16,25,-98,-98,85,-81,14,99,55,-14,28, 
  /* [1][0][][] */ 53,-33,-66,10,-20,127,-9,-14,120,-61,-12,-99,-58,102,20,-58,93,-56,-5,-93,-122,11,118,68,11,27,11,-4,-68,-52,-11,-25, 
  /* [2][0][][] */ -39,74,-77,67,73,-48,-22,116,93,32,127,-33,-96,-86,74,121,-89,-46,-60,-116,72,114,-49,-113,86,-50,-10,-117,11,-83,86,-17, 
};
const TfArray<4, int> tensor_dimension3 = { 4, { 3,1,1,32 } };
const TfArray<3, float> quant3_scale = { 3, { 0.0035559481475502253, 0.0031375528778880835, 0.0030759966466575861, } };
const TfArray<3, int> quant3_zero = { 3, { 0,0,0 } };
const TfLiteAffineQuantization quant3 = { (TfLiteFloatArray*)&quant3_scale, (TfLiteIntArray*)&quant3_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data4[32] = { -374, -1286, -289, -997, -431, -127, -816, -197, -244, 285, -195, 1228, -836, -1062, -1019, -203, -106, -806, 1629, -580, 493, -524, -29, 249, -818, -397, -680, -892, -27, 99, -700, -433, };
const TfArray<1, int> tensor_dimension4 = { 1, { 32 } };
const TfArray<32, float> quant4_scale = { 32, { 4.604674904840067e-05, 4.5397249778034166e-05, 4.5260723709361628e-05, 4.6395132812904194e-05, 4.7472141886828467e-05, 4.6111828851280734e-05, 4.6243196266004816e-05, 4.7751367674209177e-05, 4.2698495235526934e-05, 4.8784950195113197e-05, 5.5547268857480958e-05, 5.1683375204447657e-05, 4.8167279601329938e-05, 4.6742323320358992e-05, 4.8110548959812149e-05, 4.8833346227183938e-05, 4.2900937842205167e-05, 4.6818808186799288e-05, 6.3366031099576503e-05, 5.1007376896450296e-05, 4.645960871130228e-05, 4.9679711082717404e-05, 5.4155396355781704e-05, 4.9192174628842622e-05, 4.6436849515885115e-05, 4.4658754632109776e-05, 4.6801495045656338e-05, 4.5184533519204706e-05, 6.320438114926219e-05, 4.3280961108393967e-05, 4.5857756049372256e-05, 5.109537232783623e-05, } };
const TfArray<32, int> quant4_zero = { 32, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant4 = { (TfLiteFloatArray*)&quant4_scale, (TfLiteIntArray*)&quant4_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data5[32*1*1*96] = { 
  /* [0][0][][] */ -56,-53,80,-13,90,-34,-90,-117,43,-117,-110,-3,24,3,-42,44,-60,-37,-96,-37,-75,-108,-8,93,-99,21,64,-63,97,8,-29,-92,78,-72,-90,54,-112,-75,-100,16,68,-94,-106,85,46,-105,103,-26,13,-45,-38,93,-100,-63,-89,108,-19,44,88,-111,-69,64,-47,20,-20,12,54,73,58,64,41,-127,83,-89,20,63,-101,-1,-65,35,48,6,88,-18,-60,-46,-88,102,20,107,-24,-126,14,67,42,-6, 
  /* [1][0][][] */ -52,8,-82,1,-37,76,9,0,62,-127,-113,78,78,-35,-6,27,75,-54,40,68,-43,-32,4,2,6,-22,-70,97,-8,-22,62,-56,21,92,-19,6,22,43,-19,-86,-76,-6,-22,-117,-67,29,-39,-71,-59,-110,121,49,12,78,-95,-67,45,98,59,43,-53,95,-88,-95,-40,13,47,27,66,33,33,15,-109,73,-60,23,77,-73,-65,32,-112,86,-45,-114,-10,78,-81,-75,69,53,86,40,83,61,13,10, 
  /* [2][0][][] */ 37,35,-46,93,-37,-63,74,83,-96,72,-45,92,66,-27,69,-113,-67,-34,43,30,-110,-19,102,8,-35,56,94,14,-85,54,71,74,-53,-22,-102,-102,-92,74,85,-32,-127,-85,3,-20,-91,96,-42,-88,96,13,64,-104,-37,54,2,-2,-68,77,-81,2,-36,43,-54,106,69,27,43,82,-108,59,1,-87,15,-86,24,34,-71,-25,-3,39,22,-108,-31,-52,-9,-103,-28,-49,67,93,-26,-11,-62,-91,-65,-64, 
  /* [3][0][][] */ 65,-25,8,-39,-49,20,-123,14,31,6,46,-9,54,77,47,-11,72,78,-88,23,100,-104,-84,-127,-49,-113,-93,74,23,-81,62,13,-77,-48,-21,-38,-110,-30,27,27,-32,-101,-106,-19,52,17,-48,-94,41,-77,54,8,65,19,74,-99,-121,-116,-117,-76,-81,12,-29,41,64,-40,-80,-48,-37,-85,-23,93,11,-114,26,-72,32,66,69,-82,-98,-113,23,79,9,26,-58,-106,-88,30,-24,-86,93,-117,-80,17, 
  /* [4][0][][] */ 57,65,-47,46,5,-49,25,0,49,55,-3,24,23,-32,-16,-60,-53,-4,-101,27,96,-92,62,16,-7,6,48,-96,-81,42,-76,-35,51,101,38,-118,-76,1,-94,41,-39,-54,-70,-87,32,98,44,54,10,-79,-2,-53,-127,42,-64,26,14,-84,46,0,16,-106,-18,-82,-24,50,-10,-43,-8,-99,-106,-66,-39,25,56,5,64,-40,-13,-81,-6,-24,-81,-16,-11,59,75,5,-113,-70,-6,57,35,42,-78,-41, 
  /* [5][0][][] */ 56,-93,61,47,38,-127,25,42,85,-111,-5,-47,-5,87,30,37,60,50,-76,107,28,44,11,-26,-89,-39,-15,-93,57,45,13,-115,-52,-29,-71,8,-106,-32,43,22,-80,-50,38,89,-31,21,28,-45,-109,123,2,40,22,-16,34,-14,62,28,-79,-37,29,-66,48,-98,-43,100,-30,22,90,-1,72,-70,-51,74,-43,-75,13,-51,11,66,109,22,30,91,-72,79,-5,50,66,-91,103,63,-8,32,83,-92, 
  /* [6][0][][] */ -1,-53,96,54,-95,-62,-19,46,-31,-89,-12,-100,63,34,90,-127,23,-15,4,75,-90,-126,87,38,-74,-95,-24,-61,-43,-125,28,-89,-96,-127,-60,-106,-21,18,-117,58,62,34,5,-30,-122,80,-101,-60,-94,-4,-42,-80,50,-66,70,-43,-5,-24,-79,90,-107,-13,43,-58,-62,43,78,-61,55,-19,-111,-23,33,-83,-92,-50,-93,-107,-69,-42,14,-44,22,-41,-83,-97,-54,-106,60,70,79,1,-67,-77,52,-37, 
  /* [7][0][][] */ 114,41,-71,-34,-47,35,-17,2,108,-8,-82,5,66,-77,40,68,27,17,-45,113,55,28,-18,75,-51,98,31,21,-82,99,-26,44,13,21,77,73,-125,83,5,-21,90,-66,80,61,50,4,102,-94,40,-67,-83,-50,-54,80,27,38,104,-74,23,-61,16,-50,76,-123,-49,-46,57,24,-127,24,61,87,84,29,6,27,-46,31,-91,14,77,-19,-2,-61,73,27,-99,91,78,8,61,74,-38,-62,11,-56, 
  /* [8][0][][] */ 13,16,-66,-102,83,-8,-76,-28,1,35,46,112,18,-45,-89,-70,-71,92,-79,36,36,-30,72,108,-105,112,43,62,-56,37,-96,127,78,-44,58,-18,110,8,-11,74,86,104,-108,-120,-8,-23,-14,114,9,-65,-45,13,-84,-15,-24,-97,24,112,-78,102,123,34,57,78,-17,-39,-28,99,-65,55,15,7,80,77,-18,5,-36,95,82,48,41,91,-98,-81,26,43,75,-29,42,-36,-96,116,-66,-98,-28,63, 
  /* [9][0][][] */ 26,9,107,54,81,-107,-37,-82,46,109,3,44,-37,-75,-82,-12,0,2,-38,-70,-4,94,-90,-87,127,-17,-14,20,-18,-36,-79,112,43,17,38,11,-68,27,-9,-37,50,-41,75,-78,83,22,-78,59,38,-27,-87,-47,103,-56,-63,85,109,-66,13,-2,-92,112,55,-33,-53,8,100,42,-41,117,102,-70,18,77,-33,-81,-82,-98,-30,100,27,25,78,28,-36,-89,36,-15,79,29,-33,117,80,-5,45,-11, 
  /* [10][0][][] */ -41,45,40,-51,3,21,83,21,30,14,22,-56,90,65,-30,9,-42,64,-127,-38,-9,18,32,-57,33,49,-96,-114,-41,67,-80,-54,60,75,108,41,-56,9,-15,29,-24,92,-85,-30,-73,-40,5,-42,-8,-6,43,18,17,18,-40,-18,-81,-7,40,-43,6,28,93,14,102,-56,-80,-15,3,-16,-73,-43,73,56,-103,60,-2,34,50,-9,56,11,56,84,-83,51,77,-14,32,9,-20,-41,-30,-56,-23,-51, 
  /* [11][0][][] */ 120,56,20,119,85,84,64,67,114,127,-42,102,-5,-55,59,-42,-28,-63,67,43,39,85,49,-64,-62,-80,3,80,77,-15,45,24,-37,-21,56,77,-24,92,17,-40,-3,104,-44,40,7,-26,-60,-48,103,86,88,48,-49,-37,56,46,-17,-42,20,-56,-67,119,69,23,63,87,-21,33,74,67,-41,-52,54,120,-33,72,-49,58,-20,111,-76,-9,-47,-29,57,-52,26,-19,-64,24,28,-49,-67,-14,-41,-17, 
  /* [12][0][][] */ -6,47,-69,-20,62,2,-52,12,-88,35,-33,-9,-59,-8,22,-89,-108,-48,-56,-93,3,-30,-3,66,-80,-66,69,-110,60,-7,-52,85,-89,-80,-63,0,-20,-86,-35,17,83,16,-120,-54,-39,-94,-55,83,-57,-127,-89,73,14,-16,-94,54,-34,-70,26,42,33,82,-22,56,83,9,-61,83,29,70,23,45,55,82,-24,-24,-52,14,-125,-84,-72,73,20,-6,-52,10,29,82,-104,-117,62,22,71,-33,4,47, 
  /* [13][0][][] */ 81,-75,81,57,74,94,52,30,39,-101,-20,-23,62,34,43,-74,-111,-92,-106,14,38,-47,90,-61,-74,38,-10,5,-78,-106,49,18,43,72,-42,15,-81,-47,-14,31,-82,42,-82,-21,-64,2,-21,-51,16,105,-52,-118,-117,11,53,-95,-80,-2,-41,-82,-45,104,-83,-16,-65,-86,-105,53,-18,-46,57,-26,-108,20,-78,-54,-127,47,53,-76,39,-111,77,-4,-91,-115,53,87,-27,-67,-76,33,81,-38,62,-37, 
  /* [14][0][][] */ -76,-110,56,37,56,-47,-37,-113,-95,-56,39,57,-29,-102,-104,-98,-69,-80,-2,-1,-62,6,-26,33,-124,-87,-56,6,-116,-119,-96,11,65,-46,-52,-16,64,81,27,-124,-29,-123,13,59,38,-123,34,-98,76,29,50,24,66,-40,-36,-11,-67,6,19,65,-40,-97,-47,41,-101,-33,46,-118,-98,-95,-94,59,-17,51,21,-88,33,-120,26,4,17,-41,-126,-54,-127,-104,-118,72,75,-16,60,-66,68,70,-38,40, 
  /* [15][0][][] */ 29,-44,-57,77,102,-91,58,-71,59,-20,-52,-12,113,9,-81,-96,53,17,-72,31,52,16,53,94,80,59,31,69,-3,54,47,-74,1,11,-27,-22,-86,107,117,-53,22,105,-39,-24,26,-61,-25,-63,-9,-105,16,40,-90,-82,-28,6,77,58,-18,87,-26,-75,7,88,-15,88,-62,10,-85,12,72,-49,30,-4,38,-127,14,31,94,37,-85,71,-77,-48,-117,-35,-79,39,82,66,-104,-18,63,73,-80,106, 
  /* [16][0][][] */ -89,3,98,-91,31,102,-56,28,27,57,-127,95,-33,-67,-53,-59,118,60,65,72,102,77,-39,53,-35,82,-64,60,-83,36,47,-3,-122,57,32,56,-26,43,-53,-80,12,-62,63,93,53,89,90,-11,-65,-74,-12,104,-51,101,59,35,-19,74,42,20,101,112,40,37,40,13,-41,75,75,87,-59,-30,9,85,51,86,-27,98,-109,3,34,-109,51,2,24,109,99,12,77,12,-18,96,60,-16,-69,11, 
  /* [17][0][][] */ 16,-117,12,59,-59,-100,-79,-109,23,-99,60,29,75,38,-125,29,12,32,119,85,-103,-30,-4,-66,4,44,-89,5,79,-66,-46,54,-39,-37,11,-11,-62,74,-102,-45,-106,-55,-33,-27,-12,-6,61,-121,63,77,-31,6,-34,58,72,-88,-127,63,-97,-119,12,-111,52,81,-94,-86,22,-111,-122,-21,87,9,-3,-127,-81,47,90,-61,21,-63,-8,-32,-90,-101,-121,50,57,-49,1,53,-110,-124,-29,67,-8,-5, 
  /* [18][0][][] */ -51,109,121,30,-32,-5,-44,34,18,63,-57,-39,-40,47,101,-36,39,2,78,-46,75,11,49,-17,22,-81,29,108,31,-9,36,53,25,12,-80,-54,4,27,-18,64,-26,14,9,33,-30,37,17,25,24,-30,33,-9,-52,59,87,108,41,22,-12,-66,34,127,-17,47,6,-54,23,77,41,-11,-3,68,-68,-32,25,-22,27,72,18,-5,-20,-26,-3,-39,100,-10,68,51,-55,8,-2,107,42,11,21,18, 
  /* [19][0][][] */ 7,12,54,23,39,-80,71,66,69,-21,14,-21,-90,17,-58,43,52,-97,127,38,-29,58,-55,1,54,-79,17,-64,-86,-108,-63,29,59,-78,16,-10,-5,-69,-60,-92,26,-107,-79,-4,-85,14,52,-89,-33,21,-43,28,-16,3,-7,-20,15,74,-14,-26,36,-60,74,47,-69,-13,-31,-8,-14,39,62,-51,-62,-112,-54,-74,-29,-102,3,84,39,-107,-35,55,-82,-15,-2,-14,65,14,-11,44,-37,87,48,-41, 
  /* [20][0][][] */ 20,77,79,121,-54,91,30,64,38,4,-47,-5,17,94,49,-55,-44,0,-55,-39,35,-27,-57,-55,93,-75,-83,52,15,2,-31,126,111,71,9,64,41,2,-16,-77,-52,-5,-49,-86,-4,-25,-33,106,68,-127,-72,45,55,122,54,2,124,99,119,89,122,-49,-58,-2,-8,-50,-33,36,-91,-26,-30,104,-21,-54,38,29,119,83,113,96,77,42,80,5,58,115,2,84,23,94,37,80,27,11,-14,36, 
  /* [21][0][][] */ 80,38,30,-99,103,-127,56,2,-12,-69,-66,38,18,27,-59,-84,-104,-34,-6,-4,94,-55,44,-82,53,-68,-39,40,12,10,-22,-14,-12,3,-72,-126,75,80,-95,-34,-9,0,20,-21,-94,-73,-66,77,-37,-50,8,83,5,49,-82,30,-60,48,-32,93,52,-91,18,-45,81,24,-45,-68,-92,-8,43,71,28,-21,22,33,-90,-45,45,59,4,93,41,-66,11,-91,49,-69,-89,63,74,-86,-29,-88,69,-32, 
  /* [22][0][][] */ 21,-46,-58,-74,39,-65,88,33,-68,51,-66,-73,64,-112,-87,-73,26,59,-109,-60,52,-58,17,27,78,-6,-88,69,-49,-97,-55,-103,22,34,1,2,0,-19,-80,-87,-5,-20,0,69,45,-101,66,-21,-64,-85,-5,-31,-85,-97,-43,21,-44,-45,-43,-33,57,-65,-82,-127,-70,12,-89,-33,81,82,-49,-22,-79,4,32,-69,5,-50,53,84,84,-108,-76,-107,61,63,71,-33,-62,24,-63,29,-10,-3,-18,68, 
  /* [23][0][][] */ 58,-71,-2,35,97,-127,10,-108,47,-72,83,85,1,-87,18,-21,85,-22,-122,121,87,-70,94,-25,-1,16,81,7,74,82,74,-13,-33,55,2,36,92,16,-119,13,-73,11,14,-90,-50,-60,116,27,73,-63,-82,-20,46,-31,-49,86,-108,-11,65,49,-80,-86,-17,-24,34,52,-104,-97,83,-12,73,-56,-1,-11,-51,-84,57,-17,89,127,-26,-47,6,-64,97,21,110,12,-73,51,80,12,112,-38,-10,63, 
  /* [24][0][][] */ -87,-64,-117,-81,-86,-115,-84,-102,42,91,59,22,90,39,-57,-127,-95,20,5,-55,71,-95,-71,-54,33,-39,47,33,79,38,-16,-70,-3,-10,32,9,-69,-96,46,-39,-86,-112,76,-14,1,-52,25,48,-92,-63,-32,-1,45,38,46,-85,-23,57,13,-18,70,-25,23,68,-100,71,-63,-99,-112,29,-72,-43,-52,26,72,74,40,-90,-117,-93,-93,-101,-76,-125,-18,40,-31,-110,6,-119,-13,-73,-32,6,-98,9, 
  /* [25][0][][] */ -99,-35,-57,-26,74,-75,-117,41,31,-107,-53,11,77,-11,-118,-37,-66,40,45,-112,72,68,-17,-69,0,-47,-125,54,73,-90,38,97,100,-117,22,89,92,-74,-34,-37,-98,82,-118,-6,83,28,44,21,-99,92,-51,-15,58,11,-81,15,-36,27,-118,57,98,58,27,32,-11,58,-29,-12,-77,5,-25,-122,17,-123,86,37,65,66,-1,42,32,34,-40,69,-84,-65,-110,-105,-16,-54,93,5,-127,-22,-16,-70, 
  /* [26][0][][] */ 38,-115,-45,-35,-81,-39,-20,-37,-118,-127,87,92,-85,-50,-12,-16,108,23,-92,-55,7,51,85,1,35,66,8,22,14,35,-96,-16,7,-66,-89,-92,-51,72,-68,93,-96,87,42,14,-83,56,-43,79,-24,72,-18,87,-20,-54,-23,-113,63,60,36,26,1,-119,83,6,92,-29,-68,39,-43,66,69,82,70,-98,-103,-13,40,-104,-97,1,-30,67,62,-77,63,1,-47,5,-84,90,-12,3,-54,77,33,74, 
  /* [27][0][][] */ -4,-4,-79,-115,-125,-83,-86,-33,21,81,17,-124,-41,36,58,-47,40,-32,67,-98,-107,-10,-46,-43,-111,24,-119,-23,39,5,-46,-22,38,49,-109,-94,-107,-79,-126,66,-107,-1,11,-1,74,7,47,77,64,44,41,-88,-97,-95,-24,-51,83,-39,-127,-94,-72,-79,-52,78,24,-62,-127,82,23,-4,-124,-9,40,-69,-16,-102,27,-81,63,80,62,-44,91,-67,-88,-93,-49,13,-64,-23,-53,-61,36,1,63,39, 
  /* [28][0][][] */ 121,-9,110,98,-45,-19,-61,-3,-19,-77,40,41,-40,-67,12,64,61,73,-45,81,-17,20,-76,-47,-83,52,-48,14,-2,-40,-4,27,-76,12,102,43,7,-77,1,37,67,29,-63,29,-75,-12,6,38,12,-17,-40,-21,-16,28,75,61,59,-12,76,32,-1,-16,-34,44,79,44,-8,127,-1,82,-40,19,41,60,44,-70,-51,-46,-55,26,-55,-74,106,-26,39,-78,42,-61,-27,-54,36,-19,-34,115,-76,9, 
  /* [29][0][][] */ 48,71,81,-43,-104,48,-115,12,102,-16,-91,-105,90,-105,-45,-46,-90,103,48,-79,-80,-27,66,6,-43,-6,-38,-8,-108,-22,109,-14,77,6,-89,-37,0,95,-16,95,42,103,-46,80,-4,98,-61,-30,32,-23,41,-108,93,-99,101,37,-75,55,-64,114,-127,103,-103,0,-72,-70,61,95,83,-84,58,-80,106,66,93,47,100,-46,-28,67,-9,-93,-21,-81,63,116,46,66,-42,91,86,66,-114,22,-16,64, 
  /* [30][0][][] */ 11,-81,-78,21,-92,14,-79,-27,96,-30,-20,-41,-94,9,33,8,-21,16,86,-115,32,-111,-122,-6,-119,3,-77,104,-15,-101,-97,-12,-83,-84,-120,-67,-2,-37,-16,11,63,-91,-27,72,76,-56,75,37,-82,70,16,-105,87,-61,-31,74,-75,-52,-87,-53,-100,-21,-84,-27,-108,-99,-104,76,-62,91,44,-38,-85,78,19,-93,-56,-82,-85,-68,13,22,-100,-26,-117,7,-106,91,-40,-127,64,49,-33,-15,2,-73, 
  /* [31][0][][] */ 63,-81,24,88,34,43,-57,-93,-26,-25,-74,-83,103,-112,-47,87,-61,-80,40,32,44,23,-33,81,46,-106,26,25,-71,-70,34,83,57,12,-86,-9,72,-35,-67,-99,42,-66,-36,17,-3,-55,-42,11,58,11,69,-35,-107,-56,-78,11,-59,48,18,88,-43,-13,63,-17,49,85,-44,-60,52,-67,-110,-1,-90,-44,-7,-1,-60,-47,35,127,-72,-45,39,53,40,74,-88,6,5,-52,-82,-56,0,80,19,-16, 
};
const TfArray<4, int> tensor_dimension5 = { 4, { 32,1,1,96 } };
const TfArray<32, float> quant5_scale = { 32, { 0.0019569867290556431, 0.0019293831428512931, 0.0019235806539654732, 0.0019717931281775236, 0.0020175660029053688, 0.0019597527571022511, 0.0019653358031064272, 0.0020294331479817629, 0.001814685994759202, 0.0020733603741973639, 0.0023607588373124599, 0.0021965433843433857, 0.002047109417617321, 0.0019865487702190876, 0.0020446982234716415, 0.0020754172001034021, 0.0018232897855341434, 0.0019897993188351393, 0.0026930561289191246, 0.0021678134799003601, 0.001974533312022686, 0.0021113876719027758, 0.0023016042541712523, 0.0020906673744320869, 0.0019735661335289478, 0.0018979969900101423, 0.001989063574001193, 0.0019203426782041788, 0.0026861859951168299, 0.0018394407816231251, 0.0019489545375108719, 0.0021715532056987286, } };
const TfArray<32, int> quant5_zero = { 32, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant5 = { (TfLiteFloatArray*)&quant5_scale, (TfLiteIntArray*)&quant5_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data6[96] = { -2268, 4036, 9880, 2735, -3130, -3066, 20645, 3186, 3141, -1556, 10139, -307, -12220, 2232, 13583, 6025, 13984, 6093, -3746, -4943, 4624, 25347, 18196, 15867, 1248, 1018, 5030, 3322, 2543, -4433, 5694, -2338, -350, 3253, 79, 1000, 9640, 3316, -2627, 5482, 5672, 6882, 5845, 15730, 3979, 4431, 1588, 7833, 3629, -4737, -6511, 374, -185, -980, 3082, -38, 189, -1015, -4537, 12664, -1013, 1189, -1147, -1802, -4299, 6237, 17953, 1781, -2228, 826, 1370, 2601, 1392, -1177, 336, 11689, -469, 17927, 16834, -1088, 9363, 1951, -889, 1252, 11238, 7970, 4212, 1084, 6797, 4065, -435, -8558, 650, -2569, -1828, -2309, };
const TfArray<1, int> tensor_dimension6 = { 1, { 96 } };
const TfArray<96, float> quant6_scale = { 96, { 0.00028981396462768316, 0.00037740875268355012, 0.00024313386529684067, 0.00032549002207815647, 0.00033566119964234531, 0.00047472174628637731, 0.00015973791596479714, 0.00021659159392584115, 0.00019214568601455539, 0.00039698756881989539, 0.00019440337200649083, 0.00027714460156857967, 0.00020854150352533907, 0.0012324545532464981, 0.00022737251129001379, 0.00019920199702028185, 0.00013430012040771544, 0.00023601124121341854, 0.00049976760055869818, 0.00033825548598542809, 0.00013108445273246616, 0.00010732080409070477, 0.0001670467754593119, 0.00010354437836213037, 0.00014265827485360205, 0.00046560546616092324, 0.00057602970628067851, 0.00032140375697053969, 0.00019398021686356515, 0.00043946842197328806, 0.00025797533453442156, 0.00033733388409018517, 0.00026068810257129371, 0.00035320108872838318, 0.00037865462945774198, 0.00024828157620504498, 0.00019785230688285083, 0.00031445693457499146, 0.0002231093094451353, 0.00031604926334694028, 0.00021812967315781862, 0.00037016725400462747, 0.00026975531363859773, 0.0001465822133468464, 0.00049307895824313164, 0.00022650761820841581, 0.0003262152022216469, 0.00027083806344307959, 0.00037616808549501002, 0.00027456079260446131, 0.00024950786610133946, 0.0001991800672840327, 0.00060684181516990066, 0.00035048412973992527, 0.00024967081844806671, 0.00044352043187245727, 0.00044409339898265898, 0.00028350562206469476, 0.00034415416303090751, 0.00014082822599448264, 0.00076038361294195056, 0.00051213125698268414, 0.00023745176440570503, 0.00047738954890519381, 0.00044717921991832554, 0.00030062219593673944, 0.00012133523705415428, 0.00019610596064012498, 0.00051289063412696123, 0.00032914444454945624, 0.00026996087399311364, 0.00018771100440062582, 0.00093223940348252654, 0.00034988473635166883, 0.00026792721473611891, 0.00018173144781030715, 0.00027017516549676657, 0.000104168699181173, 0.00010801365715451539, 0.00021110237867105752, 0.00015558463928755373, 0.00016299127310048789, 0.00014092876517679542, 0.00020542286802083254, 0.00016922902432270348, 0.00029750156681984663, 0.00055287015857174993, 0.000241232686676085, 0.0001014646768453531, 0.00050383707275614142, 0.00033132627140730619, 0.00023528383462689817, 0.00027564232004806399, 0.00040802935836836696, 0.00036463016294874251, 0.00019116549810860306, } };
const TfArray<96, int> quant6_zero = { 96, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant6 = { (TfLiteFloatArray*)&quant6_scale, (TfLiteIntArray*)&quant6_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data7[96*1*1*16] = { 
  /* [0][0][][] */ -43,127,16,-116,-21,-11,-31,-56,-23,13,26,98,22,-59,-37,42, 
  /* [1][0][][] */ -17,24,-64,4,-25,-28,-4,-66,76,85,-73,28,3,-5,41,-127, 
  /* [2][0][][] */ 2,50,-10,28,82,14,-15,-127,91,-78,-22,-29,-17,96,-114,-13, 
  /* [3][0][][] */ 17,-60,19,-89,-22,-7,-15,-8,127,17,94,60,-22,63,10,105, 
  /* [4][0][][] */ -68,105,41,47,36,-2,18,28,7,2,0,-43,-127,14,-7,17, 
  /* [5][0][][] */ 20,38,127,9,-11,57,-77,84,-41,41,-59,-18,45,-12,-114,16, 
  /* [6][0][][] */ 40,-65,100,-127,61,-11,-57,107,-78,-89,65,-38,-9,68,-14,94, 
  /* [7][0][][] */ 17,120,-7,-36,64,29,-6,-40,127,100,-2,30,35,-5,42,25, 
  /* [8][0][][] */ 23,-70,8,-106,46,-68,26,-83,16,55,83,-19,3,127,96,75, 
  /* [9][0][][] */ 13,-25,1,127,34,-1,33,-47,-57,109,36,-12,2,-50,-18,-11, 
  /* [10][0][][] */ -15,37,-24,-127,15,2,13,6,23,-54,12,-51,-9,-36,6,11, 
  /* [11][0][][] */ 10,58,29,-33,115,-32,105,-73,-101,34,20,19,-19,127,-63,-36, 
  /* [12][0][][] */ 114,-1,-127,-17,67,-90,-28,12,11,48,-120,-56,-18,109,-5,67, 
  /* [13][0][][] */ -9,16,-12,25,9,-127,-11,30,8,-21,15,-6,6,-2,-1,2, 
  /* [14][0][][] */ 10,-40,92,-9,29,23,-127,-83,3,-15,22,35,20,-39,-35,-46, 
  /* [15][0][][] */ -34,9,-36,-60,16,-19,127,25,-36,39,32,-45,5,47,64,11, 
  /* [16][0][][] */ 25,95,127,59,26,-17,120,18,-115,1,-60,-55,-6,-4,49,-73, 
  /* [17][0][][] */ -25,-82,-5,15,-53,-31,-43,6,-127,-45,10,-54,-7,32,68,-28, 
  /* [18][0][][] */ -3,10,3,55,41,-3,2,0,65,39,-36,-34,28,127,44,39, 
  /* [19][0][][] */ 14,19,25,-8,97,-6,-54,-34,-105,-65,-48,-127,-25,-47,18,-15, 
  /* [20][0][][] */ 93,19,0,-51,50,-127,25,6,-82,-5,-80,-47,-12,-20,19,1, 
  /* [21][0][][] */ 108,-4,89,6,22,24,1,66,15,-41,37,19,34,127,25,46, 
  /* [22][0][][] */ -32,93,-1,127,-18,2,105,83,-25,17,-8,31,2,-13,26,-11, 
  /* [23][0][][] */ 71,36,127,-105,105,-20,3,49,-2,-5,29,-34,-18,-25,-118,34, 
  /* [24][0][][] */ -127,25,0,-22,-10,97,12,-12,-2,19,108,-1,-76,10,67,-9, 
  /* [25][0][][] */ 21,37,13,11,19,0,74,-30,-16,-127,-2,-24,-20,-29,18,29, 
  /* [26][0][][] */ -50,-3,5,15,-65,2,-33,-42,-102,-28,21,-24,16,127,-45,46, 
  /* [27][0][][] */ 5,36,1,65,56,-36,-19,-26,54,9,-35,-7,14,127,-19,20, 
  /* [28][0][][] */ 22,49,75,87,127,10,12,51,-71,4,-72,-21,40,103,26,-60, 
  /* [29][0][][] */ -1,-9,-2,6,29,-12,14,32,-33,-25,-6,-127,-12,-25,-6,15, 
  /* [30][0][][] */ -22,68,-42,82,-57,-9,34,-127,-33,67,-28,9,67,43,69,74, 
  /* [31][0][][] */ -34,-35,3,-57,-31,16,-64,-127,41,103,-13,-121,-23,-29,-2,4, 
  /* [32][0][][] */ -127,-61,-12,50,63,28,-25,23,-15,-59,-87,39,58,-7,61,77, 
  /* [33][0][][] */ -5,-15,-42,-15,-61,-16,-71,17,43,2,24,-49,14,-114,127,38, 
  /* [34][0][][] */ 48,-4,53,15,59,9,-37,73,-89,-50,65,11,1,-16,-8,127, 
  /* [35][0][][] */ -57,50,-72,4,12,-25,42,-127,67,71,-55,-9,-10,-20,78,-120, 
  /* [36][0][][] */ -13,103,-2,75,48,-7,127,-25,-37,13,-25,-51,-32,2,-32,-69, 
  /* [37][0][][] */ 3,-5,-7,-66,11,-9,-45,-16,-86,-18,25,2,-13,-127,29,-12, 
  /* [38][0][][] */ 48,78,-24,-68,114,1,24,-50,127,85,-11,3,13,-28,68,14, 
  /* [39][0][][] */ -35,94,6,-127,-14,20,-30,32,50,-105,-38,-16,25,87,38,14, 
  /* [40][0][][] */ -1,-28,6,-71,13,-21,-7,7,-127,-24,12,-16,-23,-96,36,-38, 
  /* [41][0][][] */ -2,-20,-65,-14,20,-41,11,5,-44,3,-48,1,-127,4,0,39, 
  /* [42][0][][] */ 127,104,-13,-9,-79,-56,7,-66,0,14,59,-103,-46,59,96,-123, 
  /* [43][0][][] */ -8,123,-45,22,49,11,127,-54,76,112,-70,19,51,-95,-91,-78, 
  /* [44][0][][] */ -13,-29,7,-44,67,4,-82,-127,-32,-12,11,-5,6,-30,56,-57, 
  /* [45][0][][] */ -22,41,-127,-23,-15,-56,53,-93,25,44,-19,-54,44,-2,92,-37, 
  /* [46][0][][] */ 12,16,-3,-90,-1,15,35,-49,-127,-36,-3,-6,-5,-115,58,-43, 
  /* [47][0][][] */ 6,62,34,-20,127,-12,-17,8,-65,21,1,11,-1,0,99,-35, 
  /* [48][0][][] */ -127,-79,-35,17,47,16,-1,8,6,-13,-83,57,43,1,79,58, 
  /* [49][0][][] */ 14,9,91,70,127,6,28,5,-99,-32,-48,-76,26,80,15,-59, 
  /* [50][0][][] */ 38,-42,-99,5,31,-127,-44,-35,3,7,6,-26,48,-6,-21,-9, 
  /* [51][0][][] */ 24,53,56,10,55,-5,13,41,-127,51,-11,-125,15,-96,-22,82, 
  /* [52][0][][] */ 0,31,-7,30,22,-17,14,44,60,66,-21,-44,46,127,85,37, 
  /* [53][0][][] */ 15,-27,-16,-15,-63,-7,-78,3,40,20,20,-56,19,-127,107,6, 
  /* [54][0][][] */ 49,3,-23,27,108,96,-127,-54,-57,12,-49,-53,15,-25,-32,-58, 
  /* [55][0][][] */ 32,-27,7,12,18,-24,-71,-127,-20,-53,-37,20,0,-16,11,-16, 
  /* [56][0][][] */ -9,16,21,-28,8,-36,-110,76,-18,127,-30,-22,16,3,-8,16, 
  /* [57][0][][] */ 0,47,-19,-15,-34,-41,127,83,-25,16,-4,-10,27,41,124,-10, 
  /* [58][0][][] */ 55,-1,-127,33,81,116,-59,41,-105,-62,-13,27,-31,1,-22,-14, 
  /* [59][0][][] */ -93,69,93,13,-73,-70,102,127,9,-3,33,-23,5,5,75,10, 
  /* [60][0][][] */ -1,18,-31,0,35,-127,-8,29,-25,-30,-4,-35,20,-15,2,5, 
  /* [61][0][][] */ 23,-20,26,127,66,25,13,-9,-77,5,5,52,-3,12,26,-54, 
  /* [62][0][][] */ -45,127,11,-60,-7,-21,-57,53,-37,-73,-73,-67,-21,-102,-44,-22, 
  /* [63][0][][] */ -40,-3,-20,127,-84,17,-67,-38,-51,-20,-15,-26,27,62,25,29, 
  /* [64][0][][] */ 15,17,16,57,-9,25,-34,77,41,16,-28,-35,-3,-127,44,-17, 
  /* [65][0][][] */ -7,-22,32,-109,60,-21,70,-62,8,57,13,-127,-6,6,-1,-65, 
  /* [66][0][][] */ 41,-60,18,6,-27,30,-34,83,40,-24,40,58,16,127,11,52, 
  /* [67][0][][] */ -3,74,-4,-127,-27,14,-70,38,101,-96,-26,7,42,99,73,26, 
  /* [68][0][][] */ -21,44,127,10,15,7,1,-23,-38,-27,-27,-39,18,-12,74,-30, 
  /* [69][0][][] */ 116,111,7,13,-76,-12,-8,-64,27,21,127,-48,-63,75,53,-127, 
  /* [70][0][][] */ 35,25,-36,-6,-14,35,64,127,-35,72,-39,-6,12,57,-28,1, 
  /* [71][0][][] */ 9,-9,-12,17,-2,-78,-8,44,-25,22,-65,16,-127,5,27,44, 
  /* [72][0][][] */ 10,20,-127,10,28,11,16,79,-38,-23,21,0,22,3,-5,3, 
  /* [73][0][][] */ -34,52,60,3,66,-46,-69,-127,12,-84,25,-28,24,-36,31,-19, 
  /* [74][0][][] */ -3,34,31,127,6,13,-86,68,-44,14,-40,58,10,-89,25,-98, 
  /* [75][0][][] */ 14,35,-61,12,46,-14,11,13,127,19,-11,2,26,124,-59,10, 
  /* [76][0][][] */ -8,-32,-1,28,-10,-9,-22,76,28,-22,-24,-127,-4,4,0,17, 
  /* [77][0][][] */ 8,68,-23,121,-17,7,99,127,-8,37,-23,19,-41,-36,-14,-23, 
  /* [78][0][][] */ 2,35,9,-127,81,-50,-20,38,-3,-81,24,-55,0,93,31,62, 
  /* [79][0][][] */ 13,-66,-64,-56,-49,13,14,-92,127,28,43,56,13,32,40,-58, 
  /* [80][0][][] */ -39,56,116,-122,59,-41,53,-85,-95,8,-5,-127,-7,72,49,-69, 
  /* [81][0][][] */ -40,41,-115,20,75,-127,-27,64,-17,-60,55,-6,12,-4,8,7, 
  /* [82][0][][] */ 121,-3,4,27,5,-113,-19,30,-11,-12,-115,7,127,16,-15,15, 
  /* [83][0][][] */ -34,68,-92,-66,-31,-8,59,-127,66,49,-52,-127,24,54,43,78, 
  /* [84][0][][] */ 0,15,40,-46,-6,-37,52,-127,-41,29,-22,-72,17,23,62,-2, 
  /* [85][0][][] */ -67,127,22,-124,-50,-38,-12,-5,-64,-21,-57,4,-5,-18,-73,-18, 
  /* [86][0][][] */ -7,20,-17,5,-15,-18,127,6,-6,-36,-13,1,13,27,67,-26, 
  /* [87][0][][] */ -31,-52,6,-127,-7,-23,51,-24,4,25,-7,-106,-12,11,10,-48, 
  /* [88][0][][] */ 108,23,48,-35,66,-127,-6,26,-76,19,-95,-67,15,22,98,27, 
  /* [89][0][][] */ 29,-32,-52,-55,14,-16,-51,0,-21,28,46,-50,127,-8,43,-27, 
  /* [90][0][][] */ -13,-33,29,-49,51,9,-54,61,64,-73,-15,-24,-15,114,10,-127, 
  /* [91][0][][] */ -37,-13,-10,-31,24,0,-51,-127,27,93,-22,-123,-22,0,-7,-2, 
  /* [92][0][][] */ -23,17,47,23,-6,-127,17,-8,9,-18,-2,-11,9,-20,12,-8, 
  /* [93][0][][] */ 32,-37,127,2,-68,73,19,-9,-7,43,-62,-4,-24,9,39,-13, 
  /* [94][0][][] */ 17,9,-2,-10,16,4,-3,-14,-21,-11,8,-47,127,-6,22,-40, 
  /* [95][0][][] */ 127,78,6,45,-93,5,-73,-9,-10,0,98,-29,-33,34,64,-79, 
};
const TfArray<4, int> tensor_dimension7 = { 4, { 96,1,1,16 } };
const TfArray<96, float> quant7_scale = { 96, { 0.00070723035605624318, 0.0009209870477207005, 0.00059331732336431742, 0.00079429027391597629, 0.00081911089364439249, 0.0011584591120481491, 0.00038980695535428822, 0.00052854645764455199, 0.00046889134682714939, 0.00096876511815935373, 0.00047440073103643954, 0.00067631341516971588, 0.00050890189595520496, 0.0030075474642217159, 0.00055485503980889916, 0.00048611077363602817, 0.00032773133716545999, 0.00057593605015426874, 0.0012195782037451863, 0.00082544173346832395, 0.00031988415867090225, 0.00026189396157860756, 0.00040764271398074925, 0.00025267837918363512, 0.00034812767989933491, 0.001136212726123631, 0.001405679970048368, 0.00078431860310956836, 0.00047336812713183463, 0.0010724307503551245, 0.0006295348284766078, 0.00082319276407361031, 0.00063615472754463553, 0.00086191337322816253, 0.00092402735026553273, 0.00060587923508137465, 0.00048281715135090053, 0.00076736632036045194, 0.00054445158457383513, 0.00077125208918005228, 0.0005322998040355742, 0.00090331572573632002, 0.00065828138031065464, 0.00035770321846939623, 0.0012032559607177973, 0.00055274443002417684, 0.00079605990322306752, 0.00066092360066249967, 0.00091795949265360832, 0.00067000818671658635, 0.00060887174913659692, 0.00048605728079564869, 0.0014808705309405923, 0.00085528317140415311, 0.00060926936566829681, 0.0010823188349604607, 0.0010837169829756021, 0.00069183611776679754, 0.00083983619697391987, 0.00034366181353107095, 0.0018555570859462023, 0.0012497490970417857, 0.00057945132721215487, 0.001164969289675355, 0.0010912473080679774, 0.00073360558599233627, 0.00029609326156787574, 0.00047855556476861238, 0.0012516021961346269, 0.0008032081532292068, 0.0006587830139324069, 0.00045806943671777844, 0.0022749351337552071, 0.00085382047109305859, 0.00065382028697058558, 0.0004434775619301945, 0.00065930595155805349, 0.00025420190650038421, 0.00026358471950516105, 0.00051515118684619665, 0.00037967175012454391, 0.00039774610195308924, 0.00034390715882182121, 0.00050129153532907367, 0.00041296801646240056, 0.00072599033592268825, 0.0013491639401763678, 0.0005886778817512095, 0.00024760331143625081, 0.0012295088963583112, 0.00080853246618062258, 0.00057416094932705164, 0.00067264738027006388, 0.00099571025930345058, 0.00088980357395485044, 0.00046649939031340182, } };
const TfArray<96, int> quant7_zero = { 96, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant7 = { (TfLiteFloatArray*)&quant7_scale, (TfLiteIntArray*)&quant7_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data8[16] = { -6881, 24643, -16508, -10672, -18210, 6210, 29911, -55867, -3140, 17384, 28023, 8289, -14485, 20129, -10020, 7412, };
const TfArray<1, int> tensor_dimension8 = { 1, { 16 } };
const TfArray<16, float> quant8_scale = { 16, { 0.00049172178842127323, 0.0004096274496987462, 0.00015639602497685701, 0.00018042474403046072, 0.00036546090268529952, 0.00023565591254737228, 0.00026285205967724323, 0.00023975680232979357, 0.00025003703194670379, 0.00018388473836239427, 0.00049110036343336105, 0.00074530253186821938, 0.00039126101182773709, 0.00020089272584300488, 0.00043475869460962713, 0.00048353357124142349, } };
const TfArray<16, int> quant8_zero = { 16, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant8 = { (TfLiteFloatArray*)&quant8_scale, (TfLiteIntArray*)&quant8_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data9[16*1*1*96] = { 
  /* [0][0][][] */ -26,-8,-6,-42,16,1,-30,5,10,13,-8,3,3,0,-37,1,-7,15,-6,-12,0,-42,5,9,9,20,-5,127,42,-38,39,-8,-3,15,1,-7,-11,7,-47,5,-7,-1,1,57,-5,8,6,-5,24,8,-12,22,12,-30,-4,103,-5,2,-20,-45,7,22,2,-31,-34,-80,-21,49,16,1,15,-25,-5,-34,1,-34,-34,28,-12,-2,-19,-33,24,34,4,-16,-13,-110,-4,-9,5,3,42,-46,-42,-4, 
  /* [1][0][][] */ 3,-53,-104,-59,125,5,80,6,-25,-86,-33,-35,30,-12,-66,47,-53,-1,67,45,16,44,-1,-38,-2,3,-24,-92,5,-24,-73,11,-27,29,23,-14,8,-13,24,4,-56,26,-42,-16,-57,67,-6,3,15,-55,42,-17,-55,30,44,35,-64,-28,-56,-37,21,-11,-2,-49,-67,89,0,-37,9,-33,-21,34,26,-88,-52,-15,33,-55,46,-4,23,-43,47,11,-94,29,-36,8,41,-36,2,-12,8,-12,8,127, 
  /* [2][0][][] */ -16,49,-84,-11,51,106,7,96,19,-105,25,-12,24,2,60,18,21,8,-66,86,18,28,86,1,55,43,6,-51,39,-52,72,-91,9,12,-46,45,-127,-53,73,4,-5,-32,-9,-8,17,-24,-2,70,-57,35,-49,-10,120,14,-4,73,38,36,35,26,39,-4,60,7,32,-6,-39,-41,-59,13,-8,-49,-3,-89,32,76,59,42,33,-34,11,-54,12,6,-44,37,21,0,-42,21,-1,-49,12,-42,-27,-12, 
  /* [3][0][][] */ 76,-3,127,14,43,-25,-4,16,8,-51,3,-21,-27,28,75,97,-3,7,4,13,114,7,33,-5,63,-32,-6,45,60,66,20,116,-119,18,1,20,31,20,7,29,-54,-5,57,13,40,71,-72,-79,-48,-28,-80,24,27,-110,-2,15,-95,104,88,43,9,78,-22,-6,41,63,29,-58,17,-14,-2,-82,89,-47,79,-4,-5,-5,17,18,-63,-49,9,2,0,10,50,80,22,-1,43,51,56,119,67,-68, 
  /* [4][0][][] */ 32,12,4,-8,127,38,7,-13,-54,-27,-34,-32,-43,28,-21,38,37,-91,-35,66,-19,5,18,-13,22,7,4,-28,11,5,58,-19,76,9,-6,9,45,-3,10,14,-38,-16,-19,-39,4,-61,-28,-5,39,-28,-2,18,3,-35,40,-29,35,-27,-11,26,5,62,59,-2,34,52,-10,2,-44,-8,25,42,17,-69,-30,-14,18,44,60,-15,5,-21,-3,-29,-46,20,38,51,-33,41,-5,-26,5,15,13,-23, 
  /* [5][0][][] */ -6,-9,9,28,3,-11,20,0,2,-31,6,6,4,21,16,0,-2,14,-5,-22,12,26,-46,45,5,4,-1,-72,2,41,23,34,-22,0,-12,-71,42,44,-10,8,7,-4,35,-25,0,15,8,-30,3,-2,-3,-14,13,-9,-13,-57,4,33,-10,27,76,-2,-21,34,9,57,22,70,-9,71,-2,14,36,5,0,69,-44,8,-6,-9,-10,7,-11,36,10,8,-6,127,10,32,4,-2,2,12,-28,-3, 
  /* [6][0][][] */ 28,-70,3,-51,-31,40,35,-1,19,14,-9,-124,-11,5,-19,39,13,-31,22,14,-51,74,37,10,-31,-17,-13,-45,-49,-10,47,-26,67,-4,85,-28,-9,-5,30,-116,-61,121,-15,-32,6,-48,-38,47,-40,44,4,97,-44,-46,-33,7,5,-67,-33,9,-35,60,37,-76,43,-38,24,-40,6,22,127,14,-73,13,-47,12,-86,-32,34,-9,25,35,50,-19,-69,10,25,9,24,-11,71,-75,-18,-35,-65,14, 
  /* [7][0][][] */ 14,125,67,6,127,38,31,-59,-21,-54,14,-6,-50,10,33,43,6,-66,39,53,62,-61,44,-1,-3,-3,32,12,12,-39,24,4,61,-9,-28,35,-11,12,-9,71,-75,46,7,17,0,-12,34,-2,-56,-62,20,-16,11,-31,-48,33,78,12,-33,6,-34,-43,-22,5,-89,90,-69,-102,-31,-20,1,84,-59,-22,18,6,94,44,58,0,-40,-54,2,-42,-29,80,-29,-30,-39,17,-27,-54,-40,8,9,-18, 
  /* [8][0][][] */ 17,4,57,36,-127,-27,25,11,22,96,71,10,-13,20,16,-4,-35,35,-15,-22,-56,3,-118,44,20,-21,38,5,23,29,7,58,49,-26,-21,3,-17,-16,4,44,39,-35,55,14,-46,3,51,-18,-76,39,28,15,-36,-52,-15,30,-5,-48,-64,-18,42,-44,-8,-5,10,55,3,-30,31,-39,-54,-37,65,39,69,5,-36,-11,-38,9,-80,31,-33,25,47,17,86,66,4,-20,39,65,7,-7,61,-6, 
  /* [9][0][][] */ 0,-95,8,31,74,59,-61,-1,22,-41,58,106,-33,22,70,-116,-12,1,-30,-15,-96,-36,15,-33,-15,8,-33,53,3,19,8,9,-7,10,-127,57,16,23,-14,79,68,-70,3,59,-31,12,-61,-56,49,47,-11,-96,86,17,-10,-64,6,88,1,15,33,-82,-19,112,-77,-21,9,7,-20,-2,-67,-94,77,-14,47,11,30,-47,3,-18,-26,4,-15,44,23,-94,40,25,2,-12,-45,61,-70,-12,39,-71, 
  /* [10][0][][] */ -39,-29,-22,-42,-26,-1,75,-22,44,3,41,42,28,-27,0,-7,3,33,-31,-30,8,13,21,30,-27,-8,-2,-127,8,-44,50,-46,6,28,-12,8,2,26,-10,-19,16,-44,23,-52,7,-9,35,16,2,41,-34,-12,-53,-27,-3,-58,10,18,-6,-42,24,22,-3,20,-21,84,-17,-25,13,-40,1,-13,-30,0,-2,59,4,44,-13,34,1,-3,4,41,11,-13,-16,122,-4,-10,17,9,16,-44,53,-32, 
  /* [11][0][][] */ -54,10,18,1,8,-49,-25,15,-29,2,-16,14,10,28,-6,-7,27,3,-71,-46,13,0,-19,-13,69,-3,52,7,-1,19,-23,36,-10,21,-26,-2,-7,20,11,7,-3,7,1,-1,77,27,56,12,40,38,-21,17,27,-42,24,-17,-53,-3,-14,-36,2,-18,4,21,-5,-15,15,3,18,9,-6,21,3,0,37,23,-24,62,-116,40,-47,20,-66,-49,127,18,35,2,-11,43,-29,-36,-10,38,-11,-19, 
  /* [12][0][][] */ -5,-2,-29,8,43,-2,10,2,3,32,40,-15,-10,-18,28,-15,-8,7,-6,-29,-22,3,-43,49,-15,48,-30,90,-6,43,-21,11,-17,19,21,15,-3,-6,4,18,14,-5,13,-10,-1,10,32,8,12,-5,9,-39,-17,21,-19,-25,7,7,15,25,10,-35,-17,33,24,-59,9,63,-44,-127,-15,37,18,0,14,55,-13,4,-13,3,-8,28,-47,6,21,-15,16,-23,6,28,9,12,-29,30,10,6, 
  /* [13][0][][] */ -94,32,-28,-25,-66,10,11,-6,-36,-24,34,20,102,23,12,8,-26,26,-52,1,0,-20,66,-56,61,19,6,-31,-12,-38,-47,-127,-87,-55,17,-11,44,56,47,5,-39,95,2,40,44,8,-12,0,58,-1,33,99,41,-42,59,-12,-6,62,12,19,21,33,-24,36,33,-4,46,16,25,16,16,-126,35,-13,-112,-10,22,24,-50,-48,63,-21,21,-36,16,-85,104,-10,-31,-18,-6,6,50,-63,17,25, 
  /* [14][0][][] */ -25,49,-36,-72,51,1,11,3,-105,-3,15,2,1,-52,-88,-22,-94,-28,97,16,37,8,37,-88,-30,13,-54,-38,7,-25,-20,-26,-55,-40,41,-5,-46,-28,7,8,-20,70,-73,-3,-59,7,-5,1,127,-43,57,9,17,50,-111,36,-35,12,-115,1,-10,40,41,14,-3,22,-7,0,-37,17,70,37,11,-5,-24,10,68,-85,-19,-97,23,-47,26,23,14,-56,-61,3,-37,21,27,48,1,24,-10,16, 
  /* [15][0][][] */ -35,-68,15,-43,8,-5,-14,-48,-17,6,-1,-15,23,-20,43,-14,-63,-6,-17,12,1,-23,-57,-47,-13,7,-2,34,-19,4,54,-72,2,16,13,14,-10,15,-79,0,27,-53,-5,32,-1,-40,60,30,50,17,-4,-127,1,39,-6,19,33,-9,-16,25,3,10,34,3,-8,14,-8,-11,-23,24,9,34,-37,4,-3,-16,-15,1,-8,20,-1,38,-10,52,5,-34,-19,-22,-35,45,16,60,-15,20,-1,-23, 
};
const TfArray<4, int> tensor_dimension9 = { 4, { 16,1,1,96 } };
const TfArray<16, float> quant9_scale = { 16, { 0.020898174494504929, 0.017409166321158409, 0.0066468310542404652, 0.007668051403015852, 0.015532088465988636, 0.010015375912189484, 0.011171212419867516, 0.010189663618803024, 0.010626573115587234, 0.007815101183950901, 0.020871764048933983, 0.03167535737156868, 0.01662859320640564, 0.0085379406809806824, 0.018477244302630424, 0.020550176501274109, } };
const TfArray<16, int> quant9_zero = { 16, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant9 = { (TfLiteFloatArray*)&quant9_scale, (TfLiteIntArray*)&quant9_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data10[96] = { 16718, 64867, -1436, 921, 14032, -6308, 2110, -2686, 3843, 12916, -5318, 183, 462, 5970, 5314, -9069, -4481, -239, -900, 14403, -12250, 15212, -4207, 10545, 28346, -6, -5397, 16722, 12325, -18410, 2489, 21928, 23067, -11199, 1906, 825, -288, -1167, 4685, -8321, -5584, -1017, -7238, -617, 28918, 153, 13560, -1090, -14922, 2318, 76, 27, -2603, 18103, -4215, 12580, 13123, 1443, 32731, -263, -7708, -801, 1502, 425, -1084, 16449, -1108, -7844, -435, 2646, -69, -4012, 1860, 630, -1911, -6962, -490, -2033, 48131, -6804, -7258, -10393, -187, 122, 53410, -13527, -10054, -26297, -7276, 8953, -3518, 38034, -8, -1213, -4559, 49, };
const TfArray<1, int> tensor_dimension10 = { 1, { 96 } };
const TfArray<96, float> quant10_scale = { 96, { 0.00013838580343872309, 8.7166001321747899e-05, 0.00012886160402558744, 0.00024071121879387647, 0.00025682800333015621, 0.00013178073277231306, 7.6478951086755842e-05, 8.486877050017938e-05, 0.0002472015330567956, 0.00018465601897332817, 0.00020661874441429973, 0.00012188819528091699, 0.00028896128060296178, 0.00020327906531747431, 0.0003748958115465939, 0.00011538763646967709, 0.00019927715766243637, 0.00020593151566572487, 0.00025431267567910254, 8.0068079114425927e-05, 5.7238183217123151e-05, 4.8744132072897628e-05, 0.00014735998411197215, 7.7771699579898268e-05, 8.9400651631876826e-05, 0.00014869256119709462, 0.00015242834342643619, 0.00020099256653338671, 7.8534656495321542e-05, 0.00054158741841092706, 0.00037281293771229684, 0.00025228096637874842, 0.00011758836626540869, 0.00015114111010916531, 0.00031738154939375818, 0.00050301570445299149, 0.00046959874453023076, 0.00021006114548072219, 0.00022576939954888076, 9.9216915259603411e-05, 0.0002120041026500985, 0.00024790375027805567, 0.00037404781323857605, 7.1588503487873822e-05, 0.0003799163387157023, 8.9574510639067739e-05, 0.00011069271567976102, 0.0003039329603780061, 0.00063402316300198436, 8.9915338321588933e-05, 0.00051967223407700658, 0.00018131191609427333, 0.00023206297191791236, 0.00015274497854989022, 0.00020740216132253408, 0.0001324609766015783, 0.00020873425819445401, 0.00050595682114362717, 5.2417031838558614e-05, 0.00018735091725829989, 0.00034304405562579632, 0.00026479261578060687, 0.00010168508015340194, 0.00018929746875073761, 0.0001280708092963323, 0.00010531597217777744, 0.00027870218036696315, 0.00027144630439579487, 0.00024809420574456453, 0.00016301916912198067, 0.00018904184980783612, 0.00010584465780993924, 0.00029362892382778227, 0.00013121128722559661, 0.00039126150659285486, 0.00025905820075422525, 0.00014186296903062612, 8.183907630154863e-05, 5.3375468269223347e-05, 0.00019892858108505607, 0.00024145688803400844, 0.00024322154058609158, 0.00017784882220439613, 0.0002787806442938745, 6.7350578319747001e-05, 0.00010845326323760673, 8.4253319073468447e-05, 0.0004879726329818368, 0.00023696446442045271, 0.00013763501192443073, 0.00027887715259566903, 2.7183010388398543e-05, 0.00017197274428326637, 0.0001115528866648674, 0.00025351482327096164, 0.00035941647365689278, } };
const TfArray<96, int> quant10_zero = { 96, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant10 = { (TfLiteFloatArray*)&quant10_scale, (TfLiteIntArray*)&quant10_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data11[1*3*3*96] = { 
  /* [0][0][][] */ -46,-71,-9,127,21,73,72,-54,-13,-16,-14,-5,-30,-3,36,1,-11,-42,27,23,22,-127,127,3,-84,6,49,-30,-126,-127,38,25,10,31,61,-18,-31,-10,32,-4,-12,28,-105,75,26,-74,37,44,-19,50,-26,20,2,-6,15,45,-87,-52,-56,-18,-11,1,11,37,66,-26,72,11,-9,14,19,127,21,24,21,-1,-10,19,127,-8,-11,8,-24,19,121,20,-73,-18,3,-32,-17,-40,1,-24,-17,28, 127,-126,73,106,-43,127,34,-111,72,6,-16,127,127,-99,-13,18,12,127,127,-112,127,-72,82,69,-127,-121,-69,-36,-53,48,-59,-16,-87,57,-127,-82,-44,-14,-51,65,1,127,127,83,-50,-122,-114,-53,-17,-127,44,82,63,-31,47,84,-44,39,-79,-111,-46,-42,-59,68,71,-62,25,24,-81,30,-16,93,119,127,12,21,3,57,-34,54,67,127,127,-23,-7,75,-117,127,-126,-100,3,-127,-88,-127,-40,110, -75,-11,31,-84,14,-31,63,-4,84,-32,-28,16,-19,-4,-10,-16,-8,-17,5,-3,-25,-103,31,2,-127,97,7,-56,123,52,-15,12,0,15,16,-28,-10,34,19,10,27,10,-118,61,19,-23,38,12,-16,8,-18,6,-13,1,-6,21,-62,4,-29,52,1,40,29,-2,-9,-40,-30,20,8,2,-2,39,9,-25,-8,0,-11,-6,-85,2,-18,-28,-16,2,-101,14,-35,-24,-55,24,-28,-30,28,-27,-15,2, 
  /* [0][1][][] */ -64,-74,75,-37,-48,96,15,127,-50,-20,0,-123,26,-118,77,29,-4,-60,-16,90,58,-67,48,46,-97,-15,127,-24,-23,63,119,-105,-127,97,-81,-55,-35,33,-29,95,-48,-26,-4,53,-46,127,-24,67,15,94,19,-1,16,-56,26,33,127,117,-76,-84,127,-32,-34,127,71,-39,-15,-1,127,37,127,95,-43,105,-5,11,15,14,-31,7,-41,124,-76,110,1,32,-54,32,79,-127,-33,-6,127,-65,-25,1, 67,127,57,-116,127,80,86,40,-127,127,127,-41,-65,-127,-127,127,127,-65,-120,-48,110,92,-119,127,-116,-91,-45,127,-127,110,-127,-127,-88,127,107,127,127,-127,-106,127,-62,-100,108,-127,-127,-41,-127,-127,127,-111,85,42,127,127,127,-127,100,-127,-127,127,18,-82,-127,-101,-127,127,-127,127,-122,-127,-106,-77,-127,-77,127,127,-127,127,-81,127,127,29,-40,-127,55,127,100,45,127,11,127,3,-66,34,127,-127, -53,-14,-47,37,-57,-105,39,-22,22,-32,-32,89,2,91,12,7,-3,62,-5,127,6,-86,-38,9,-77,106,-73,-21,-68,-16,16,-63,-69,35,6,-16,-27,44,-68,44,127,-3,-39,32,-39,-65,15,45,8,116,-13,7,-29,-74,3,21,97,1,-3,21,-37,127,89,-55,-45,11,39,-12,-78,122,-17,-79,-2,-53,-11,-24,28,16,73,-35,-67,-43,54,15,-49,-17,77,22,11,62,-44,18,-44,94,-27,-4, 
  /* [0][2][][] */ -69,2,-3,-22,12,-47,65,28,59,-25,-38,-5,18,15,-28,1,-9,65,9,2,-52,-83,6,-12,-17,-56,35,-70,96,36,23,22,16,-12,-16,-44,-8,37,-4,0,-14,-4,-66,105,4,27,9,38,-14,-29,11,-19,18,-3,7,19,-65,20,-18,13,16,3,-19,-6,1,-48,-47,11,-35,10,28,32,12,-22,27,3,18,9,-77,-15,-5,-13,18,14,-127,4,12,-46,-62,-49,-29,-54,7,57,-20,-7, 119,35,-127,-37,-72,-27,38,32,-63,-26,-22,-111,-79,107,-14,43,1,-51,-6,-126,100,-50,-35,-2,-111,127,-50,20,-34,-25,-10,-8,-57,11,22,7,-24,-27,127,56,12,-13,65,-37,-37,75,-69,-42,-32,-6,-127,-127,-44,-27,0,42,-61,-9,19,40,-8,-16,5,-48,-52,32,59,-24,115,105,-21,-64,10,-59,5,4,101,6,50,22,16,-103,-44,3,3,9,127,38,-64,38,-20,-111,40,77,-11,28, -56,12,-18,25,15,-72,-127,5,-8,-7,-19,-1,14,-28,18,-4,-7,-19,8,18,-46,-70,18,2,26,-101,20,3,-45,-43,-7,12,47,-24,-9,-23,-35,3,41,-35,23,0,-49,-40,4,64,22,18,-8,-2,7,-8,16,5,-3,-2,-57,-14,5,-37,-13,12,80,-13,32,-3,1,25,0,5,-2,23,-19,12,2,11,37,6,75,-10,7,-12,-6,-16,66,2,44,-41,60,42,-17,-59,-7,2,-16,-14, 
};
const TfArray<4, int> tensor_dimension11 = { 4, { 1,3,3,96 } };
const TfArray<96, float> quant11_scale = { 96, { 0.0058813965879380703, 0.0037045548669993877, 0.0054766181856393814, 0.010230226442217827, 0.010915189981460571, 0.0056006810627877712, 0.0032503553666174412, 0.0036069226916879416, 0.010506064631044865, 0.007847880944609642, 0.0087812962010502815, 0.0051802480593323708, 0.012280854396522045, 0.0086393598467111588, 0.015933072194457054, 0.0049039744772017002, 0.0084692789241671562, 0.0087520889937877655, 0.010808288119733334, 0.0034028931986540556, 0.0024326227139681578, 0.0020716255530714989, 0.006262799259275198, 0.0033052971120923758, 0.0037995276506990194, 0.0063194339163601398, 0.0064782043918967247, 0.0085421837866306305, 0.0033377227373421192, 0.023017464205622673, 0.015844549983739853, 0.010721940547227859, 0.0049975053407251835, 0.0064234971068799496, 0.013488715514540672, 0.021378166973590851, 0.019957946613430977, 0.0089275985956192017, 0.0095951994881033897, 0.0042167189531028271, 0.0090101743116974831, 0.010535908862948418, 0.015897031873464584, 0.0030425114091485739, 0.016146443784236908, 0.0038069167640060186, 0.0047044404782354832, 0.012917150743305683, 0.026945983991026878, 0.0038214018568396568, 0.022086068987846375, 0.0077057559974491596, 0.0098626762628555298, 0.0064916615374386311, 0.0088145919144153595, 0.0056295911781489849, 0.0088712060824036598, 0.021503163501620293, 0.0022277238313108683, 0.007962414063513279, 0.014579371549189091, 0.011253685690462589, 0.0043216156773269176, 0.0080451425164937973, 0.0054430095478892326, 0.0044759288430213928, 0.011844842694699764, 0.01153646782040596, 0.010544002987444401, 0.0069283144548535347, 0.0080342786386609077, 0.0044983979314565659, 0.012479228898882866, 0.0055764797143638134, 0.016628613695502281, 0.011009973473846912, 0.0060291760601103306, 0.0034781605936586857, 0.0022684573195874691, 0.0084544643759727478, 0.010261917486786842, 0.010336915031075478, 0.0075585749000310898, 0.011848176829516888, 0.0028623996768146753, 0.004609263502061367, 0.0035807660315185785, 0.020738836377859116, 0.010070989839732647, 0.0058494880795478821, 0.011852279305458069, 0.0011552779469639063, 0.0073088416829705238, 0.0047409976832568645, 0.010774379596114159, 0.015275199897587299, } };
const TfArray<96, int> quant11_zero = { 96, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant11 = { (TfLiteFloatArray*)&quant11_scale, (TfLiteIntArray*)&quant11_zero, 3 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data12[96] = { 1996, 7576, 10689, 30169, 18638, -3237, -1553, 7103, 21832, -3953, 11535, 3594, 7569, 11387, 36320, 1570, 2229, 5454, 16241, 4855, -323, 604, 10805, -1313, 3629, -3, 10359, 8083, 5251, 31597, 31773, 16651, 8962, -508, 14112, 3375, 2146, 3903, 20105, -1006, 9036, 8634, 38432, -2710, 24897, 12279, 7001, 14598, 54340, 8584, -190, 7792, -3252, 6, -2466, 7594, -1103, 15426, 5176, 6832, 5779, 14244, 5072, 4376, 8571, 10518, 3360, -4705, 6169, -2449, 5435, 10902, 20077, 6178, -5203, -4125, 4893, 247, 2573, -1430, 18258, 16188, 3818, 12139, 2424, 227, 9441, 61325, 7571, 14916, 6435, -204, 6718, 6510, 7539, 11304, };
const TfArray<1, int> tensor_dimension12 = { 1, { 96 } };
const TfArray<96, float> quant12_scale = { 96, { 0.00019690532644744962, 0.00019894370052497834, 0.00017253376427106559, 7.7468721428886056e-05, 0.00015839058323763311, 0.00030099251307547092, 0.00055759371025487781, 0.00022561881633009762, 0.00010668446338968351, 0.00033989484654739499, 0.00023953089839778841, 0.00026236291159875691, 0.0001698799169389531, 0.00020633429812733084, 6.4849344198592007e-05, 0.00037972081918269396, 0.00022265047300606966, 0.00024704585666768253, 0.00013681234850082546, 0.00024960999144241214, 0.00031372866942547262, 0.0004742661549244076, 0.0001810373505577445, 0.00049340637633576989, 0.00072505447315052152, 0.00024246460816357285, 0.00021839450346305966, 0.00017610938812140375, 0.00030425001750700176, 0.00011186017218278721, 8.9183209638576955e-05, 0.00014692461991216987, 0.00018839395488612354, 0.00042864980059675872, 0.00014159730926621705, 0.00017296371515840292, 0.00025367067428305745, 0.00037087281816639006, 0.00013317179400473833, 0.00054723030189052224, 0.00020412873709574342, 0.00017914049385581166, 7.0687026891391724e-05, 0.00038659723941236734, 0.00012974721903447062, 0.00012925564078614116, 0.00025275215739384294, 0.00018736829224508256, 7.6109463407192379e-05, 0.00024501376901753247, 0.00025569475837983191, 0.00022285581508185714, 0.00030908896587789059, 0.00026696955319494009, 0.00041893345769494772, 0.00014710097457282245, 0.00017740152543410659, 0.00012283754767850041, 0.00040511792758479714, 0.00021399774414021522, 0.00027096699341200292, 9.6472169389016926e-05, 0.00028110048151575029, 0.00023160516866482794, 0.00029481289675459266, 0.00017939219833351672, 0.00037697597872465849, 0.00028627822757698596, 0.00027435625088401139, 0.00028564329841174185, 0.00021376110089477152, 0.00014519153046421707, 9.9847173260059208e-05, 0.00017735916480887681, 0.00029542666743509471, 0.00032669419306330383, 0.0003360715345479548, 0.00039825119893066585, 0.00030421590781770647, 0.00024627550737932324, 0.00010883698269026354, 0.00010649169416865334, 0.00021737089264206588, 0.00015917305427137762, 0.00048428212176077068, 0.00024600850883871317, 0.00024073445820249617, 6.2253297073766589e-05, 0.00018295687914360315, 0.00015530239033978432, 0.00020563598081935197, 0.00032482203096151352, 0.00021971539536025375, 0.00024233016301877797, 0.00015784113202244043, 0.00014625529001932591, } };
const TfArray<96, int> quant12_zero = { 96, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant12 = { (TfLiteFloatArray*)&quant12_scale, (TfLiteIntArray*)&quant12_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data13[96*1*1*16] = { 
  /* [0][0][][] */ 8,45,-28,53,34,48,0,-19,50,39,-36,65,30,-110,53,-127, 
  /* [1][0][][] */ 7,49,6,-9,34,-13,127,-125,9,96,-30,-50,11,-63,19,-2, 
  /* [2][0][][] */ -79,32,-48,-9,7,-20,28,47,-31,45,127,-18,-72,8,23,-33, 
  /* [3][0][][] */ -69,104,16,73,-79,79,111,127,-10,127,-70,29,62,-100,-48,-90, 
  /* [4][0][][] */ 45,-10,38,108,-11,107,-11,28,36,12,14,-1,18,-15,-127,-43, 
  /* [5][0][][] */ 6,-21,77,-35,11,-38,-21,32,-83,82,-12,-127,-11,34,-79,-13, 
  /* [6][0][][] */ 24,-17,16,-26,19,29,-27,-39,-15,-6,35,-11,127,9,7,-11, 
  /* [7][0][][] */ 35,54,127,17,-20,-52,44,13,-99,86,-112,-43,56,-26,57,-64, 
  /* [8][0][][] */ 38,24,65,68,-23,88,5,44,34,3,33,-69,-24,-19,-127,-50, 
  /* [9][0][][] */ 49,-108,63,39,39,-48,-2,-10,115,8,48,-40,18,12,-127,-12, 
  /* [10][0][][] */ 7,-29,37,-56,-94,-31,-44,46,82,65,81,-32,-51,108,127,31, 
  /* [11][0][][] */ 20,-18,-47,-97,17,-36,-72,-127,-19,52,34,-23,24,-9,46,-30, 
  /* [12][0][][] */ 19,-13,-12,-17,-34,-61,-49,-10,-41,-42,-17,-2,-1,127,62,33, 
  /* [13][0][][] */ -57,-38,-85,-8,-25,-42,20,-45,39,-17,-7,19,2,21,127,50, 
  /* [14][0][][] */ 28,-35,116,63,21,13,21,111,17,76,37,-59,49,74,-124,127, 
  /* [15][0][][] */ 23,-17,-5,48,6,-25,125,2,-15,-127,19,-58,24,-24,14,-25, 
  /* [16][0][][] */ 40,-73,-43,29,29,14,32,-61,127,57,91,-23,-8,98,27,110, 
  /* [17][0][][] */ -94,-30,18,127,27,75,46,-91,115,-50,14,-28,6,27,-40,36, 
  /* [18][0][][] */ -55,-26,14,-10,-127,-51,11,50,-90,-60,-30,-76,0,-112,46,-13, 
  /* [19][0][][] */ 67,-27,63,-43,127,47,-119,68,114,-34,28,61,-26,29,20,117, 
  /* [20][0][][] */ 19,-32,44,-71,-2,-11,-24,29,-32,-67,71,-28,13,-2,-39,127, 
  /* [21][0][][] */ -82,6,-58,-19,27,43,-103,-68,18,104,45,39,-127,-17,56,-9, 
  /* [22][0][][] */ 17,-9,32,-46,7,-85,-4,-33,-127,7,-19,-39,-31,-1,86,-12, 
  /* [23][0][][] */ 2,19,7,2,-9,21,0,17,4,7,50,-15,127,-2,14,-46, 
  /* [24][0][][] */ -6,3,-26,-45,8,8,28,-10,-38,2,32,-90,0,-127,54,-18, 
  /* [25][0][][] */ -36,23,-8,22,127,-10,-66,-99,-12,59,-22,-63,-15,-124,71,-79, 
  /* [26][0][][] */ 25,-9,46,-91,127,-63,2,-29,106,-39,61,-68,-20,77,-24,57, 
  /* [27][0][][] */ 114,-42,30,-27,-30,-66,6,-12,-32,28,-127,18,25,0,0,42, 
  /* [28][0][][] */ -127,-25,-114,-85,79,-49,83,-16,25,40,-101,-8,90,20,-69,40, 
  /* [29][0][][] */ -53,-56,-2,104,-28,-40,40,-13,-33,-28,-127,23,3,-72,40,-21, 
  /* [30][0][][] */ 54,-3,127,-20,20,33,-25,55,-34,-75,74,-45,-18,24,-35,57, 
  /* [31][0][][] */ -6,-77,-10,-59,11,-17,10,-46,-11,27,127,28,-21,65,64,102, 
  /* [32][0][][] */ -10,71,-26,23,-127,18,-48,-46,-70,-42,8,55,18,21,39,56, 
  /* [33][0][][] */ 22,-14,26,127,16,2,33,-35,-88,66,52,-27,12,14,0,86, 
  /* [34][0][][] */ 43,0,21,25,54,-3,-127,-17,8,113,-49,25,-14,17,34,27, 
  /* [35][0][][] */ 85,45,30,17,27,-109,-28,21,-43,8,-127,-35,57,34,-40,12, 
  /* [36][0][][] */ 16,10,-127,6,63,42,-40,0,-20,1,15,18,28,-10,-1,-30, 
  /* [37][0][][] */ -37,23,127,8,-100,33,64,-39,-24,23,-67,-14,10,-26,-20,20, 
  /* [38][0][][] */ -127,26,-20,55,33,105,33,-34,16,17,117,-29,-21,71,1,-44, 
  /* [39][0][][] */ -11,-9,18,29,39,-13,-127,29,5,13,1,-33,-12,-18,-16,-8, 
  /* [40][0][][] */ -58,56,-86,99,-13,52,49,26,-127,-65,21,52,-14,39,-84,-22, 
  /* [41][0][][] */ -20,-16,-36,-51,-78,1,127,57,-76,48,-44,-18,-7,34,45,-5, 
  /* [42][0][][] */ -25,-40,-14,-30,-34,-33,-70,77,17,-127,31,56,-7,25,-126,25, 
  /* [43][0][][] */ -81,10,26,-18,21,40,-68,-50,2,48,30,17,-127,-16,12,-16, 
  /* [44][0][][] */ 20,26,0,6,24,-7,-1,-50,127,7,10,48,37,81,-10,31, 
  /* [45][0][][] */ -111,-104,-36,-92,42,-59,12,95,-18,-76,-3,-4,59,51,-13,127, 
  /* [46][0][][] */ 5,6,-43,126,-93,-24,-5,-97,-127,77,-53,-22,-14,12,6,-49, 
  /* [47][0][][] */ -38,-24,-127,115,-23,29,16,-8,-5,68,42,46,-12,-35,67,-24, 
  /* [48][0][][] */ -23,-11,46,-127,56,8,-43,-28,-26,-3,63,-11,2,22,92,-6, 
  /* [49][0][][] */ -52,-5,33,69,24,-5,-26,55,-20,-85,-94,-54,-4,-67,-127,-89, 
  /* [50][0][][] */ 50,82,33,-127,78,14,-2,29,93,-33,-42,28,-15,-27,-13,-34, 
  /* [51][0][][] */ 0,11,55,-8,33,-12,-90,-127,1,-126,-2,-10,20,-22,-24,16, 
  /* [52][0][][] */ 0,35,2,47,46,-12,-65,127,-92,105,-33,-43,53,93,-10,-51, 
  /* [53][0][][] */ -14,127,23,-61,-36,29,37,-32,-50,-36,-50,57,29,-116,-104,-45, 
  /* [54][0][][] */ -8,8,5,37,58,-30,15,-51,3,50,-15,-30,-18,127,-36,-8, 
  /* [55][0][][] */ -127,90,73,-39,17,-19,-15,9,-62,-9,68,-27,-30,-29,82,6, 
  /* [56][0][][] */ 95,22,46,-126,100,21,-7,66,59,-31,39,127,-57,55,117,31, 
  /* [57][0][][] */ -42,-16,38,43,-49,19,32,127,-22,4,-5,-26,9,55,13,48, 
  /* [58][0][][] */ 13,127,15,-120,13,33,-11,3,82,-31,-8,80,11,-61,38,62, 
  /* [59][0][][] */ 34,33,70,-124,-12,47,10,-34,-6,9,-1,-52,-14,-127,-94,16, 
  /* [60][0][][] */ 16,-7,127,-18,-42,104,51,-8,-17,42,-46,-22,58,1,18,-36, 
  /* [61][0][][] */ 16,39,5,-52,-39,47,103,93,21,107,24,49,0,127,77,-57, 
  /* [62][0][][] */ 13,-24,-20,4,-82,29,-66,20,-44,9,18,-21,0,-102,127,-31, 
  /* [63][0][][] */ -9,-28,-4,-46,31,0,-68,-127,-22,81,1,20,55,8,14,-36, 
  /* [64][0][][] */ -8,-12,47,44,60,-17,-7,-127,28,-74,18,-40,10,-11,-5,-14, 
  /* [65][0][][] */ -127,50,49,-78,20,114,50,-80,-76,52,45,-5,1,13,-16,-48, 
  /* [66][0][][] */ 53,9,127,14,-101,45,16,20,-35,-6,89,-33,-16,80,-75,-17, 
  /* [67][0][][] */ 2,-65,-58,27,45,-46,-85,-127,18,-102,-25,-52,-52,-14,-9,33, 
  /* [68][0][][] */ 15,-18,-23,41,-42,-80,-20,-51,3,-19,17,-48,9,28,127,4, 
  /* [69][0][][] */ -15,75,-32,78,4,-66,19,65,57,23,44,-33,127,17,-26,-32, 
  /* [70][0][][] */ 42,16,-30,-21,0,14,127,106,10,61,4,28,54,71,44,-26, 
  /* [71][0][][] */ 18,-18,-75,-57,3,-43,15,97,-9,-40,12,20,31,-127,99,24, 
  /* [72][0][][] */ 52,28,38,15,27,5,-68,-127,48,40,-33,38,34,100,57,49, 
  /* [73][0][][] */ -18,-127,-54,-41,-109,-48,45,-23,-14,39,0,-12,-1,-17,80,17, 
  /* [74][0][][] */ -30,49,102,104,17,-11,-127,111,14,-32,-69,-65,10,9,-20,-26, 
  /* [75][0][][] */ -18,-4,-40,3,-41,95,127,99,14,61,-23,-12,-118,64,47,13, 
  /* [76][0][][] */ -8,-41,-45,-56,49,-83,-127,-74,-32,-45,3,12,38,16,35,5, 
  /* [77][0][][] */ -53,-8,-58,-35,-71,-22,-6,-37,127,5,-106,-23,-35,41,23,-93, 
  /* [78][0][][] */ -125,-127,28,27,15,42,5,37,-39,-117,-55,76,41,-12,78,68, 
  /* [79][0][][] */ 23,11,38,-52,127,32,86,54,95,35,-5,14,-5,-42,-14,-40, 
  /* [80][0][][] */ -20,0,70,-97,-24,-75,27,-6,-127,-92,-31,-81,22,120,-91,-27, 
  /* [81][0][][] */ 7,-33,70,61,-36,66,20,-8,24,9,-20,-75,34,82,-127,24, 
  /* [82][0][][] */ -40,77,111,-127,-11,-56,-25,-65,11,-77,-62,-122,35,-101,-67,20, 
  /* [83][0][][] */ -7,-28,-40,-49,-36,17,-3,45,-68,-1,0,-68,-2,-127,-12,36, 
  /* [84][0][][] */ 95,43,25,28,-49,-33,-26,12,-39,-30,127,6,-71,25,73,-29, 
  /* [85][0][][] */ -3,-38,-1,-59,19,-35,-28,112,-112,-98,-101,-62,-25,77,125,-127, 
  /* [86][0][][] */ -30,15,6,-12,-72,39,-34,48,-56,-46,3,-15,-38,-127,-30,29, 
  /* [87][0][][] */ 90,-22,-4,-20,-65,-65,27,19,-23,21,-127,-27,46,-6,10,45, 
  /* [88][0][][] */ 51,46,-43,-9,-27,-30,12,10,63,-8,26,-14,14,-48,127,-64, 
  /* [89][0][][] */ 32,62,-127,-48,-53,-19,54,-82,54,42,67,-28,-38,-21,12,-104, 
  /* [90][0][][] */ 11,-26,-11,-58,-37,69,127,-39,113,-11,31,-65,64,2,-31,2, 
  /* [91][0][][] */ 16,15,-45,-95,-57,-44,78,1,-91,11,33,34,1,1,-12,127, 
  /* [92][0][][] */ 1,6,-13,26,71,-15,-67,-127,25,-56,-23,-14,-50,-34,1,-50, 
  /* [93][0][][] */ 30,-27,38,-127,-25,6,-39,22,39,-84,37,-26,-32,83,-53,43, 
  /* [94][0][][] */ -86,89,-125,25,75,-127,16,-51,40,11,95,-72,5,15,-7,-46, 
  /* [95][0][][] */ -43,46,23,-127,-17,-9,3,28,-14,-55,-77,-96,-6,-1,-61,-16, 
};
const TfArray<4, int> tensor_dimension13 = { 4, { 96,1,1,16 } };
const TfArray<96, float> quant13_scale = { 96, { 0.00055315362988039851, 0.0005588799249380827, 0.00048468817840330303, 0.00021762796677649021, 0.0004449566185940057, 0.00084555917419493198, 0.0015664126258343458, 0.00063381664222106338, 0.00029970190371386707, 0.00095484504709020257, 0.00067289895378053188, 0.00073703948874026537, 0.00047723288298584521, 0.00057964184088632464, 0.00018217714386992157, 0.0010667255846783519, 0.00062547787092626095, 0.00069401029031723738, 0.00038433825829997659, 0.00070121354656293988, 0.00088133802637457848, 0.0013323258608579636, 0.00050857674796134233, 0.0013860951876267791, 0.0020368495024740696, 0.00068114046007394791, 0.000613521842751652, 0.00049473292892798781, 0.00085471029160544276, 0.00031424168264493346, 0.00025053674471564591, 0.00041274601244367659, 0.0005292432033456862, 0.0012041786685585976, 0.00039778032805770636, 0.00048589598736725748, 0.0007126209675334394, 0.0010418695164844394, 0.00037411108496598899, 0.0015372993657365441, 0.00057344592642039061, 0.00050324801122769713, 0.0001985765848075971, 0.0010860430775210261, 0.00036449063918553293, 0.00036310969153419137, 0.00071004062192514539, 0.00052636186592280865, 0.00021380948601290584, 0.00068830163218080997, 0.00071830704109743237, 0.00062605470884591341, 0.00086830405052751303, 0.00074998062336817384, 0.0011768832337111235, 0.00041324144694954157, 0.00049836287507787347, 0.00034507975215092301, 0.0011380721116438508, 0.00060117029352113605, 0.00076121039455756545, 0.0002710131520871073, 0.00078967772424221039, 0.00065063370857387781, 0.00082819920498877764, 0.00050395511789247394, 0.001059014699421823, 0.00080422323662787676, 0.00077073165448382497, 0.00080243957927450538, 0.00060050550382584333, 0.00040787737816572189, 0.0002804943360388279, 0.00049824384041130543, 0.00082992343232035637, 0.00091776129556819797, 0.00094410445308312774, 0.0011187818599864841, 0.00085461442358791828, 0.00069184612948447466, 0.00030574883567169309, 0.00029916036874055862, 0.00061064626788720489, 0.00044715477270074189, 0.0013604630948975682, 0.00069109612377360463, 0.00067628006218001246, 0.00017488423327449709, 0.00051396916387602687, 0.00043628114508464932, 0.00057768012629821897, 0.00091250194236636162, 0.00061723252292722464, 0.00068076275056228042, 0.00044341309694573283, 0.00041086573037318885, } };
const TfArray<96, int> quant13_zero = { 96, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant13 = { (TfLiteFloatArray*)&quant13_scale, (TfLiteIntArray*)&quant13_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data14[16] = { 414, 15865, -8941, 30736, 25506, 5306, -7066, -5971, 24042, 3872, 27845, 26738, -15664, 14059, 38760, -21858, };
const TfArray<1, int> tensor_dimension14 = { 1, { 16 } };
const TfArray<16, float> quant14_scale = { 16, { 0.00041907228296622634, 0.00028620060766115785, 0.00019284199515823275, 0.00013970771396998316, 0.00023791106650605798, 0.00036064311279915273, 0.0002530303318053484, 0.00038233902887441218, 0.00016530264110770077, 0.00023970335314515978, 0.00024853996001183987, 0.00049880478763952851, 0.0003372307401150465, 0.00047160335816442966, 0.00042123874300159514, 0.00027865974698215723, } };
const TfArray<16, int> quant14_zero = { 16, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant14 = { (TfLiteFloatArray*)&quant14_scale, (TfLiteIntArray*)&quant14_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data15[16*1*1*96] = { 
  /* [0][0][][] */ 0,14,-18,14,32,54,-41,-13,30,-4,7,2,-11,29,10,61,-11,-16,-15,-18,78,-61,7,-9,-16,-26,-47,-85,-8,3,54,32,81,-4,23,8,3,-31,-6,-1,-42,23,25,-23,-9,-31,3,13,-16,28,-6,-18,-35,-16,8,6,26,23,-11,-7,-65,23,-20,6,6,0,51,9,17,15,-5,-34,-63,2,-9,7,-14,-90,22,-4,49,-25,-53,-38,28,-7,-3,23,11,2,10,-32,26,13,-20,127, 
  /* [1][0][][] */ 69,-92,26,49,-37,-77,-30,-15,-38,-93,-4,-5,19,-67,-50,-50,-92,-20,11,19,-14,-114,-49,13,108,-45,-105,13,-67,127,-4,24,-40,36,-71,-12,-27,8,13,-89,-3,-105,81,-25,-74,3,83,-19,-1,13,7,-43,45,-74,27,-20,-16,37,-1,-95,48,-103,-11,-96,-15,-84,22,29,26,58,-39,-21,-78,37,-5,67,-9,77,11,2,48,-13,-87,47,-86,76,-3,46,-30,11,28,9,-25,-67,15,8, 
  /* [2][0][][] */ 12,-8,22,-51,7,9,-4,31,55,2,26,56,4,-10,49,-7,0,-25,29,25,0,-46,-12,-50,-15,-17,-37,21,4,27,91,-43,6,-72,3,56,19,127,-27,0,12,-1,-9,14,-33,-60,-59,26,42,30,-19,-15,45,3,-21,-48,-19,25,2,15,25,8,72,83,33,-77,-8,-76,-20,-37,3,13,-21,26,3,7,24,57,6,11,6,59,34,-23,-3,20,32,15,34,12,8,-43,-40,8,-19,40, 
  /* [3][0][][] */ -46,-54,-22,64,20,-9,20,-46,-26,39,-11,21,15,-25,54,-15,49,-58,41,14,3,-19,7,6,-18,24,21,-6,35,-67,57,-6,26,-30,-28,70,-11,-9,-2,38,22,-48,30,-62,10,-4,15,18,-127,58,-41,56,-106,-57,-33,-37,48,27,-30,72,-36,51,22,18,-3,-105,-71,20,77,14,12,46,-70,10,-2,-57,-26,-86,36,-62,2,-22,50,57,55,53,47,45,26,10,-31,6,-56,4,-13,-33, 
  /* [4][0][][] */ 24,-18,24,-70,-12,-37,-68,19,-24,-66,-43,-28,0,85,0,16,39,-1,-8,-20,-17,15,67,-24,-5,-4,19,8,6,-22,7,-13,-25,15,6,44,28,-38,13,-19,-7,9,-126,-19,31,9,11,-2,5,89,-31,-32,-32,-32,0,-4,-20,-50,33,49,-6,0,-21,-36,27,-127,42,-17,-50,3,-4,-5,-17,22,-38,9,-19,72,44,2,-3,17,124,-60,49,-26,-29,9,80,32,-4,16,-61,18,11,-28, 
  /* [5][0][][] */ -33,2,8,6,-18,-6,25,82,61,-3,-19,-33,-11,36,30,22,-22,-16,-6,-24,-41,6,15,-16,13,32,31,45,73,-6,35,47,-32,35,22,-19,-57,-45,0,15,-13,-1,-1,2,56,15,27,30,-4,18,-16,18,-15,10,25,-22,-3,-12,-5,36,-56,24,-6,-27,11,30,-18,-4,15,24,3,-17,9,-8,-6,-1,16,4,1,93,11,-42,23,15,-5,-20,22,-28,-25,-7,14,-127,-12,-31,-7,-103, 
  /* [6][0][][] */ -29,-9,-16,-80,5,-10,-15,23,-13,33,2,-2,57,-57,3,-26,13,-80,34,0,25,6,-31,6,75,-14,-20,-23,-7,12,8,-27,-40,-4,-41,-19,18,21,15,14,13,-50,-48,-35,-24,-26,38,-127,-7,32,19,28,-4,10,-5,-57,34,19,2,-11,13,-12,-1,-19,48,87,15,-16,-28,5,-2,-60,30,15,72,-10,-37,18,-1,14,1,-18,52,-31,-5,59,48,-36,15,8,-41,4,-11,-9,-5,46, 
  /* [7][0][][] */ 19,1,41,-15,4,13,-30,-34,5,-5,-12,-8,-1,-82,32,14,9,-24,1,-17,20,-6,-49,-29,-21,-18,-13,-12,-1,14,3,9,-13,-11,-22,-11,6,-7,-45,20,15,-21,-21,16,4,5,-32,63,11,9,-17,-18,-18,-2,-10,-51,31,7,-8,-6,4,2,-1,2,-13,26,7,-23,-7,-16,14,47,-1,-15,-4,-6,37,15,2,-28,-15,-28,49,3,-26,127,9,-6,-37,-21,7,26,-21,3,-15,1, 
  /* [8][0][][] */ -55,62,-70,16,29,-34,-4,39,6,69,-68,10,-11,69,61,0,-34,22,-117,10,-1,21,10,38,-20,-14,35,4,67,-127,2,-21,61,10,44,10,-40,20,-12,94,-9,58,-7,-113,-60,4,66,-12,-70,24,12,59,-127,75,48,24,46,-49,-13,-31,-31,0,-6,64,-37,28,46,-33,3,12,18,71,-41,29,20,57,32,-104,0,-35,20,-3,29,-22,-85,-25,-17,-24,16,25,-52,-7,61,-23,8,-5, 
  /* [9][0][][] */ 0,-47,-36,-21,5,10,1,20,45,-27,-19,6,-61,-11,0,45,-27,17,32,-27,-44,18,-1,5,44,33,38,-36,-19,-66,-39,8,-9,34,44,47,-4,32,-10,52,6,36,-22,-75,-7,-2,35,-118,-33,14,-12,12,-20,30,6,-10,-30,4,-21,-1,-14,42,18,69,19,127,51,32,28,8,-2,-26,-32,-17,-54,19,85,-8,7,-4,14,21,21,-45,-14,1,-18,-16,-67,10,-47,-20,30,-5,15,-21, 
  /* [10][0][][] */ 39,66,-10,42,49,-127,-25,-3,22,47,38,7,22,33,16,-92,20,60,-54,79,-67,-10,11,-21,-57,-9,-81,60,-28,5,-13,9,2,9,17,-30,91,-45,-27,1,3,59,49,-2,-59,-4,-21,19,-33,-51,-35,2,-26,-15,19,-7,-9,27,-17,-30,-35,40,-21,41,21,53,61,-6,37,-1,22,48,-54,-16,-53,-15,-5,48,-25,18,20,17,-62,-40,71,-49,116,-24,-38,24,42,-15,64,19,-6,-122, 
  /* [11][0][][] */ 24,34,-22,34,-14,20,83,6,1,-17,-42,-10,3,4,38,0,-18,-13,-102,-19,-12,10,15,8,-44,-17,-12,2,26,18,-8,6,6,3,2,34,1,-2,-4,43,5,35,66,-113,43,-36,2,6,-127,21,-8,-61,26,27,-12,19,7,-12,-6,9,-9,-1,-2,16,-51,62,-20,-14,33,10,34,25,-82,-31,1,30,21,-57,-61,0,7,4,118,40,-2,-18,-12,-2,0,23,-38,5,-23,31,-12,20, 
  /* [12][0][][] */ -38,8,14,6,-30,21,15,-6,127,-22,-27,2,-5,20,-44,-66,17,15,4,72,46,28,2,-12,5,7,34,-41,-108,-15,22,-54,13,-21,10,-11,9,1,63,6,-64,-10,13,23,-3,43,22,9,19,23,-34,2,15,9,29,-6,102,-35,2,6,20,-29,-7,-12,-9,43,-32,15,16,-2,1,-45,18,7,-88,-3,48,-28,11,-78,6,-8,45,43,-19,-27,-45,1,20,26,0,23,-1,-3,25,17, 
  /* [13][0][][] */ 24,-17,-15,15,22,5,11,17,6,1,17,1,20,-17,-2,0,9,-54,-41,19,-6,-18,6,2,0,22,-1,-13,-21,-27,4,7,-8,9,5,27,20,-3,7,19,11,10,14,39,13,11,8,-127,26,-11,-2,17,-12,2,11,-7,-19,27,29,1,-9,15,1,3,2,-23,-23,-63,4,7,-10,-43,-9,44,-9,10,-30,4,18,-5,27,-2,-22,21,15,32,20,17,-5,-2,20,-12,1,5,-8,0, 
  /* [14][0][][] */ -119,-52,13,44,24,-19,-33,36,-26,6,7,26,76,8,-124,-8,69,-20,-17,37,-21,-31,26,8,-116,-14,29,-18,-54,-49,-14,-17,-8,3,21,-59,12,22,-25,19,0,20,-22,-43,-9,116,62,47,-127,44,22,24,-71,-29,44,5,-4,5,62,62,34,48,5,60,-18,-32,-35,43,-29,110,-75,-96,-56,1,-6,65,-6,22,72,15,59,17,21,40,20,15,49,18,23,45,65,-9,-4,32,28,-11, 
  /* [15][0][][] */ 117,0,51,-6,-4,70,51,-36,-5,44,21,3,-92,-11,-10,39,18,52,48,-14,15,49,-93,50,15,-2,41,12,37,41,-61,-65,28,-56,38,11,56,-57,-61,-110,-4,28,-1,41,22,33,-38,-59,127,-54,-31,20,35,-126,38,-97,-6,-80,14,21,-110,-13,42,30,87,-25,-48,42,-35,37,-11,48,34,33,-4,-9,28,-27,61,-41,-17,14,56,65,1,6,0,-44,1,-24,73,20,-61,73,10,-36, 
};
const TfArray<4, int> tensor_dimension15 = { 4, { 16,1,1,96 } };
const TfArray<16, float> quant15_scale = { 16, { 0.017810571938753128, 0.012163525447249413, 0.0081957848742604256, 0.0059375776909291744, 0.010111220180988312, 0.015327331610023975, 0.010753788985311985, 0.016249408945441246, 0.007025361992418766, 0.010187392123043537, 0.010562947951257229, 0.021199202165007591, 0.014332305639982224, 0.020043142139911652, 0.017902646213769913, 0.011843038722872734, } };
const TfArray<16, int> quant15_zero = { 16, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant15 = { (TfLiteFloatArray*)&quant15_scale, (TfLiteIntArray*)&quant15_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data16[96] = { 10245, -8906, 2775, 62118, -926, -4585, -10587, 2649, -2291, 1713, 3233, -485, 1284, -5905, 5939, -4654, -2112, 18383, 25089, 3254, 1047, -605, -122, 224, 17310, -707, -755, -3664, -5324, -7656, 7797, -9155, -2550, -1457, -882, 14295, 2040, -4990, 19763, 1085, -21, 2257, -9126, -7844, -7900, 65585, -463, 15450, 35047, -12441, -255, 5392, 7415, -243, -7341, 3059, -10587, 1191, 3170, -7831, -439, -897, -13291, -1027, 1513, 15745, 30266, 15958, -12836, -216, -11975, 996, 40316, 10693, -338, -24, 18491, 34961, -669, -7232, -869, -4626, 885, -3792, 1279, -1409, 2287, -791, 2503, -2742, 585, -1019, -2169, -688, 70, 47839, };
const TfArray<1, int> tensor_dimension16 = { 1, { 96 } };
const TfArray<96, float> quant16_scale = { 96, { 0.00021492615633178502, 0.00012973225966561586, 0.00024810750619508326, 3.4679753298405558e-05, 0.00015075535338837653, 0.00019207276636734605, 0.00037070101825520396, 9.376486559631303e-05, 0.00013400941679719836, 0.00041112516191788018, 0.00016876120935194194, 0.00034748486359603703, 0.00013065723760519177, 0.00041704616160131991, 0.00016286733443848789, 0.00017977748939301819, 9.5262927061412483e-05, 0.00027429405599832535, 6.5091335272882134e-05, 0.00020132126519456506, 5.5758107919245958e-05, 0.00018678067135624588, 0.00012485726620070636, 0.0003251158632338047, 0.00020575677626766264, 0.00024453713558614254, 0.0001513459428679198, 0.00017113666399382055, 0.00011047339648939669, 7.3137751314789057e-05, 0.00011506186274345964, 0.00016066092939581722, 0.0001196496668853797, 0.00013060173660051078, 0.00017862052482087165, 0.00016553579189348966, 0.00033835452632047236, 0.00012515073467511684, 4.4347805669531226e-05, 0.00018234466551803052, 0.0002567631599958986, 0.0002894519129768014, 0.00036198657471686602, 0.0001635015505598858, 0.00016128561401274055, 5.421328023658134e-05, 0.00020044564735144377, 0.00022152680321596563, 7.4451141699682921e-05, 0.00019530839927028865, 0.00023098851670511067, 0.00021355866920202971, 0.00019125820836052299, 0.00028878982993774116, 0.00016467992099933326, 0.00040768433245830238, 0.00037505661020986736, 0.00030005184817127883, 0.00010213234054390341, 8.2720856880769134e-05, 0.00032840523635968566, 0.00014614609244745225, 0.0002888092421926558, 0.00012741486716549844, 0.00020391453290358186, 0.00018874634406529367, 7.0987553044687957e-05, 0.00010081059008371085, 0.00036904410808347166, 0.00024894808302633464, 0.00016564226825721562, 6.6851105657406151e-05, 0.00014466946595348418, 0.00014570850180462003, 0.0001605981815373525, 0.00023241847520694137, 0.00050465762615203857, 5.3235606173984706e-05, 0.00026053356123156846, 0.00025713403010740876, 0.00015206119860522449, 0.00044741705642081797, 0.0001220460471813567, 0.000164068813319318, 0.00014326149539556354, 0.00011080136755481362, 0.00012762323603965342, 0.00014908025332260877, 0.00019612404867075384, 0.00011046877625631168, 0.00020050168677698821, 0.00013654549547936767, 0.00012878527923021466, 0.0004620110266841948, 0.01868835836648941, 7.7990560384932905e-05, } };
const TfArray<96, int> quant16_zero = { 96, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant16 = { (TfLiteFloatArray*)&quant16_scale, (TfLiteIntArray*)&quant16_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data17[1*3*3*96] = { 
  /* [0][0][][] */ -19,3,-19,102,-18,22,-60,-55,19,-7,100,-10,7,-8,-4,57,6,14,-41,3,75,1,33,-37,2,16,48,49,17,17,21,49,-5,27,9,-14,3,48,-56,-29,21,9,26,13,-81,-57,25,-5,-9,-18,-9,-5,51,6,-10,1,-2,-34,24,-29,25,-5,127,26,-15,-12,13,-48,69,34,2,-3,78,-32,72,6,17,-127,30,64,-1,16,127,-11,35,30,-28,-20,34,-9,-14,83,3,19,-2,15, -50,10,-7,11,49,-9,-10,-127,36,13,-127,-6,-16,-12,3,91,20,-30,-21,-16,75,-96,127,24,-63,-29,-106,-16,24,41,-50,17,117,12,-12,0,-26,127,-72,-127,8,-19,-30,11,45,-127,127,-44,-73,51,49,8,39,-63,32,54,38,44,-105,88,127,33,5,-21,41,-40,-98,1,38,-30,-19,53,-47,-49,111,-5,-30,-62,-32,64,-71,60,-17,11,-10,49,56,-42,-28,55,-14,74,-127,-32,23,-127, 0,-7,-23,-73,5,16,37,21,-2,-4,2,61,2,-11,-11,7,-11,16,-17,16,7,21,25,-36,2,-29,9,13,-10,5,44,-80,46,18,12,-22,9,7,-34,-23,6,3,-18,7,23,-33,21,1,7,-7,5,-3,7,6,-6,7,-8,48,5,-4,8,-17,4,-14,-28,-17,25,-46,-106,-6,42,-14,-127,-19,-7,15,11,-9,8,13,-17,24,-37,-4,-41,19,29,37,-24,-17,22,-9,23,8,-1,-46, 
  /* [0][1][][] */ -43,55,-17,39,-87,-15,-9,-45,25,93,-45,-23,-29,-25,-6,61,127,-26,-92,-14,61,-127,-29,13,-36,-48,-127,-4,35,38,-51,29,-118,-75,-55,3,63,-2,-52,42,-18,86,2,-2,36,-48,-14,-32,127,42,-4,23,45,9,-39,-18,32,10,-127,127,-40,-89,23,127,-58,-37,-89,127,-34,127,-18,56,-32,-56,37,12,-35,-70,127,46,127,37,7,0,127,29,-99,46,-38,-37,-100,41,102,101,27,-58, 127,127,127,127,-48,-127,127,24,127,-127,16,-127,-110,127,127,127,-15,-127,-65,-127,127,20,-69,127,127,127,-54,-127,127,127,-127,127,-5,-127,-122,127,-127,-43,101,107,-127,-127,127,127,127,-115,-106,127,-13,127,-127,-127,-127,127,127,-127,127,-127,-46,42,-113,-26,-70,-66,127,127,-73,125,127,-104,-127,127,-41,127,-127,-127,-127,122,-72,127,-48,127,75,127,-38,-127,-29,-110,-127,-127,-22,-127,-20,-127,-127,120, -24,64,-39,2,127,71,-21,50,5,21,-13,-8,-26,-30,-13,-4,-104,-24,-127,35,34,72,-14,-122,-34,-40,127,74,8,36,71,6,127,-49,127,-3,54,7,-105,12,35,22,-23,2,-21,-51,-13,-42,81,12,-11,41,-4,-4,-76,-15,24,22,124,-92,-1,127,1,-34,-111,-29,-80,101,16,-15,112,28,-15,-59,15,48,-28,-5,-42,-2,-51,10,46,26,-89,32,127,-47,32,-25,127,-7,-114,35,32,20, 
  /* [0][2][][] */ -6,-40,-21,-62,-6,17,41,30,-4,10,-22,49,23,-35,-7,5,42,0,14,20,-7,26,-16,-29,1,-25,8,4,-10,-1,46,-75,-35,67,-28,-26,32,-31,-63,13,9,9,-34,18,14,-78,-13,-2,-7,6,19,-6,11,0,-23,6,-3,47,-52,37,-6,-13,6,17,-5,-11,19,-48,-54,11,-19,-8,-120,-34,1,19,15,-14,55,18,19,26,-40,14,12,27,-22,8,5,24,21,-3,24,40,-8,-26, -29,-11,-26,28,-45,72,-23,64,4,-6,53,-13,127,-8,-18,-21,0,-42,-60,23,-25,76,-44,-118,-48,-48,103,68,13,46,52,-3,-114,64,-15,-22,-27,-22,-127,-4,47,-12,-14,35,-26,-46,-17,-47,-60,27,71,27,-11,-81,29,51,17,27,72,-53,19,-57,-6,3,19,-27,-127,-53,-12,-25,-61,27,39,-46,-29,-8,-44,23,-55,2,48,38,14,9,8,64,-97,127,15,126,-31,-16,119,-22,44,86, -9,-36,-26,29,13,65,-42,11,2,-8,-1,12,13,-51,-10,-11,-43,-8,18,18,-21,5,-20,2,-4,19,-1,51,-3,1,33,27,15,42,19,-22,-5,-22,-31,6,12,-13,20,7,-72,-34,-13,4,0,-17,0,-2,-5,7,-18,-5,-4,-49,10,-30,-21,49,11,-27,-11,-5,37,-51,48,8,51,-18,31,-58,-16,23,10,12,-4,22,-31,34,84,-13,-16,53,39,-8,107,10,1,-17,10,-20,-8,-52, 
};
const TfArray<4, int> tensor_dimension17 = { 4, { 1,3,3,96 } };
const TfArray<96, float> quant17_scale = { 96, { 0.009134361520409584, 0.0055136210285127163, 0.010544568300247192, 0.0014738894533365965, 0.0064071021042764187, 0.0081630926579236984, 0.015754792839288712, 0.0039850068278610706, 0.0056953998282551765, 0.017472818493843079, 0.0071723512373864651, 0.014768106862902641, 0.0055529326200485229, 0.017724461853504181, 0.0069218617863953114, 0.0076405429281294346, 0.0040486743673682213, 0.01165749691426754, 0.0027663817163556814, 0.0085561536252498627, 0.0023697195574641228, 0.0079381782561540604, 0.0053064338862895966, 0.013817423954606056, 0.0087446626275777817, 0.010392827913165092, 0.0064322026446461678, 0.0072733080014586449, 0.0046951193362474442, 0.0031083545181900263, 0.0048901289701461792, 0.0068280892446637154, 0.0050851106643676758, 0.0055505735799670219, 0.0075913723558187485, 0.0070352712646126747, 0.014380067586898804, 0.005318906158208847, 0.0018847817555069923, 0.0077496478334069252, 0.010912434197962284, 0.012301705777645111, 0.015384429134428501, 0.0069488156586885452, 0.0068546384572982788, 0.002304064342752099, 0.0085189398378133774, 0.0094148889183998108, 0.0031641735695302486, 0.0083006070926785469, 0.0098170116543769836, 0.0090762432664632797, 0.0081284735351800919, 0.012273567728698254, 0.006998896598815918, 0.017326584085822105, 0.015939906239509583, 0.012752203270792961, 0.0043406244367361069, 0.0035156363155692816, 0.013957221992313862, 0.0062112091109156609, 0.012274391949176788, 0.0054151318036019802, 0.008666367270052433, 0.008021719753742218, 0.0030169710516929626, 0.0042844498530030251, 0.015684373676776886, 0.010580292902886868, 0.0070397965610027313, 0.0028411720413714647, 0.0061484519392251968, 0.0061926110647618771, 0.0068254228681325912, 0.0098777851089835167, 0.021447949111461639, 0.0022625131532549858, 0.011072675697505474, 0.010928195901215076, 0.006462601013481617, 0.019015224650502205, 0.0051869568414986134, 0.0069729243405163288, 0.0060886135324835777, 0.0047090579755604267, 0.0054239872843027115, 0.0063359104096889496, 0.0083352718502283096, 0.0046949228271842003, 0.0085213212296366692, 0.0058031836524605751, 0.0054733743891119957, 0.019635468721389771, 0.79425519704818726, 0.0033145986963063478, } };
const TfArray<96, int> quant17_zero = { 96, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant17 = { (TfLiteFloatArray*)&quant17_scale, (TfLiteIntArray*)&quant17_zero, 3 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data18[96] = { 18910, 1692, 7588, -527, 5476, 6063, 22395, 8267, -1248, 14690, 5423, 2857, 9203, 35802, -365, -1779, 8676, 16739, 1570, -129, -409, 16101, 6018, 4546, 7552, 7416, 4927, 4938, -54, -125, 7139, 13860, 11465, 2232, 9455, -868, 20819, 8606, 916, 7153, 824, 11034, 17065, 5361, 3180, 12221, 17819, 6070, 4868, 3149, 124, -904, 18859, 7163, 9655, 12886, -286, 8504, 2899, 4733, 10983, 5736, 3508, 9609, 5355, 5887, 8128, -895, 23494, 6989, 10203, -1269, -2089, 3915, 2215, 3440, 23141, 20563, 14268, -3178, 5452, -6508, -1668, 409, 6174, 6212, 5990, 6016, 12399, 5365, 6831, 4303, 11752, 17922, -12222, 11782, };
const TfArray<1, int> tensor_dimension18 = { 1, { 96 } };
const TfArray<96, float> quant18_scale = { 96, { 0.00013894375297240913, 0.0002130801003659144, 0.00017382590158376843, 0.0010621922556310892, 0.00029807162354700267, 0.00014955289952922612, 0.00013765128096565604, 0.00042879729880951345, 0.00040347620961256325, 0.00012291991151869297, 0.00026554294163361192, 0.00030193314887583256, 0.0003060000017285347, 0.00013452113489620388, 0.00021363412088248879, 0.00038976594805717468, 0.00025728644686751068, 0.00014296475274022669, 0.00028081855271011591, 0.00067823176505044103, 0.00051593105308711529, 0.00015764184354338795, 0.00025281411944888532, 0.00016802852042019367, 0.0002356812619837001, 0.00028207534342072904, 0.00029152882052585483, 0.0001884497469291091, 0.00066769192926585674, 0.00040601720684207976, 0.0001971590390894562, 0.00013775142724625766, 0.00016895121370907873, 0.0004768957442138344, 0.00018280405492987484, 0.00040651438757777214, 0.00010865875810850412, 0.00029167140019126236, 0.00052562548080459237, 0.00013747419870924205, 0.00051002891268581152, 0.00017095496878027916, 0.00012389798939693719, 0.00013992146705277264, 0.00034539011539891362, 0.00026648936909623444, 0.00015733363397885114, 0.00023505925491917878, 0.00031746688182465732, 0.00027987116482108831, 0.00050460087368264794, 0.00080605119001120329, 0.00014611957885790616, 0.00021929376816842705, 0.00024215398298110813, 0.00014861377712804824, 0.0003912973334081471, 0.00019993117894046009, 0.00027130972011946142, 0.00033745702239684761, 0.00014959758846089244, 0.00023567440803162754, 0.0003051751118618995, 0.0002798750065267086, 0.00020223800675012171, 0.00019339237769600004, 0.00023648279602639377, 0.00032690752414055169, 0.00011445662676123902, 0.00015681661898270249, 0.00019448067178018391, 0.00041826779488474131, 0.00019963335944339633, 0.00032775357249192894, 0.00020136852981522679, 0.00036899209953844547, 0.00010259500413667411, 0.00021720409858971834, 0.00011462890688562766, 0.00037429039366543293, 0.00020078550733160228, 0.00031487023807130754, 0.00020787942048627883, 0.00049432361265644431, 0.00026263960171490908, 0.00025349165662191808, 0.00026252979296259582, 0.00022747086768504232, 0.00013580221275333315, 0.00030336948111653328, 0.00019754657114390284, 0.00038518710061907768, 0.00017666124040260911, 0.00015238400374073535, 0.0002447109145577997, 0.00027355796191841364, } };
const TfArray<96, int> quant18_zero = { 96, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant18 = { (TfLiteFloatArray*)&quant18_scale, (TfLiteIntArray*)&quant18_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data19[96*1*1*16] = { 
  /* [0][0][][] */ 47,9,98,127,68,-26,15,-72,74,-6,-63,-73,35,25,-53,-88, 
  /* [1][0][][] */ 17,127,-16,-37,67,-26,56,59,36,98,-57,109,28,-22,108,-113, 
  /* [2][0][][] */ 24,-4,20,29,-36,59,-17,90,-42,-76,-28,71,9,-127,63,-124, 
  /* [3][0][][] */ 110,127,2,33,-106,-1,-38,-25,-19,9,107,-58,-55,79,-31,-34, 
  /* [4][0][][] */ 32,3,26,10,-35,51,-15,61,-72,-32,13,-1,10,-127,32,-31, 
  /* [5][0][][] */ 114,-5,-36,29,-5,-60,-21,4,31,42,-127,-18,-2,-28,-16,6, 
  /* [6][0][][] */ -53,-101,-73,83,-74,-47,-30,34,-122,-66,-125,-65,-17,42,-34,127, 
  /* [7][0][][] */ -20,15,-68,25,20,-127,-14,47,-9,-52,-10,-27,-13,7,-42,34, 
  /* [8][0][][] */ 8,-30,-1,1,-7,37,-37,-61,-4,18,-61,-2,-127,-2,-7,10, 
  /* [9][0][][] */ 12,-43,50,127,-23,49,9,-99,109,-84,20,-40,-10,47,-67,12, 
  /* [10][0][][] */ -127,-77,-3,-43,44,-3,29,47,72,-78,-29,37,65,-48,-74,83, 
  /* [11][0][][] */ 74,-12,-127,65,58,41,-54,28,-6,-33,-16,-13,-11,36,-61,-4, 
  /* [12][0][][] */ 100,-49,-51,-30,-86,-127,-16,-40,-20,-42,-43,-41,-115,-62,59,80, 
  /* [13][0][][] */ 68,-70,-4,-3,101,0,-101,-127,27,-38,-3,26,41,-16,-3,-29, 
  /* [14][0][][] */ -58,-2,-101,-53,58,-127,69,-46,37,41,-2,-105,-9,90,3,55, 
  /* [15][0][][] */ 39,-3,65,51,29,-34,-22,19,28,-52,-93,0,-127,-36,-22,34, 
  /* [16][0][][] */ -91,106,36,14,-23,81,53,116,-2,51,127,28,-11,23,-25,-32, 
  /* [17][0][][] */ -25,-51,-74,-69,-76,-4,34,67,-62,127,-21,-13,-15,86,111,22, 
  /* [18][0][][] */ 8,127,-51,-6,7,-28,56,-70,-4,71,-18,82,2,-37,58,-55, 
  /* [19][0][][] */ 5,-35,36,17,-25,13,5,18,-10,3,-36,-2,-127,17,-7,31, 
  /* [20][0][][] */ 84,-53,13,-46,-35,35,31,-52,-25,34,-127,-39,35,22,25,-13, 
  /* [21][0][][] */ 58,-37,127,24,-47,34,10,-11,-21,3,-56,-48,16,65,0,13, 
  /* [22][0][][] */ 36,70,64,10,10,32,21,127,28,8,22,39,43,79,26,21, 
  /* [23][0][][] */ -24,-11,-59,4,-9,-127,16,19,-26,-39,-11,1,-38,14,6,36, 
  /* [24][0][][] */ 62,-36,-14,-35,66,-1,-31,-37,50,53,60,84,-26,42,127,42, 
  /* [25][0][][] */ -127,-64,-25,-35,4,5,0,-37,-81,-10,-70,37,3,54,-1,85, 
  /* [26][0][][] */ 2,20,51,-2,43,35,-49,66,-77,-68,-22,13,25,-127,25,-41, 
  /* [27][0][][] */ -90,32,-55,48,58,-18,-25,9,45,-56,127,18,-20,9,-13,-53, 
  /* [28][0][][] */ -11,-7,4,41,-4,3,21,88,39,28,-34,-8,-127,-18,-6,39, 
  /* [29][0][][] */ -5,38,26,-45,15,-34,16,-27,-127,6,-21,-13,14,22,-53,21, 
  /* [30][0][][] */ -11,18,-127,-31,34,-57,8,11,-16,-14,-21,-32,-52,-42,33,3, 
  /* [31][0][][] */ 127,-16,-90,26,49,22,-18,54,-1,9,-89,-30,12,1,-96,19, 
  /* [32][0][][] */ -113,45,18,42,6,103,24,46,27,-19,127,-9,-29,34,-37,-37, 
  /* [33][0][][] */ 8,-24,127,-10,-72,22,9,-22,-10,3,-30,18,7,24,11,-1, 
  /* [34][0][][] */ -11,-84,-65,-127,-13,12,18,22,-7,-17,-34,-115,-1,27,72,41, 
  /* [35][0][][] */ 14,-57,21,70,46,-11,-15,-12,-85,47,-28,33,-16,127,-49,-34, 
  /* [36][0][][] */ -91,17,-48,-18,-36,18,66,113,5,38,127,-32,26,21,43,-2, 
  /* [37][0][][] */ -40,-3,127,-33,-93,-56,18,-51,24,25,3,19,-25,-69,58,37, 
  /* [38][0][][] */ 30,-22,-40,24,40,-58,-82,-10,-24,-30,73,-20,127,10,-6,14, 
  /* [39][0][][] */ -55,7,-39,-15,-74,-91,49,127,-57,-8,28,-61,-9,27,15,66, 
  /* [40][0][][] */ 74,-15,-26,-15,-3,68,-29,11,23,-9,15,9,127,10,-19,-2, 
  /* [41][0][][] */ -4,-18,-9,-127,2,-20,-6,-33,-24,-33,15,-34,8,-19,-6,37, 
  /* [42][0][][] */ 29,70,-29,82,-106,-20,-104,-63,-127,-94,96,-118,-84,92,-72,-1, 
  /* [43][0][][] */ 15,99,-31,127,49,20,30,-76,-32,107,6,44,24,-56,43,41, 
  /* [44][0][][] */ -17,-25,-127,-8,9,41,-37,38,-23,-38,35,-41,-2,-19,-33,28, 
  /* [45][0][][] */ -10,5,-15,-127,-38,-24,37,-10,-25,3,-30,-2,-9,-55,25,0, 
  /* [46][0][][] */ 35,23,82,-32,1,13,-68,74,29,-70,-66,35,12,127,51,-9, 
  /* [47][0][][] */ 14,-19,72,-46,-46,36,-77,78,-23,-29,22,48,-11,-127,58,-4, 
  /* [48][0][][] */ 9,-8,29,-127,10,9,-19,15,30,-11,2,-15,20,56,-6,47, 
  /* [49][0][][] */ 24,-32,13,119,84,16,16,60,-53,31,46,44,-28,52,127,-6, 
  /* [50][0][][] */ -17,-24,37,-9,-26,98,3,-41,32,0,127,33,100,-20,4,-63, 
  /* [51][0][][] */ 9,60,14,8,22,9,-56,7,28,-1,-4,127,9,18,24,-48, 
  /* [52][0][][] */ -51,25,97,-27,-5,-50,31,-27,-49,3,34,26,-36,97,-127,-16, 
  /* [53][0][][] */ 0,-30,-50,-127,-34,-18,0,-33,-43,14,14,3,7,-26,36,16, 
  /* [54][0][][] */ -19,51,-21,-76,52,37,-20,-1,67,-41,19,-36,-1,127,-9,3, 
  /* [55][0][][] */ 3,-30,1,-19,11,-18,-3,-127,-8,-60,-35,-8,-21,-56,-13,-16, 
  /* [56][0][][] */ 28,63,37,60,7,-14,24,127,42,-16,20,-46,94,0,-41,38, 
  /* [57][0][][] */ 76,-23,127,-3,-29,-3,-26,-3,40,16,-63,52,38,1,14,4, 
  /* [58][0][][] */ 102,70,24,108,-127,-10,-6,18,15,38,68,-32,-69,-62,-2,-55, 
  /* [59][0][][] */ 119,-26,-15,26,-22,-3,-15,16,3,26,-127,1,7,8,-12,30, 
  /* [60][0][][] */ -127,15,-1,-10,-4,22,36,21,-1,-4,38,0,-11,3,9,-6, 
  /* [61][0][][] */ -84,27,5,8,58,-13,-41,-14,127,-87,92,-26,35,-71,-59,-84, 
  /* [62][0][][] */ 21,-78,-98,66,93,127,-17,61,17,-30,40,15,-94,14,-97,-37, 
  /* [63][0][][] */ -57,19,127,-55,-60,-48,31,-44,-12,-5,7,21,-17,-41,52,1, 
  /* [64][0][][] */ 35,13,22,47,68,17,58,-127,3,76,-7,-13,6,-74,-19,52, 
  /* [65][0][][] */ -3,17,22,-57,65,-2,19,-127,-58,-94,-21,-79,-29,-27,67,17, 
  /* [66][0][][] */ -127,-20,-25,7,-51,20,-12,-24,-80,-57,-83,53,49,27,54,88, 
  /* [67][0][][] */ 9,-26,4,127,4,26,-20,26,71,30,-17,-31,10,-32,-60,-19, 
  /* [68][0][][] */ 0,-9,-8,127,-66,-29,23,-58,-71,32,24,-44,-42,-92,-47,-58, 
  /* [69][0][][] */ 94,62,-4,112,-17,-71,39,-127,1,41,-39,-11,-4,-54,60,45, 
  /* [70][0][][] */ -4,-40,-4,-75,104,-25,35,-15,74,-35,53,-24,32,88,71,127, 
  /* [71][0][][] */ -16,53,-9,57,23,-127,-21,107,21,-77,20,2,-3,-52,-41,53, 
  /* [72][0][][] */ -127,-88,7,79,35,56,-90,20,-37,-38,-126,71,94,41,2,93, 
  /* [73][0][][] */ 4,-30,-56,70,45,-32,-30,-33,85,4,-3,22,45,127,-72,-46, 
  /* [74][0][][] */ 48,-65,4,-20,-74,-6,69,-17,-53,-11,-91,2,-127,-41,20,41, 
  /* [75][0][][] */ -15,-35,-34,127,-45,-9,-15,-25,-61,13,-2,-46,-8,-43,10,-22, 
  /* [76][0][][] */ -13,-7,11,47,18,24,-5,-127,8,-108,42,42,-43,-31,5,-46, 
  /* [77][0][][] */ 127,-13,42,61,-40,6,-72,-27,62,10,-51,46,8,8,-48,30, 
  /* [78][0][][] */ -3,-3,70,6,16,-11,-69,37,57,-29,-53,-29,17,127,41,-22, 
  /* [79][0][][] */ -32,26,35,29,57,77,-50,-43,19,-56,91,-7,127,-29,-37,-52, 
  /* [80][0][][] */ -91,-71,-31,127,42,55,22,-67,-45,49,-101,11,69,-62,18,14, 
  /* [81][0][][] */ 75,41,-52,65,97,109,-8,-21,15,-49,-13,-35,-24,-12,-127,14, 
  /* [82][0][][] */ 127,89,-23,22,-103,-35,-94,2,-80,-24,126,-26,-34,79,10,-21, 
  /* [83][0][][] */ 127,37,-2,-19,-7,-18,-4,42,32,22,68,-34,-55,-21,-15,-69, 
  /* [84][0][][] */ 33,-40,-43,-9,46,1,-21,-127,-9,-56,18,-9,-19,24,31,4, 
  /* [85][0][][] */ -13,-30,-73,-84,-20,-36,55,87,-31,127,-9,-72,60,63,80,-1, 
  /* [86][0][][] */ 78,-26,4,2,-2,-18,-6,-20,3,1,-127,11,-5,13,-15,22, 
  /* [87][0][][] */ 2,26,-8,-44,-14,74,7,94,-45,-2,64,20,25,-127,55,-28, 
  /* [88][0][][] */ 28,-20,-62,-90,88,27,-31,-127,-39,-25,35,-9,-82,1,51,-19, 
  /* [89][0][][] */ -9,-34,47,-127,25,43,-41,92,81,-116,70,14,-55,23,53,52, 
  /* [90][0][][] */ -3,18,-58,32,23,-20,102,-69,81,127,-46,1,43,18,18,25, 
  /* [91][0][][] */ 77,-21,-37,-2,22,-127,-26,-10,-1,18,-91,-5,67,-14,-5,20, 
  /* [92][0][][] */ 0,-11,127,66,12,22,34,33,-33,27,-45,12,8,32,-12,-15, 
  /* [93][0][][] */ 127,-26,-51,-16,-51,-49,15,48,-10,48,-98,-17,13,-3,-8,61, 
  /* [94][0][][] */ 47,-9,69,84,127,3,9,77,14,3,-7,79,-19,91,8,-106, 
  /* [95][0][][] */ -94,38,39,68,44,37,27,36,38,31,127,23,12,52,-27,-56, 
};
const TfArray<4, int> tensor_dimension19 = { 4, { 96,1,1,16 } };
const TfArray<96, float> quant19_scale = { 96, { 0.00045950806816108525, 0.00070468825288116932, 0.00057486863806843758, 0.0035128311719745398, 0.0009857681579887867, 0.00049459416186437011, 0.00045523367589339614, 0.0014180977595970035, 0.0013343570753931999, 0.00040651482413522899, 0.00087819079635664821, 0.00099853880237787962, 0.0010119884973391891, 0.0004448818217497319, 0.00070652045542374253, 0.001289015170186758, 0.00085088534979149699, 0.0004728061321657151, 0.00092870963271707296, 0.0022430154494941235, 0.0017062623519450426, 0.00052134552970528603, 0.00083609472494572401, 0.0005556958494707942, 0.00077943369979038835, 0.00093286600895226002, 0.00096413004212081432, 0.00062323192832991481, 0.0022081586066633463, 0.0013427605153992772, 0.0006520348833873868, 0.00045556487748399377, 0.00055874732788652182, 0.0015771666076034307, 0.00060456077335402369, 0.0013444047654047608, 0.00035935101914219558, 0.00096460164058953524, 0.0017383233644068241, 0.00045464804861694574, 0.0016867431113496423, 0.00056537403725087643, 0.00040974948205985129, 0.00046274153282865882, 0.0011422575917094946, 0.00088132079690694809, 0.00052032619714736938, 0.00077737664105370641, 0.0010499111376702785, 0.00092557643074542284, 0.0016687916358932853, 0.0026657336857169867, 0.00048323962255381048, 0.00072523776907473803, 0.00080083997454494238, 0.00049148831749334931, 0.001294079702347517, 0.00066120276460424066, 0.00089726242003962398, 0.0011160216527059674, 0.0004947419511154294, 0.00077941105701029301, 0.0010092604206874967, 0.00092558917822316289, 0.00066883180988952518, 0.00063957797829061747, 0.00078208453487604856, 0.0010811327956616879, 0.00037852543755434453, 0.00051861634710803628, 0.0006431771325878799, 0.0013832751428708434, 0.00066021783277392387, 0.0010839308379217982, 0.00066595629323273897, 0.0012203129008412361, 0.00033929725759662688, 0.00071832694811746478, 0.00037909520324319601, 0.0012378351530060172, 0.00066402816446498036, 0.0010413236450403929, 0.00068748882040381432, 0.001634803251363337, 0.00086858903523534536, 0.00083833542885258794, 0.00086822587763890624, 0.0007522806990891695, 0.00044911855366080999, 0.001003288896754384, 0.00065331649966537952, 0.0012738722143694758, 0.00058424554299563169, 0.00050395703874528408, 0.00080929615069180727, 0.00090469769202172756, } };
const TfArray<96, int> quant19_zero = { 96, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant19 = { (TfLiteFloatArray*)&quant19_scale, (TfLiteIntArray*)&quant19_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data20[16] = { -42300, 7914, -914, -6376, -2765, -7856, -5746, -39520, -6811, 22051, 41875, -16446, -2884, -19815, 43633, -38948, };
const TfArray<1, int> tensor_dimension20 = { 1, { 16 } };
const TfArray<16, float> quant20_scale = { 16, { 0.00050710374489426613, 0.00069812597939744592, 0.0011581499129533768, 0.0010422932682558894, 0.00068385311169549823, 0.0011149501660838723, 0.0011062683770433068, 0.00052940682508051395, 0.00053911423310637474, 0.00049516797298565507, 0.00073321675881743431, 0.00053370610112324357, 0.0012414795346558094, 0.001232532667927444, 0.00056596961803734303, 0.00048915285151451826, } };
const TfArray<16, int> quant20_zero = { 16, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant20 = { (TfLiteFloatArray*)&quant20_scale, (TfLiteIntArray*)&quant20_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data21[16*1*1*48] = { 
  /* [0][0][][] */ -22,5,-28,-1,15,5,-2,104,-39,7,43,-2,7,-13,-16,9,6,-22,-16,-16,50,-1,7,110,-9,88,2,-6,18,-7,16,-30,-10,40,31,-50,28,39,24,-8,-109,-17,32,33,28,25,-127,-47, 
  /* [1][0][][] */ -35,-28,21,-1,-22,3,1,56,-53,-39,9,-54,-23,-16,-1,-8,-31,106,-10,8,14,5,-21,-23,29,-37,16,27,-21,-89,-41,17,-21,-10,32,-8,2,-46,5,4,-29,-37,127,6,11,4,20,-11, 
  /* [2][0][][] */ -26,-7,-11,-27,-14,-63,6,-17,9,15,19,27,18,10,-5,6,-4,-19,-24,-15,-9,-1,6,-127,-16,70,-22,3,2,-20,-5,-8,-6,54,9,5,29,5,28,0,-39,3,-24,-7,-8,-14,23,3, 
  /* [3][0][][] */ -3,39,4,-25,-1,-2,10,-9,-22,-11,1,45,-127,8,3,-4,2,1,-11,-2,-30,3,11,21,-4,-15,5,18,-10,18,1,27,9,6,23,24,-11,27,33,11,17,-2,28,-24,11,-1,6,18, 
  /* [4][0][][] */ 30,-27,0,9,27,32,18,-19,-20,-3,-4,-15,-6,21,7,17,14,-12,-83,-2,-8,1,5,62,1,-21,13,57,4,-63,13,10,23,-39,35,-127,2,31,4,14,-24,24,14,-26,-11,-16,8,12, 
  /* [5][0][][] */ 24,4,-19,66,-6,-7,14,22,6,-3,33,17,1,46,17,2,-24,20,30,1,8,-1,9,-26,0,-43,-4,-22,-10,-7,7,8,57,-127,1,36,-12,-17,10,11,-86,55,19,-7,-17,7,-98,-10, 
  /* [6][0][][] */ 31,-10,13,3,-7,-12,10,-17,2,-1,-4,-1,-11,-1,11,11,3,-10,127,5,6,11,22,-29,2,3,21,-17,-10,-8,-2,7,-4,-6,27,-34,-1,-40,62,14,0,-12,-11,8,-14,6,9,-14, 
  /* [7][0][][] */ -40,-19,-5,42,-6,77,0,41,-11,29,-17,127,90,-8,-2,0,-1,-17,-3,-9,-21,1,-10,80,-7,-48,-58,-37,-6,-22,-5,-7,-6,9,-35,8,7,-64,96,4,33,23,-78,-56,23,-15,9,-3, 
  /* [8][0][][] */ -59,-38,-11,7,-13,15,-3,38,12,-69,11,55,41,4,4,-2,-14,56,-35,-4,-49,10,-23,-22,8,7,9,49,-16,117,-17,38,8,-9,-12,-127,-107,62,40,47,12,10,126,-28,-4,39,13,37, 
  /* [9][0][][] */ -41,22,-8,30,5,-40,-11,18,56,-37,12,67,-76,7,6,-1,-8,23,-50,-40,37,-4,-19,-19,-9,30,59,-19,2,25,17,7,-16,19,76,-122,-19,-127,-92,16,-29,28,70,9,9,15,-33,-32, 
  /* [10][0][][] */ -14,-13,-28,-127,52,87,11,-19,-3,23,23,7,54,-50,-34,13,44,-46,49,-53,31,-44,42,61,-42,-123,12,-35,52,29,60,-27,-27,-81,32,-16,6,-42,-3,-52,-29,-71,6,15,16,29,9,-25, 
  /* [11][0][][] */ -14,66,-4,71,28,-49,-78,-20,10,66,0,17,-11,61,3,-83,21,44,3,-8,-58,-127,5,-11,13,-49,9,-19,50,-40,18,-83,62,-31,-41,-82,94,19,11,-106,-5,82,93,-70,-15,-53,40,27, 
  /* [12][0][][] */ 118,1,-19,9,-4,29,11,106,22,-7,-46,-19,-5,15,-14,3,-13,12,-11,7,1,2,-13,11,0,13,-8,8,-3,21,-10,-5,13,-70,-14,11,-25,-22,-25,7,-24,-70,6,-127,-59,-14,84,-8, 
  /* [13][0][][] */ 15,-4,-3,3,6,-14,1,-3,-3,9,5,17,-2,6,9,0,7,2,10,-5,8,-17,2,-1,0,1,15,127,15,-4,5,0,4,-1,-39,22,9,2,3,-22,-13,-4,-3,10,2,1,-10,-8, 
  /* [14][0][][] */ 59,-35,38,-79,46,21,28,-90,-50,-45,-3,105,-58,-7,50,5,46,33,-55,62,24,17,77,-23,27,57,-42,-100,28,-39,63,27,-18,58,-127,-54,-37,39,79,7,-40,-4,48,-2,-3,-2,12,-28, 
  /* [15][0][][] */ 6,-52,-80,-35,55,27,73,-63,-9,19,-29,-32,93,19,-110,52,57,71,73,-62,-88,-12,39,-10,-77,29,24,50,61,39,51,-57,20,59,72,127,-21,-97,-15,-22,98,72,103,-38,7,-39,-12,63, 
};
const TfArray<4, int> tensor_dimension21 = { 4, { 16,1,1,48 } };
const TfArray<16, float> quant21_scale = { 16, { 0.021551908925175667, 0.029670353978872299, 0.049221370369195938, 0.044297464191913605, 0.029063755646348, 0.047385383397340775, 0.047016404569149017, 0.022499788552522659, 0.02291235513985157, 0.021044638007879257, 0.031161712482571602, 0.022682508453726768, 0.052762880921363831, 0.052382636815309525, 0.024053707718849182, 0.020788995549082756, } };
const TfArray<16, int> quant21_zero = { 16, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant21 = { (TfLiteFloatArray*)&quant21_scale, (TfLiteIntArray*)&quant21_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data22[48] = { 8704, 33964, -955, 41263, 5169, 128617, -2712, 63641, 2598, 31295, 24027, -747, 67, 23911, 352, -1008, -380, 1835, 2160, -2633, 786, -2224, -2069, -18270, -3174, 55979, 128655, 99023, 3112, 2504, -1511, 3, 21574, 369, 28258, 2825, 50697, 8098, 59313, 1103, 675, 59579, -1422, 6392, 51047, -38, 5626, 49897, };
const TfArray<1, int> tensor_dimension22 = { 1, { 48 } };
const TfArray<48, float> quant22_scale = { 48, { 5.4538937547476962e-05, 3.7415898987092078e-05, 0.00021276995539665222, 5.3600873798131943e-05, 0.00013166753342375159, 6.28581692581065e-05, 0.0001580882235430181, 8.6848456703592092e-05, 7.8168923209886998e-05, 4.0322029235539958e-05, 5.4269854445010424e-05, 0.00010317658598069102, 4.5133860112400725e-05, 3.2235744583886117e-05, 0.00015551671094726771, 0.00018627545796334743, 0.00015200737107079476, 5.6638335081515834e-05, 0.00010944231325993314, 0.00014711385301779956, 9.603566286386922e-05, 0.00027207762468606234, 8.2356076745782048e-05, 0.00011858221841976047, 0.00012583757052198052, 7.4274808866903186e-05, 2.9215085305622779e-05, 7.1810172812547535e-05, 0.00011514562356751412, 0.00010668782488210127, 0.00011346254905220121, 0.0001585906429681927, 5.0015398301184177e-05, 0.00013508458505384624, 3.138326428597793e-05, 4.9850856157718226e-05, 0.00011190552322659642, 6.3347870309371501e-05, 9.5133451395668089e-05, 7.8963166743051261e-05, 0.00011243120388826355, 9.7443284175824374e-05, 0.00011412211460992694, 6.7550099629443139e-05, 2.7197413146495819e-05, 0.0001614579523447901, 8.7271109805442393e-05, 2.7713169401977211e-05, } };
const TfArray<48, int> quant22_zero = { 48, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant22 = { (TfLiteFloatArray*)&quant22_scale, (TfLiteIntArray*)&quant22_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data23[1*3*3*48] = { 
  /* [0][0][][] */ 70,-93,94,72,53,-33,105,-64,63,-86,-78,48,53,-41,-70,-89,101,76,51,76,65,-74,-80,51,-33,-43,-61,-64,-68,62,-66,-113,-13,42,67,65,-67,53,-50,-21,51,-74,61,62,-91,12,76,-38, 81,-97,127,89,-76,-67,-58,-83,100,-100,-63,76,82,-109,-93,89,-39,98,81,127,82,-116,54,71,-54,-79,-71,-83,54,80,-31,-127,-86,81,79,85,-86,81,-83,-15,79,-95,84,87,-72,-127,94,-100, 33,-43,70,38,16,-41,-45,-39,43,-30,5,34,31,-74,-56,-2,-55,37,35,71,35,-54,34,26,-33,-45,-45,-26,31,33,89,-92,-89,45,35,34,-31,38,-36,-11,43,-37,36,44,-9,-10,38,-72, 
  /* [0][1][][] */ 92,-110,-40,94,55,-79,127,-90,90,-116,-70,84,82,-106,-14,-127,127,107,73,-40,86,107,-127,74,73,-80,-96,-99,-127,86,-72,-7,-89,74,104,95,-85,80,-82,90,84,-98,89,88,-86,106,91,-91, 127,-127,-63,127,-127,-127,-90,-127,127,-127,-127,127,127,-127,-23,93,-69,127,127,-51,127,127,68,127,127,-127,-127,-127,51,127,-29,18,-127,127,127,127,-127,127,-127,127,127,-127,127,127,-127,-12,127,-127, 63,-76,-33,66,36,-60,-51,-63,56,-45,-58,58,49,-52,-20,8,-64,49,57,-29,58,65,52,65,64,-65,-59,-47,36,58,127,15,-61,72,48,49,-48,62,-59,55,64,-57,55,61,-62,-68,58,-56, 
  /* [0][2][][] */ 47,-50,-45,49,27,-52,65,-41,43,-72,-7,41,55,-74,82,-79,75,47,35,-33,28,-12,-76,32,-24,-44,-46,-46,-74,32,-40,93,-76,37,46,41,-28,37,-43,-85,49,-47,40,47,-20,15,37,-74, 75,-78,-46,84,-55,-63,-71,-66,48,-76,-68,63,73,-47,127,31,-48,59,63,-43,55,-17,40,65,-48,-68,-75,-58,31,61,-25,120,-61,66,56,58,-50,62,-68,-104,71,-65,62,62,-83,49,61,-53, 41,-39,-26,47,28,-21,-18,-35,14,-25,-49,31,32,7,64,23,-36,20,30,-29,31,0,39,41,-27,-27,-45,-24,34,36,60,67,1,35,14,21,-22,31,-28,-56,28,-30,27,30,-62,10,32,-4, 
};
const TfArray<4, int> tensor_dimension23 = { 4, { 1,3,3,48 } };
const TfArray<48, float> quant23_scale = { 48, { 0.0023179047275334597, 0.0015901756705716252, 0.0090427231043577194, 0.0022780371364206076, 0.005595870316028595, 0.0026714722625911236, 0.0067187491804361343, 0.0036910593044012785, 0.0033221791964024305, 0.0017136861570179462, 0.0023064687848091125, 0.0043850047513842583, 0.0019181889947503805, 0.0013700190465897322, 0.0066094598732888699, 0.0079167066141963005, 0.0064603132195770741, 0.002407129155471921, 0.0046512982808053493, 0.0062523386441171169, 0.0040815155953168869, 0.011563299223780632, 0.003500133054330945, 0.0050397440791130066, 0.00534809660166502, 0.0031566794496029615, 0.0012416411191225052, 0.0030519322026520967, 0.0048936889506876469, 0.0045342324301600456, 0.0048221582546830177, 0.0067401020787656307, 0.0021256543695926666, 0.0057410947047173977, 0.0013337887357920408, 0.0021186613012105227, 0.0047559845261275768, 0.002692284295335412, 0.0040431716479361057, 0.0033559345174580812, 0.0047783260233700275, 0.0041413395665585995, 0.0048501896671950817, 0.0028708791360259056, 0.0011558900587260723, 0.0068619628436863422, 0.0037090219557285309, 0.0011778096668422222, } };
const TfArray<48, int> quant23_zero = { 48, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant23 = { (TfLiteFloatArray*)&quant23_scale, (TfLiteIntArray*)&quant23_zero, 3 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data24[48] = { 455, 769, 2103, 533, 8648, 11898, 2478, -109, -462, -579, 718, 931, 2823, 829, 15555, 2877, 1410, -1114, -149, 2788, -1605, 10153, 9686, 6943, 4057, 185, 3486, 3677, 9614, -329, 7710, 6171, 442, -85, 5331, 4171, -2177, 255, -1000, 5317, 544, -492, -1006, 110, 901, 3866, 503, 854, };
const TfArray<1, int> tensor_dimension24 = { 1, { 48 } };
const TfArray<48, float> quant24_scale = { 48, { 0.00050250691128894687, 0.00088171951938420534, 0.00029011431615799665, 0.00037586616235785186, 0.00025082670617848635, 0.00034347825567238033, 0.00044601334957405925, 0.00029476956115104258, 0.0004613641940522939, 0.00070747884456068277, 0.0013630379689857364, 0.00028890889370813966, 0.00044297042768448591, 0.00084948010044172406, 0.00023218075511977077, 0.00044685436296276748, 0.00033834372879937291, 0.00033528803032822907, 0.00030880689155310392, 0.00032745639327913523, 0.00036641425685957074, 0.00025616431958042085, 0.00037940972833894193, 0.00022095587337389588, 0.00038910351577214897, 0.00039110865327529609, 0.0020099289249628782, 0.00050083920359611511, 0.00018331294995732605, 0.00048762853839434683, 0.00033730562427081168, 0.00019319429702591151, 0.00079631537664681673, 0.00030614272691309452, 0.00068417389411479235, 0.00043581519275903702, 0.00065728311892598867, 0.0004551297752186656, 0.00021943741012364626, 0.00034891124232672155, 0.00025959606864489615, 0.00031498010503128171, 0.0003249943838454783, 0.00033653268474154174, 0.0021778668742626905, 0.00036126215127296746, 0.00041882859659381211, 0.0013447121018543839, } };
const TfArray<48, int> quant24_zero = { 48, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant24 = { (TfLiteFloatArray*)&quant24_scale, (TfLiteIntArray*)&quant24_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data25[48*1*1*8] = { 
  /* [0][0][][] */ -127,-4,-7,71,-26,31,22,-21, 
  /* [1][0][][] */ 17,54,45,45,23,-20,106,127, 
  /* [2][0][][] */ -72,2,-1,9,-127,-8,5,-1, 
  /* [3][0][][] */ 109,-8,-16,-118,-127,-3,38,-29, 
  /* [4][0][][] */ -127,22,-34,2,36,-16,61,42, 
  /* [5][0][][] */ -83,-22,11,-23,-127,-5,-3,76, 
  /* [6][0][][] */ -14,-127,36,-19,15,6,2,35, 
  /* [7][0][][] */ -66,2,12,104,127,-48,-26,15, 
  /* [8][0][][] */ 64,17,6,-127,-10,-24,-38,-48, 
  /* [9][0][][] */ -8,-127,3,-7,5,83,-71,103, 
  /* [10][0][][] */ -75,5,-12,24,-127,43,61,-10, 
  /* [11][0][][] */ 108,14,41,17,-72,-52,-28,-127, 
  /* [12][0][][] */ 5,19,44,2,17,-69,127,43, 
  /* [13][0][][] */ 54,-22,15,12,126,121,127,-19, 
  /* [14][0][][] */ 127,8,-21,12,-42,7,-18,-6, 
  /* [15][0][][] */ -2,-127,14,-21,2,2,7,35, 
  /* [16][0][][] */ -78,-11,-15,-2,-127,6,31,0, 
  /* [17][0][][] */ -18,97,-112,18,29,-127,87,-66, 
  /* [18][0][][] */ 127,-17,-32,25,-59,17,15,122, 
  /* [19][0][][] */ 127,-41,-48,-18,94,-30,27,28, 
  /* [20][0][][] */ 127,47,2,114,17,35,-19,-56, 
  /* [21][0][][] */ 1,22,-56,-12,-20,90,-127,-68, 
  /* [22][0][][] */ -127,4,24,-32,-23,18,-79,-31, 
  /* [23][0][][] */ -44,-59,10,32,-127,72,70,-25, 
  /* [24][0][][] */ 127,6,5,-19,-67,-24,6,0, 
  /* [25][0][][] */ 18,-17,-1,-17,127,47,57,-27, 
  /* [26][0][][] */ 18,-97,13,-50,-14,-127,74,-61, 
  /* [27][0][][] */ -13,-127,32,-15,-6,-59,36,-5, 
  /* [28][0][][] */ -127,43,-24,20,24,-48,110,107, 
  /* [29][0][][] */ -6,40,7,7,4,38,-98,-127, 
  /* [30][0][][] */ 127,-51,-1,34,-69,34,-54,-29, 
  /* [31][0][][] */ -48,-51,85,-24,33,-123,127,110, 
  /* [32][0][][] */ -95,16,22,2,-127,-97,-123,73, 
  /* [33][0][][] */ -29,26,6,28,-127,-65,-51,20, 
  /* [34][0][][] */ 14,-127,41,-20,-8,93,-89,54, 
  /* [35][0][][] */ -30,-55,-127,-1,-6,-79,55,-50, 
  /* [36][0][][] */ 18,127,62,10,-10,-6,-29,-52, 
  /* [37][0][][] */ -127,18,26,-31,71,-26,-20,-118, 
  /* [38][0][][] */ -127,-15,-59,-25,40,31,24,91, 
  /* [39][0][][] */ 14,-48,38,-12,11,-88,127,120, 
  /* [40][0][][] */ -40,1,-3,-18,-111,98,127,-31, 
  /* [41][0][][] */ -127,32,14,-70,-79,-65,-94,36, 
  /* [42][0][][] */ -41,-120,-101,-15,-15,66,-127,-84, 
  /* [43][0][][] */ -71,16,-5,-127,73,61,49,1, 
  /* [44][0][][] */ 127,-11,36,-115,42,-9,-11,31, 
  /* [45][0][][] */ -44,-127,8,-24,-3,-66,95,28, 
  /* [46][0][][] */ 101,-3,1,-17,127,-28,-20,1, 
  /* [47][0][][] */ 127,-2,-7,105,-20,16,19,-31, 
};
const TfArray<4, int> tensor_dimension25 = { 4, { 48,1,1,8 } };
const TfArray<48, float> quant25_scale = { 48, { 0.00082714465679600835, 0.0014513423666357994, 0.00047753873514011502, 0.00061868940247222781, 0.00041286987834610045, 0.00056537770433351398, 0.0007341541931964457, 0.00048520142445340753, 0.00075942225521430373, 0.0011645358754321933, 0.0022436100989580154, 0.00047555455239489675, 0.00072914542397484183, 0.0013982751406729221, 0.00038217796827666461, 0.0007355385459959507, 0.00055692606838420033, 0.0005518962861970067, 0.00050830736290663481, 0.0005390051519498229, 0.00060313119320198894, 0.00042165580089204013, 0.00062452221754938364, 0.00036370140151120722, 0.00064047856722027063, 0.00064377905800938606, 0.003308416111394763, 0.00082439952529966831, 0.00030173978302627802, 0.00080265430733561516, 0.00055521732429042459, 0.00031800483702681959, 0.0013107641134411097, 0.00050392205594107509, 0.001126175164245069, 0.00071736768586561084, 0.0010819119634106755, 0.00074916018638759851, 0.00036120196455158293, 0.00057432061294093728, 0.00042730459244921803, 0.00051846873247995973, 0.00053495255997404456, 0.00055394502123817801, 0.0035848480183631182, 0.0005946506280452013, 0.00068940711207687855, 0.0022134450264275074, } };
const TfArray<48, int> quant25_zero = { 48, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant25 = { (TfLiteFloatArray*)&quant25_scale, (TfLiteIntArray*)&quant25_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data26[8] = { -40927, 28065, -14972, -9391, 20441, 682, 42883, 44957, };
const TfArray<1, int> tensor_dimension26 = { 1, { 8 } };
const TfArray<8, float> quant26_scale = { 8, { 0.00052050792146474123, 0.0005950223421677947, 0.0012845543678849936, 0.00155211566016078, 0.00093219563132151961, 0.00053322670282796025, 0.00061384256696328521, 0.00087108038133010268, } };
const TfArray<8, int> quant26_zero = { 8, { 0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant26 = { (TfLiteFloatArray*)&quant26_scale, (TfLiteIntArray*)&quant26_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data27[8*1*1*48] = { 
  /* [0][0][][] */ -49,-13,-28,10,14,-24,-6,-6,-108,127,-20,119,103,26,73,-38,-1,-24,1,10,-17,3,-17,-44,32,-92,1,43,-15,1,22,-28,-12,-75,97,33,-15,2,-10,-30,111,11,64,-12,-17,23,-6,-15, 
  /* [1][0][][] */ -10,34,127,10,4,-31,-3,-88,22,-4,25,8,-20,17,-5,-124,16,-63,11,6,-18,25,27,-37,-10,-8,19,8,10,-10,17,-12,-12,8,11,17,-22,-41,75,-47,13,44,-11,26,-15,2,16,8, 
  /* [2][0][][] */ 12,-55,11,-48,25,24,-9,-33,8,2,-11,18,-1,62,15,0,-41,127,-16,-29,-14,18,-46,1,116,-6,8,-3,-38,-6,-3,-41,-8,-10,-5,7,1,-8,-6,13,9,-12,-10,-8,4,19,-23,-41, 
  /* [3][0][][] */ 0,-76,3,19,4,-10,-12,-11,-12,-2,20,21,18,-6,43,-9,-22,-22,0,27,-7,14,-26,2,8,57,-4,8,-4,14,127,-2,6,0,3,-4,-23,-13,-7,6,-14,10,0,33,-20,9,-3,27, 
  /* [4][0][][] */ -92,28,10,0,-4,-30,3,10,38,-54,19,-17,-21,9,-19,-13,-4,16,3,30,26,-22,22,-4,-1,41,6,98,1,-4,-9,-25,-10,-127,-27,14,-5,-6,15,0,-36,-9,-7,18,-39,11,1,0, 
  /* [5][0][][] */ -20,19,11,1,-51,-6,-4,44,23,-9,8,1,-8,-44,-6,24,-17,-93,29,4,57,-42,6,49,-3,10,-64,-45,-37,-33,19,-3,-3,-57,-9,-72,62,9,-45,22,23,-25,-2,2,127,-20,-42,1, 
  /* [6][0][][] */ -24,-32,-12,-1,55,-16,-3,-47,-5,2,42,-27,-26,47,5,-78,-5,12,-11,-28,-47,53,-13,-91,3,-2,-60,-91,21,70,-4,11,21,-77,-7,63,-63,-57,7,-84,-14,12,15,56,127,-19,54,26, 
  /* [7][0][][] */ 6,44,-65,37,-30,9,15,53,-2,-14,64,-27,-2,-39,-14,4,-12,-127,39,-38,63,32,20,-61,9,-16,11,1,-17,31,-12,-2,5,30,-6,-29,1,-54,-43,-64,-6,-39,38,64,-31,-18,0,4, 
};
const TfArray<4, int> tensor_dimension27 = { 4, { 8,1,1,48 } };
const TfArray<8, float> quant27_scale = { 8, { 0.02212158590555191, 0.025288449600338936, 0.054593559354543686, 0.065964914858341217, 0.039618313312530518, 0.02266213484108448, 0.026088308542966843, 0.037020914256572723, } };
const TfArray<8, int> quant27_zero = { 8, { 0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant27 = { (TfLiteFloatArray*)&quant27_scale, (TfLiteIntArray*)&quant27_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data28[48] = { -53663, 185, 11452, 414, 6132, -7294, 2220, 30298, 1254, 24587, -2724, 20055, -6582, -1079, 24963, 11821, -12, 15428, 31410, 10185, -22, -332, -1687, 10698, -7868, 25600, 45111, 33789, 2845, 279, -538, 850, 1361, 17551, -7050, 15797, -11752, 20281, -22702, 119822, 22838, 21361, 9395, -1943, 17789, 4205, 1145, -417, };
const TfArray<1, int> tensor_dimension28 = { 1, { 48 } };
const TfArray<48, float> quant28_scale = { 48, { 0.00032421291689388454, 0.00028519914485514164, 0.00015446635370608419, 0.00037783515290357172, 0.00014719583850819618, 9.0983558038715273e-05, 0.00054636056302115321, 0.00011716724111465737, 0.0001500731596024707, 0.00019919507030863315, 9.6377698355354369e-05, 0.00021899334387853742, 0.00028949053375981748, 0.00019135420734528452, 0.00010103872773470357, 0.0001402605848852545, 0.00020827911794185638, 0.00025755367823876441, 7.4659197707660496e-05, 0.00015612065908499062, 0.00029928178992122412, 0.00021534212282858789, 0.00025374200777150691, 0.00019077271281275898, 0.00014229145017452538, 0.00014294389984570444, 0.00065292377257719636, 0.00010378942533861846, 0.0002251158730359748, 0.0025618509389460087, 0.00010199356620432809, 0.00023994958610273898, 0.00060463370755314827, 0.00016718888946343213, 0.00018346590513829142, 0.00015097302093636245, 0.00036488517071120441, 0.0001318206632276997, 0.00027342332759872079, 2.0483599655563012e-05, 0.00014243907935451716, 6.0343376389937475e-05, 0.00017215193656738847, 0.00016401331231463701, 0.00013490831770468503, 9.6797120932023972e-05, 0.00020741033949889243, 0.0001106963973143138, } };
const TfArray<48, int> quant28_zero = { 48, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant28 = { (TfLiteFloatArray*)&quant28_scale, (TfLiteIntArray*)&quant28_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data29[1*3*3*48] = { 
  /* [0][0][][] */ 5,-3,19,108,4,83,-2,27,12,7,9,7,-9,-10,-43,7,-1,9,-22,-113,-7,-23,127,-10,-2,6,-5,9,-4,6,14,98,19,28,37,-27,2,33,-1,2,18,-68,-7,-9,16,1,-7,-9, 127,10,14,-127,28,89,9,9,119,127,6,43,-49,-23,6,7,127,-55,-59,-54,-100,25,-107,15,16,70,-31,-85,25,22,7,-121,4,-61,127,11,-39,-48,-5,117,-80,-96,-29,3,-35,-10,-127,24, -30,-4,6,14,-2,67,3,17,27,9,-15,31,-15,-2,16,4,-101,29,-4,80,-32,-12,-17,-6,-5,-61,-3,27,-15,5,4,-20,8,-23,17,-18,0,45,41,-10,49,35,-15,12,-41,1,0,-8, 
  /* [0][1][][] */ 126,7,0,-97,14,-26,34,-17,6,8,-12,52,-7,-2,18,-33,1,-49,-58,-85,9,-73,-122,-18,5,100,-16,-127,-8,34,11,-106,18,-53,-2,127,-3,59,-40,41,-63,-127,39,-88,-45,-16,-12,-4, 27,-127,-127,111,-127,-127,-127,-127,-127,-97,127,-127,127,127,35,-127,-81,127,127,-105,127,127,112,127,127,-127,-127,29,-127,-127,92,127,-127,127,-30,-40,127,-127,127,127,127,-77,127,127,127,-124,103,108, 56,4,48,-14,5,-60,41,17,-25,14,-105,-8,-1,-40,127,5,63,-65,-74,86,13,-46,12,-33,0,7,-39,67,22,41,0,4,34,-44,10,67,-5,63,66,-8,-82,7,-85,-32,28,-18,-9,4, 
  /* [0][2][][] */ -52,-7,26,-16,7,36,-38,21,6,-25,12,32,8,-7,-38,17,5,22,-12,108,7,-32,-15,-12,-4,23,1,51,-10,5,48,-31,-17,-45,-33,-38,-1,30,10,-24,9,-3,42,-36,-37,16,-12,19, 69,125,75,21,16,-19,119,34,17,-4,43,26,-3,-1,-88,25,-56,-56,-38,125,-36,50,13,21,30,-62,-39,31,78,2,127,22,86,-27,7,-34,-25,-18,-11,117,-27,-12,-40,45,20,-34,9,-127, -40,-5,7,-3,4,-4,-62,7,-20,-12,-41,-41,21,-9,-7,11,33,24,-10,-127,24,-10,6,-8,-8,10,-10,-54,-14,9,5,9,-19,87,-37,-21,-4,34,25,-59,5,7,-37,6,-43,127,-3,-4, 
};
const TfArray<4, int> tensor_dimension29 = { 4, { 1,3,3,48 } };
const TfArray<48, float> quant29_scale = { 48, { 0.013779048807919025, 0.012120964005589485, 0.006564820185303688, 0.016057994216680527, 0.0062558231875300407, 0.0038668012712150812, 0.023220323026180267, 0.0049796076491475105, 0.0063781091012060642, 0.0084657901898026466, 0.0040960521437227726, 0.0093072168529033661, 0.012303347699344158, 0.0081325536593794823, 0.0042941458523273468, 0.0059610744938254356, 0.0088518625125288963, 0.010946030728518963, 0.0031730157788842916, 0.0066351280547678471, 0.012719475664198399, 0.0091520398855209351, 0.010784035548567772, 0.0081078400835394859, 0.0060473866760730743, 0.0060751154087483883, 0.027749259024858475, 0.0044110505841672421, 0.0095674246549606323, 0.10887866467237473, 0.004334726370871067, 0.010197857394814491, 0.025696933269500732, 0.0071055279113352299, 0.0077973008155822754, 0.0064163533970713615, 0.015507618896663189, 0.0056023779325187206, 0.011620490811765194, 0.00087055296171456575, 0.0060536605305969715, 0.0025645934510976076, 0.0073164571076631546, 0.0069705657660961151, 0.0057336031459271908, 0.0041138776578009129, 0.0088149392977356911, 0.0047045969404280186, } };
const TfArray<48, int> quant29_zero = { 48, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant29 = { (TfLiteFloatArray*)&quant29_scale, (TfLiteIntArray*)&quant29_zero, 3 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data30[48] = { 49124, 7282, 6291, 7877, 1680, 11080, -1592, -199, 1306, 2244, 3368, 393, 2429, -946, 917, -2089, 5181, 13256, -244, 25687, 4386, 2882, 8921, -455, 3504, 4150, 22865, 10643, -116, -2928, 69, 2169, -2100, 7263, 3995, 6863, 7968, 4285, 21679, 2214, -1032, 2711, 1361, 4117, 4048, 2658, -1385, 2803, };
const TfArray<1, int> tensor_dimension30 = { 1, { 48 } };
const TfArray<48, float> quant30_scale = { 48, { 8.7694890680722892e-05, 0.00030565555789507926, 0.00024842447601258755, 0.00040894575067795813, 0.0013229029718786478, 0.00045661741751246154, 0.0012140612816438079, 0.0011167838238179684, 0.00048058544052764773, 0.00059102225350216031, 0.00069183437153697014, 0.00050995498895645142, 0.00042231215047650039, 0.00054104963783174753, 0.00050314387772232294, 0.0017480339156463742, 0.00031147425761446357, 0.00032317719887942076, 0.0019441337790340185, 0.0001056025066645816, 0.00028015041607432067, 0.00035460563958622515, 0.00044212528155185282, 0.00098983256611973047, 0.00034354810486547649, 0.00027638519532047212, 0.00017567603208590299, 0.0003454658726695925, 0.00069745565997436643, 0.00089726515579968691, 0.00052312668412923813, 0.00037127838004380465, 0.0009438467095606029, 0.00029624483431689441, 0.00027567116194404662, 0.00027342172688804567, 0.00045892110210843384, 0.00031180391670204699, 0.00010801374446600676, 0.0020438362844288349, 0.00064271205337718129, 0.0010206581791862845, 0.00036088743945583701, 0.00037039211019873619, 0.00029408434056676924, 0.00043441867455840111, 0.00085080834105610847, 0.00093065697001293302, } };
const TfArray<48, int> quant30_zero = { 48, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant30 = { (TfLiteFloatArray*)&quant30_scale, (TfLiteIntArray*)&quant30_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data31[48*1*1*8] = { 
  /* [0][0][][] */ 50,26,20,-20,127,44,-67,-25, 
  /* [1][0][][] */ -12,-3,6,127,24,-3,7,1, 
  /* [2][0][][] */ 33,-127,-18,40,6,-65,63,9, 
  /* [3][0][][] */ 127,-31,0,5,-22,-28,34,-4, 
  /* [4][0][][] */ 2,-53,127,-26,-8,-53,11,-83, 
  /* [5][0][][] */ 62,-26,-8,0,127,36,18,6, 
  /* [6][0][][] */ -23,68,-11,127,8,35,-42,-33, 
  /* [7][0][][] */ 0,127,-10,0,4,-11,3,-10, 
  /* [8][0][][] */ 127,70,18,-2,-50,-9,-21,-48, 
  /* [9][0][][] */ -127,22,-11,31,46,3,2,4, 
  /* [10][0][][] */ -71,4,12,127,44,11,-20,-7, 
  /* [11][0][][] */ -127,22,12,-65,19,-7,-15,-34, 
  /* [12][0][][] */ 127,52,22,17,-28,-12,-10,-66, 
  /* [13][0][][] */ 9,74,-127,1,12,105,-50,54, 
  /* [14][0][][] */ -62,-30,-30,127,39,-3,-17,-37, 
  /* [15][0][][] */ -1,127,5,2,-4,-21,7,-52, 
  /* [16][0][][] */ 13,-104,-30,-19,-10,-88,127,-18, 
  /* [17][0][][] */ 27,127,7,-6,7,87,-86,-14, 
  /* [18][0][][] */ 14,61,-127,12,4,68,-40,47, 
  /* [19][0][][] */ -80,-12,-5,-13,-127,18,7,42, 
  /* [20][0][][] */ -127,-28,9,4,40,-26,40,18, 
  /* [21][0][][] */ -62,18,-7,-3,-127,-28,11,11, 
  /* [22][0][][] */ -127,-10,-2,-11,83,-6,38,31, 
  /* [23][0][][] */ -15,-24,-8,-4,0,69,-127,-79, 
  /* [24][0][][] */ 2,38,-127,9,14,52,-19,64, 
  /* [25][0][][] */ 45,-38,7,127,-72,17,14,-39, 
  /* [26][0][][] */ -1,-20,7,-2,-5,127,121,-27, 
  /* [27][0][][] */ 73,-6,6,14,127,51,28,-39, 
  /* [28][0][][] */ 11,127,-59,14,18,81,-40,120, 
  /* [29][0][][] */ 4,10,6,10,-1,95,-127,-79, 
  /* [30][0][][] */ 8,3,-1,-127,-7,-1,-9,-12, 
  /* [31][0][][] */ -93,13,-18,16,-127,19,-4,-18, 
  /* [32][0][][] */ -29,27,-14,-127,-17,30,-58,-96, 
  /* [33][0][][] */ -48,-14,3,-4,-127,36,40,0, 
  /* [34][0][][] */ -127,101,-17,45,52,-48,23,9, 
  /* [35][0][][] */ -38,127,10,-3,40,-67,72,-19, 
  /* [36][0][][] */ 10,61,11,-19,12,88,-127,-36, 
  /* [37][0][][] */ 45,-54,-10,15,28,-127,104,-34, 
  /* [38][0][][] */ 22,-127,19,21,21,50,-36,64, 
  /* [39][0][][] */ 9,-127,-17,-9,1,0,-55,-116, 
  /* [40][0][][] */ 127,28,-13,-23,-41,84,-79,-94, 
  /* [41][0][][] */ 5,-127,21,-7,3,84,-46,119, 
  /* [42][0][][] */ 58,-3,-8,-127,-50,15,-22,-69, 
  /* [43][0][][] */ -85,-33,5,-127,59,-24,-17,-6, 
  /* [44][0][][] */ -7,-20,-12,11,-46,107,127,-18, 
  /* [45][0][][] */ -127,-75,1,16,-5,29,15,-31, 
  /* [46][0][][] */ -7,75,127,7,7,19,11,43, 
  /* [47][0][][] */ -16,13,11,127,12,-15,-6,6, 
};
const TfArray<4, int> tensor_dimension31 = { 4, { 48,1,1,8 } };
const TfArray<48, float> quant31_scale = { 48, { 0.00022072248975746334, 0.00076931575313210487, 0.00062526873080059886, 0.0010292906081303954, 0.0033296630717813969, 0.0011492772027850151, 0.0030557154677808285, 0.002810874255374074, 0.0012096031568944454, 0.0014875656925141811, 0.00174130336381495, 0.0012835244415327907, 0.001062933006323874, 0.0013617877848446369, 0.0012663813540711999, 0.0043996907770633698, 0.00078396103344857693, 0.0008134165545925498, 0.0048932619392871857, 0.00026579483528621495, 0.00070512091042473912, 0.00089252000907436013, 0.0011128013720735908, 0.0024913458619266748, 0.00086468883091583848, 0.00069564406294375658, 0.00044216547394171357, 0.00086951575940474868, 0.0017554517835378647, 0.0022583596874028444, 0.0013166768476366997, 0.00093448418192565441, 0.0023756024893373251, 0.00074562954250723124, 0.00069384690141305327, 0.00068818521685898304, 0.0011550753843039274, 0.00078479072544723749, 0.00027186376973986626, 0.0051442068070173264, 0.0016176656354218721, 0.0025689322501420975, 0.00090833083959296346, 0.00093225354794412851, 0.00074019166640937328, 0.0010934043675661087, 0.0021414307411760092, 0.0023424047976732254, } };
const TfArray<48, int> quant31_zero = { 48, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant31 = { (TfLiteFloatArray*)&quant31_scale, (TfLiteIntArray*)&quant31_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data32[8] = { -2973, -24944, 22486, 24162, 1190, -1317, 43619, 7096, };
const TfArray<1, int> tensor_dimension32 = { 1, { 8 } };
const TfArray<8, float> quant32_scale = { 8, { 0.0012969831004738808, 0.00078444008249789476, 0.0011607178021222353, 0.0013025044463574886, 0.00097612373065203428, 0.0011365504469722509, 0.00074486812809482217, 0.00077747291652485728, } };
const TfArray<8, int> quant32_zero = { 8, { 0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant32 = { (TfLiteFloatArray*)&quant32_scale, (TfLiteIntArray*)&quant32_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data33[8*1*1*48] = { 
  /* [0][0][][] */ -10,-2,15,-90,12,26,13,8,4,9,1,0,91,0,-4,7,0,-5,-5,-2,75,-7,-4,0,25,35,13,13,6,3,8,-127,15,4,0,-2,-26,3,-12,28,13,-29,-10,7,-3,-4,1,0, 
  /* [1][0][][] */ -42,-12,-6,-8,38,33,-29,77,22,4,-30,-28,26,0,19,29,0,41,14,-42,13,25,127,19,-25,4,50,4,22,-4,14,1,10,-2,-20,-19,6,-56,14,7,-1,-6,-91,-8,-31,14,-58,-100, 
  /* [2][0][][] */ -53,9,-2,16,36,-3,35,28,-19,59,1,-10,-1,0,10,46,0,17,-127,28,-10,17,5,-8,-37,5,29,5,9,34,15,-3,-3,-21,-13,-14,11,5,8,9,32,-9,-23,8,19,-45,18,13, 
  /* [3][0][][] */ 1,-2,3,18,0,29,5,-5,57,13,-4,-1,19,0,-27,-2,0,1,-5,-1,2,-4,2,-20,-8,-5,-7,7,-4,1,4,-5,-51,37,-51,-7,8,-4,3,-127,-4,-2,-2,-46,-8,-5,-4,5, 
  /* [4][0][][] */ -4,-4,91,33,5,-20,3,-7,7,-1,2,2,-76,0,-12,5,0,12,13,8,72,10,3,-7,-10,37,0,44,-7,-3,3,108,15,3,-2,6,21,-4,-10,-21,-10,-127,18,0,17,-8,8,8, 
  /* [5][0][][] */ -2,47,1,-7,4,53,19,13,-29,34,3,53,28,0,-127,-7,0,19,-43,24,-7,13,-10,43,-18,-1,26,1,-14,18,23,-13,49,32,-38,-11,4,13,-49,16,33,-13,-13,35,21,63,17,17, 
  /* [6][0][][] */ -27,-57,5,11,47,-50,-20,7,30,-30,-12,101,-10,0,-127,46,0,-18,51,-21,-11,-4,26,-33,28,-7,-1,-11,46,-16,-9,-12,90,-38,35,-4,1,-12,-82,-24,-19,-17,-5,-25,-27,-89,-17,-29, 
  /* [7][0][][] */ -115,-28,-11,9,127,24,62,26,16,2,33,-29,-22,0,95,113,0,7,-4,60,-13,2,-27,38,12,10,48,9,41,70,65,-4,-18,-25,-18,-25,5,18,35,40,75,-1,-29,-21,62,6,76,76, 
};
const TfArray<4, int> tensor_dimension33 = { 4, { 8,1,1,48 } };
const TfArray<8, float> quant33_scale = { 8, { 0.055121779441833496, 0.033338703215122223, 0.049330506473779678, 0.055356435477733612, 0.0414852574467659, 0.04830339178442955, 0.031656894832849503, 0.033042598515748978, } };
const TfArray<8, int> quant33_zero = { 8, { 0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant33 = { (TfLiteFloatArray*)&quant33_scale, (TfLiteIntArray*)&quant33_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data34[48] = { 73872, 59490, 62608, 43636, -179, 14040, 3458, 43795, 24974, 31999, 26865, 7131, -47504, -127, 31217, 549, -204, 43328, 1520, 479, -15779, -5933, 59726, 39725, -17000, 39816, 60748, 18496, -2011, 13563, -7272, -49551, 11612, 97781, 24758, 831, 80303, -321, 113134, -27414, 15584, -29203, 2604, 118143, -567, 70582, -299, -23789, };
const TfArray<1, int> tensor_dimension34 = { 1, { 48 } };
const TfArray<48, float> quant34_scale = { 48, { 8.1334736023563892e-05, 6.7031185608357191e-05, 0.00016512218280695379, 0.00011847839050460607, 0.00016277046233881265, 0.00018542367615737021, 0.00014063685375731438, 3.8270758523140103e-05, 9.0974215709138662e-05, 2.8123384254286066e-05, 4.5896285882918164e-05, 3.2580279366811737e-05, 0.00016925907402765006, 0.005541567225009203, 8.1874604802578688e-05, 0.00022875642753206193, 0.0027484172023832798, 1.3060654055152554e-05, 8.33583326311782e-05, 0.00014488278247881681, 0.00011659408482955769, 7.9000186815392226e-05, 9.9479024356696755e-05, 2.9388515031314455e-05, 7.781690510455519e-05, 6.7997963924426585e-05, 2.1052203010185622e-05, 4.7233072109520435e-05, 0.00026374394656158984, 5.0960130465682596e-05, 5.1109509513480589e-05, 0.00017019358347170055, 0.00021154471323825419, 9.9788112493115477e-06, 0.000100988159829285, 0.00024257040058728307, 9.5837131084408611e-06, 0.00038955913623794913, 3.9828657463658601e-05, 9.5664538093842566e-05, 6.6859960497822613e-05, 0.00024250066780950874, 0.00010869360994547606, 9.3562821348314174e-06, 0.00016193663759622723, 8.4120765677653253e-05, 0.00023276155116036534, 7.032864959910512e-05, } };
const TfArray<48, int> quant34_zero = { 48, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant34 = { (TfLiteFloatArray*)&quant34_scale, (TfLiteIntArray*)&quant34_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data35[1*3*3*48] = { 
  /* [0][0][][] */ -43,-22,-6,-16,6,-39,-80,-71,-25,-52,10,66,38,-71,115,-43,-113,-83,38,-6,-1,-68,-111,-77,-36,-119,-60,-16,69,14,97,25,-127,-92,-10,57,16,47,-127,79,-76,-13,101,-110,-2,-34,-86,106, 77,11,-26,-27,57,-127,-127,-86,60,-77,-114,115,37,-82,-1,60,-52,-116,62,6,11,-116,-127,-69,-3,-127,-120,-35,-58,20,-56,53,29,-127,63,90,12,72,-91,127,-127,5,122,-127,2,-23,-126,95, -32,43,6,-5,-65,-111,-58,-19,88,-9,-127,40,7,-71,-101,-12,-127,-27,29,13,-15,-79,-35,-24,-11,-31,-55,-9,-10,18,-26,8,105,-83,68,32,12,37,-6,15,-78,5,6,-85,-18,6,-55,16, 
  /* [0][1][][] */ -85,-127,-45,-44,-11,55,95,-105,-25,-110,-22,111,91,-64,27,-96,-121,-92,89,-76,40,-87,-103,-127,41,-114,-79,-35,127,-59,127,83,-43,-89,-17,90,-34,-77,-108,91,41,61,106,-108,-92,-118,81,118, 127,-85,-127,-127,127,41,91,-127,101,-127,-109,127,127,-14,-7,127,-60,-127,127,-127,127,-127,-122,-120,127,-99,-127,-127,-123,-127,-71,127,29,-117,127,127,-70,-127,-97,90,60,127,127,-120,-114,-127,127,127, -53,33,-62,-86,-119,-24,31,-39,127,-20,-77,35,54,-70,-26,-45,-37,-31,54,-71,55,-87,-40,-25,47,-5,-44,-59,15,-89,-37,20,31,-94,115,26,-37,-62,0,-17,36,55,12,-80,-60,-39,66,38, 
  /* [0][2][][] */ -62,-108,-34,-36,-1,37,-5,-37,-38,-60,-17,59,79,-127,-127,-63,-43,-19,63,76,50,-35,-10,-58,31,-51,-8,-58,75,28,78,62,105,-30,-40,-10,-98,35,-19,31,9,39,15,-56,96,-78,4,30, 88,-74,-89,-89,81,20,6,-48,48,-72,-34,55,85,-26,17,93,-24,-32,76,119,81,-55,-16,-56,93,-68,-16,-99,-52,39,-47,110,-17,-51,65,4,-127,50,-17,-1,10,32,16,-65,127,-100,-12,42, -29,30,-43,-39,-80,-18,21,-11,60,-5,-15,32,32,-43,113,-30,-93,-5,31,61,73,-29,1,-5,41,3,-2,-65,-14,30,-21,40,-90,-41,62,3,-21,22,25,-39,7,37,-6,-19,61,-36,-4,20, 
};
const TfArray<4, int> tensor_dimension35 = { 4, { 1,3,3,48 } };
const TfArray<48, float> quant35_scale = { 48, { 0.0034567262046039104, 0.0028488251846283674, 0.0070176925510168076, 0.0050353314727544785, 0.0069177444092929363, 0.0078805061057209969, 0.005977066233754158, 0.0016265071462839842, 0.0038664040621370077, 0.0011952437926083803, 0.0019505921518430114, 0.0013846618821844459, 0.0071935104206204414, 0.23551659286022186, 0.0034796707332134247, 0.0097221480682492256, 0.1168077290058136, 0.000555077800527215, 0.0035427291877567768, 0.0061575183644890785, 0.0049552484415471554, 0.0033575077541172504, 0.0042278585024178028, 0.0012490118388086557, 0.0033072184305638075, 0.0028899135068058968, 0.00089471862884238362, 0.0020074055064469576, 0.011209117248654366, 0.0021658055484294891, 0.0021721541415899992, 0.0072332271374762058, 0.0089906500652432442, 0.00042409947491250932, 0.0042919968254864216, 0.010309241712093353, 0.00040730781620368361, 0.016556262969970703, 0.0016927178949117661, 0.0040657427161931992, 0.0028415482956916094, 0.010306278243660927, 0.0046194782480597496, 0.00039764196844771504, 0.0068823071196675301, 0.0035751324612647295, 0.0098923658952116966, 0.0029889675788581371, } };
const TfArray<48, int> quant35_zero = { 48, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant35 = { (TfLiteFloatArray*)&quant35_scale, (TfLiteIntArray*)&quant35_zero, 3 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data36[48] = { 4395, 128, 12916, 7327, 6775, 8077, 7513, -48, 40, -344, 2853, 529, 13870, -88816464, 3536, 19155, -71697256, -112, -420, 11164, 3708, -16, 66, 394, 4398, -814, 82, 703, 5514, 2157, 2971, 15694, 3689, 292, 280, -2704, 394, 5020, -203, 11832, 2933, 17974, -270, -90, 15476, 124, 10468, 5395, };
const TfArray<1, int> tensor_dimension36 = { 1, { 48 } };
const TfArray<48, float> quant36_scale = { 48, { 0.00047547667054459453, 0.00054281711345538497, 0.00024711282458156347, 0.00026402150979265571, 0.00029443451785482466, 0.00026164753944613039, 0.00050673395162448287, 0.0010635944781824946, 0.00049696629866957664, 0.0021230911370366812, 0.0005143652088008821, 0.0015359858516603708, 0.00016144770779646933, 1.8260540857184537e-09, 0.00075265701161697507, 0.0001730416261125356, 1.8260540857184537e-09, 0.0062263868749141693, 0.00042428032611496747, 0.00034589000279083848, 0.00029363788780756295, 0.0039342823438346386, 0.00027889886405318975, 0.0015374142676591873, 0.00085755705367773771, 0.00063135521486401558, 0.0018281006487086415, 0.0017485951539129019, 0.00020614457025658339, 0.0013538493076339364, 0.0010012212442234159, 0.00014863244723528624, 0.00019858121231663972, 0.010896557010710239, 0.00052175519522279501, 0.00044463583617471159, 0.0038151813205331564, 0.00025276318774558604, 0.0020921609830111265, 0.0002927432069554925, 0.000977511634118855, 0.00012519265874288976, 0.0005677337758243084, 0.010047964751720428, 0.00032474115141667426, 0.00040098879253491759, 0.00027080636937171221, 0.00016922327631618828, } };
const TfArray<48, int> quant36_zero = { 48, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant36 = { (TfLiteFloatArray*)&quant36_scale, (TfLiteIntArray*)&quant36_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data37[48*1*1*8] = { 
  /* [0][0][][] */ 7,-127,44,-47,22,-22,-43,24, 
  /* [1][0][][] */ 67,64,-37,-111,9,-11,-99,127, 
  /* [2][0][][] */ 30,13,-70,127,69,116,36,104, 
  /* [3][0][][] */ 90,79,-112,127,-114,-21,-23,-1, 
  /* [4][0][][] */ -37,-93,64,9,16,28,-32,-127, 
  /* [5][0][][] */ 30,0,-24,47,-10,-32,54,127, 
  /* [6][0][][] */ 47,-15,-121,-75,127,23,59,-44, 
  /* [7][0][][] */ 127,11,28,-27,22,-12,77,-69, 
  /* [8][0][][] */ 16,-89,9,77,5,9,-16,-127, 
  /* [9][0][][] */ 96,-124,15,-4,4,27,-127,80, 
  /* [10][0][][] */ -121,-27,-127,23,1,6,-81,43, 
  /* [11][0][][] */ -22,-42,-8,-103,-127,-40,54,-16, 
  /* [12][0][][] */ 13,11,-47,57,-66,-34,47,127, 
  /* [13][0][][] */ 0,0,0,0,0,0,0,0, 
  /* [14][0][][] */ -2,-21,0,11,49,127,44,47, 
  /* [15][0][][] */ 40,10,16,-127,90,58,-56,75, 
  /* [16][0][][] */ 0,0,0,0,0,0,0,0, 
  /* [17][0][][] */ 18,16,127,16,-32,-7,19,18, 
  /* [18][0][][] */ 78,-127,21,27,-18,14,-96,-7, 
  /* [19][0][][] */ -127,-26,23,61,-61,-75,-79,51, 
  /* [20][0][][] */ 40,21,-47,108,-32,-127,-5,49, 
  /* [21][0][][] */ 62,23,127,8,-20,-14,59,4, 
  /* [22][0][][] */ -79,-22,-127,12,1,15,-104,0, 
  /* [23][0][][] */ -86,113,-21,-9,13,-1,127,-54, 
  /* [24][0][][] */ 9,-69,-63,91,-18,-65,127,127, 
  /* [25][0][][] */ 15,30,38,22,127,97,-68,-51, 
  /* [26][0][][] */ -82,-40,-127,-8,4,26,-108,4, 
  /* [27][0][][] */ 3,-11,-36,13,-39,127,49,53, 
  /* [28][0][][] */ 8,-22,-29,42,22,127,57,66, 
  /* [29][0][][] */ 86,35,127,-8,-34,-3,105,6, 
  /* [30][0][][] */ -95,127,-34,-15,10,-11,121,-45, 
  /* [31][0][][] */ -39,-95,-11,12,42,-61,127,115, 
  /* [32][0][][] */ 0,11,-38,99,-20,69,78,127, 
  /* [33][0][][] */ -5,69,-15,-88,-2,-4,2,127, 
  /* [34][0][][] */ -18,73,-8,-70,26,1,24,127, 
  /* [35][0][][] */ 127,28,10,-56,37,39,115,-77, 
  /* [36][0][][] */ -111,-1,19,39,-38,27,-127,67, 
  /* [37][0][][] */ -6,-48,-43,26,47,127,61,51, 
  /* [38][0][][] */ 10,23,4,90,127,-8,-25,37, 
  /* [39][0][][] */ 12,38,41,-119,12,3,-35,127, 
  /* [40][0][][] */ 123,19,127,16,2,2,88,7, 
  /* [41][0][][] */ 5,-26,-31,86,79,127,62,121, 
  /* [42][0][][] */ -70,-16,127,8,-23,23,-53,73, 
  /* [43][0][][] */ -13,-40,11,105,-24,15,20,-127, 
  /* [44][0][][] */ 40,-39,-95,-127,121,-74,-20,-126, 
  /* [45][0][][] */ -93,127,-4,-65,7,-11,82,-80, 
  /* [46][0][][] */ -127,-27,44,35,-27,45,-93,48, 
  /* [47][0][][] */ 76,30,127,0,24,-9,102,-21, 
};
const TfArray<4, int> tensor_dimension37 = { 4, { 48,1,1,8 } };
const TfArray<48, float> quant37_scale = { 48, { 0.0010251369094476104, 0.0011703242780640721, 0.00053278007544577122, 0.00056923547526821494, 0.0006348065217025578, 0.00056411715922877192, 0.0010925282258540392, 0.0022931303828954697, 0.0010714689269661903, 0.0045774257741868496, 0.0011089814361184835, 0.0033116154372692108, 0.00034808440250344574, 3.937008052901092e-09, 0.0016227430896833539, 0.00037308110040612519, 3.937008052901092e-09, 0.01342421118170023, 0.00091475661611184478, 0.00074574555037543178, 0.00063308898825198412, 0.0084823891520500183, 0.00060131138889119029, 0.0033146950881928205, 0.0018489096546545625, 0.0013612140901386738, 0.0039414204657077789, 0.0037700051907449961, 0.00044445169623941183, 0.0029189253691583872, 0.002158652525395155, 0.00032045444822870195, 0.00042814493644982576, 0.023493187502026558, 0.0011249143863096833, 0.00095864350441843271, 0.0082256048917770386, 0.00054496235679835081, 0.0045107398182153702, 0.00063116004457697272, 0.0021075340919196606, 0.00026991780032403767, 0.0012240450596436858, 0.0216636061668396, 0.0007001482299529016, 0.00086453964468091726, 0.00058386381715536118, 0.00036484867450781167, } };
const TfArray<48, int> quant37_zero = { 48, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant37 = { (TfLiteFloatArray*)&quant37_scale, (TfLiteIntArray*)&quant37_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data38[8] = { 3815, -42326, -7226, -10016, -35325, -4544, -24186, -27815, };
const TfArray<1, int> tensor_dimension38 = { 1, { 8 } };
const TfArray<8, float> quant38_scale = { 8, { 0.0023358764592558146, 0.0021375359501689672, 0.0023109798785299063, 0.0012327536242082715, 0.0014442401006817818, 0.0019293391378596425, 0.0015712340828031301, 0.0010077165206894279, } };
const TfArray<8, int> quant38_zero = { 8, { 0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant38 = { (TfLiteFloatArray*)&quant38_scale, (TfLiteIntArray*)&quant38_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data39[8*1*1*16] = { 
  /* [0][0][][] */ 0,-21,4,-9,116,-4,-7,-5,-5,-127,-87,-10,0,115,-9,0, 
  /* [1][0][][] */ 0,-31,0,83,76,17,18,11,-4,-4,127,-42,0,25,-16,0, 
  /* [2][0][][] */ 0,0,-3,-11,-58,7,10,2,4,7,-20,-1,0,127,-9,0, 
  /* [3][0][][] */ 0,-15,-18,-127,84,28,72,5,-5,54,-6,41,0,-10,16,0, 
  /* [4][0][][] */ 0,45,-31,35,-59,36,127,22,5,-51,19,-5,0,2,22,0, 
  /* [5][0][][] */ 0,-16,-9,4,-22,-20,-6,14,3,10,-9,-48,0,-5,127,0, 
  /* [6][0][][] */ 0,12,-2,4,52,-20,-22,-17,-5,-110,113,68,0,127,59,0, 
  /* [7][0][][] */ 0,4,-24,127,8,4,19,16,-5,65,-50,72,0,-10,56,0, 
};
const TfArray<4, int> tensor_dimension39 = { 4, { 8,1,1,16 } };
const TfArray<8, float> quant39_scale = { 8, { 0.09927474707365036, 0.090845271944999695, 0.098216637969017029, 0.052392028272151947, 0.061380203813314438, 0.081996910274028778, 0.066777445375919342, 0.042827948927879333, } };
const TfArray<8, int> quant39_zero = { 8, { 0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant39 = { (TfLiteFloatArray*)&quant39_scale, (TfLiteIntArray*)&quant39_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data40[16] = { -161, 18973, 79443, 10424, 12484, 66600, 13733, 54172, -10596, 4765, 29441, 14390, -445, 27378, 8867, -107, };
const TfArray<1, int> tensor_dimension40 = { 1, { 16 } };
const TfArray<16, float> quant40_scale = { 16, { 0.0018688903655856848, 0.0001359438756480813, 5.0457852921681479e-05, 0.00034007200156338513, 0.00014685274800285697, 4.9880974984262139e-05, 0.00037928056553937495, 7.3119037551805377e-05, 1.8391210687696002e-05, 0.0003461727756075561, 0.00020189647329971194, 0.00071630667662248015, 0.00072724185883998871, 0.00022409252414945513, 0.00024131576356012374, 0.003907486330717802, } };
const TfArray<16, int> quant40_zero = { 16, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant40 = { (TfLiteFloatArray*)&quant40_scale, (TfLiteIntArray*)&quant40_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data41[1*3*3*16] = { 
  /* [0][0][][] */ -108,19,-83,-127,39,-67,80,-75,-90,-3,-8,29,-127,5,54,-73, 51,25,-127,127,127,-127,127,-127,-85,127,-127,53,-105,6,127,-25, 24,-24,89,-2,-72,65,-95,76,-95,-27,4,-17,61,2,-102,-29, 
  /* [0][1][][] */ -127,127,-24,13,-43,15,-29,13,-28,1,3,-112,-49,9,-30,-14, -78,-41,-58,-15,-7,-26,8,-28,-108,-89,21,-11,-104,-127,1,23, 14,2,63,2,3,32,-26,33,-127,7,-2,0,-21,12,-4,43, 
  /* [0][2][][] */ -115,-71,66,2,-6,39,-40,51,-76,0,-1,6,40,2,-27,23, 23,-3,30,-2,-26,39,-35,20,-125,-13,-1,86,-58,0,-29,-111, 21,-6,18,0,5,13,9,21,-115,5,-1,-127,107,3,17,-127, 
};
const TfArray<4, int> tensor_dimension41 = { 4, { 1,3,3,16 } };
const TfArray<16, float> quant41_scale = { 16, { 0.079427838325500488, 0.0057776146568357944, 0.0021444587036967278, 0.014453059993684292, 0.0062412419356405735, 0.0021199414040893316, 0.016119424253702164, 0.0031075591687113047, 0.00078162644058465958, 0.01471234206110239, 0.0085805999115109444, 0.030443033203482628, 0.030907778069376945, 0.0095239318907260895, 0.010255919769406319, 0.16606816649436951, } };
const TfArray<16, int> quant41_zero = { 16, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant41 = { (TfLiteFloatArray*)&quant41_scale, (TfLiteIntArray*)&quant41_zero, 3 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int32_t tensor_data42[16] = { -1073741824, 34741, -43334, -139, 172101, 47097, 45870, 23466, -293859, 21622, 18030, 1195829, -1073741824, 24142, 41623, -1073741824, };
const TfArray<1, int> tensor_dimension42 = { 1, { 16 } };
const TfArray<16, float> quant42_scale = { 16, { 1.3102037232926023e-09, 0.00016120448708534241, 0.0003605083329603076, 4.7103014367166907e-05, 1.5739093214506283e-05, 0.00019753238302655518, 7.1599606599193066e-05, 0.00028553276206366718, 7.250047019624617e-06, 4.5887107262387872e-05, 0.00017950707115232944, 4.9953141569858417e-06, 1.1533876076441629e-09, 0.00013549135474022478, 5.2387615141924471e-05, 1.243374708748135e-10, } };
const TfArray<16, int> quant42_zero = { 16, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant42 = { (TfLiteFloatArray*)&quant42_scale, (TfLiteIntArray*)&quant42_zero, 0 };
const MODEL_SECTION(EI_MODEL_SECTION) ALIGN(16) int8_t tensor_data43[16*3*3*3] = { 
  /* [0][0][][] */ 0,0,0, 0,0,0, 0,0,0, 
  /* [0][1][][] */ 0,0,0, 0,0,0, 0,0,0, 
  /* [0][2][][] */ 0,0,0, 0,0,0, 0,0,0, 
  /* [1][0][][] */ 51,-15,15, 5,-32,0, -111,-127,-79, 
  /* [1][1][][] */ 6,-15,-14, -16,-7,-18, 9,-39,-10, 
  /* [1][2][][] */ -5,5,-15, -2,-4,-16, 43,-27,-3, 
  /* [2][0][][] */ 35,-26,-33, 82,113,-24, 47,-68,-5, 
  /* [2][1][][] */ 8,-5,-27, 74,127,-62, 32,-30,-25, 
  /* [2][2][][] */ 70,-93,5, 100,-70,-45, 56,-80,5, 
  /* [3][0][][] */ 18,43,9, 30,94,17, -14,-3,-10, 
  /* [3][1][][] */ 38,49,14, 77,127,23, -33,-33,-3, 
  /* [3][2][][] */ 19,11,10, 31,21,5, -11,-10,0, 
  /* [4][0][][] */ 27,73,8, 3,73,1, -27,-11,-7, 
  /* [4][1][][] */ 32,90,56, 18,112,38, 16,34,-12, 
  /* [4][2][][] */ 49,80,35, 40,127,18, -23,21,18, 
  /* [5][0][][] */ -8,-12,-119, -19,-12,-85, -20,15,-14, 
  /* [5][1][][] */ -11,-55,-127, -17,-60,-97, -16,-2,-13, 
  /* [5][2][][] */ -16,-5,-17, -13,-14,-1, -11,1,0, 
  /* [6][0][][] */ 25,-63,35, 47,-87,42, 12,-18,12, 
  /* [6][1][][] */ 36,-94,62, 63,-127,72, 8,-27,18, 
  /* [6][2][][] */ 23,-33,14, 16,-43,26, -11,9,0, 
  /* [7][0][][] */ -21,13,5, -94,-41,1, -28,18,1, 
  /* [7][1][][] */ -15,-13,-3, -127,-95,-11, -19,-11,2, 
  /* [7][2][][] */ -34,21,-14, -22,2,-6, -37,16,-6, 
  /* [8][0][][] */ 126,127,125, 126,127,124, 123,126,124, 
  /* [8][1][][] */ 126,126,124, 125,126,124, 120,125,123, 
  /* [8][2][][] */ 124,124,124, 122,124,124, -112,30,123, 
  /* [9][0][][] */ -12,-11,18, -1,3,16, 8,16,4, 
  /* [9][1][][] */ 99,98,-17, 86,127,13, -3,17,23, 
  /* [9][2][][] */ -12,-11,22, 14,-21,5, 21,9,-18, 
  /* [10][0][][] */ -24,-84,-20, 35,94,19, -7,-13,-4, 
  /* [10][1][][] */ -58,-104,-15, 81,127,13, -15,-27,-2, 
  /* [10][2][][] */ -30,-20,1, 38,22,-13, -7,-1,6, 
  /* [11][0][][] */ -27,-39,-11, -3,-58,-5, -9,-127,-75, 
  /* [11][1][][] */ -63,-58,-31, -52,-80,-51, -35,-83,-76, 
  /* [11][2][][] */ -33,-67,-110, 103,113,44, -67,-10,-76, 
  /* [12][0][][] */ 0,0,0, 0,0,0, 0,0,0, 
  /* [12][1][][] */ 0,0,0, 0,0,0, 0,0,0, 
  /* [12][2][][] */ 0,0,0, 0,0,0, 0,0,0, 
  /* [13][0][][] */ -49,-94,-15, -60,-108,-23, -9,-15,-4, 
  /* [13][1][][] */ 61,115,17, 57,127,26, 7,17,13, 
  /* [13][2][][] */ -8,-18,5, 0,-17,0, -1,5,-2, 
  /* [14][0][][] */ 42,-6,-39, 77,-25,-47, 31,-10,-18, 
  /* [14][1][][] */ 87,-17,-73, 127,-38,-83, 58,-17,-35, 
  /* [14][2][][] */ 58,-6,-40, 81,-15,-46, 30,-2,-15, 
  /* [15][0][][] */ 0,0,0, 0,0,0, 0,0,0, 
  /* [15][1][][] */ 0,0,0, 0,0,0, 0,0,0, 
  /* [15][2][][] */ 0,0,0, 0,0,0, 0,0,0, 
};
const TfArray<4, int> tensor_dimension43 = { 4, { 16,3,3,3 } };
const TfArray<16, float> quant43_scale = { 16, { 3.3410194077987398e-07, 0.041107140481472015, 0.091929621994495392, 0.012011268176138401, 0.004013468511402607, 0.050370756536722183, 0.018257899209856987, 0.072810851037502289, 0.0018487619236111641, 0.011701211333274841, 0.045774299651384354, 0.0012738050427287817, 2.9411381774480105e-07, 0.034550294280052185, 0.013358840718865395, 3.1706054670621597e-08, } };
const TfArray<16, int> quant43_zero = { 16, { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } };
const TfLiteAffineQuantization quant43 = { (TfLiteFloatArray*)&quant43_scale, (TfLiteIntArray*)&quant43_zero, 0 };
const TfArray<4, int> tensor_dimension44 = { 4, { 1,48,48,16 } };
const TfArray<1, float> quant44_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant44_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant44 = { (TfLiteFloatArray*)&quant44_scale, (TfLiteIntArray*)&quant44_zero, 0 };
const TfArray<4, int> tensor_dimension45 = { 4, { 1,48,48,16 } };
const TfArray<1, float> quant45_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant45_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant45 = { (TfLiteFloatArray*)&quant45_scale, (TfLiteIntArray*)&quant45_zero, 0 };
const TfArray<4, int> tensor_dimension46 = { 4, { 1,48,48,8 } };
const TfArray<1, float> quant46_scale = { 1, { 0.46381771564483643, } };
const TfArray<1, int> quant46_zero = { 1, { 0 } };
const TfLiteAffineQuantization quant46 = { (TfLiteFloatArray*)&quant46_scale, (TfLiteIntArray*)&quant46_zero, 0 };
const TfArray<4, int> tensor_dimension47 = { 4, { 1,48,48,48 } };
const TfArray<1, float> quant47_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant47_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant47 = { (TfLiteFloatArray*)&quant47_scale, (TfLiteIntArray*)&quant47_zero, 0 };
const TfArray<4, int> tensor_dimension48 = { 4, { 1,49,49,48 } };
const TfArray<1, float> quant48_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant48_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant48 = { (TfLiteFloatArray*)&quant48_scale, (TfLiteIntArray*)&quant48_zero, 0 };
const TfArray<4, int> tensor_dimension49 = { 4, { 1,24,24,48 } };
const TfArray<1, float> quant49_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant49_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant49 = { (TfLiteFloatArray*)&quant49_scale, (TfLiteIntArray*)&quant49_zero, 0 };
const TfArray<4, int> tensor_dimension50 = { 4, { 1,24,24,8 } };
const TfArray<1, float> quant50_scale = { 1, { 0.397308349609375, } };
const TfArray<1, int> quant50_zero = { 1, { 6 } };
const TfLiteAffineQuantization quant50 = { (TfLiteFloatArray*)&quant50_scale, (TfLiteIntArray*)&quant50_zero, 0 };
const TfArray<4, int> tensor_dimension51 = { 4, { 1,24,24,48 } };
const TfArray<1, float> quant51_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant51_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant51 = { (TfLiteFloatArray*)&quant51_scale, (TfLiteIntArray*)&quant51_zero, 0 };
const TfArray<4, int> tensor_dimension52 = { 4, { 1,24,24,48 } };
const TfArray<1, float> quant52_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant52_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant52 = { (TfLiteFloatArray*)&quant52_scale, (TfLiteIntArray*)&quant52_zero, 0 };
const TfArray<4, int> tensor_dimension53 = { 4, { 1,24,24,8 } };
const TfArray<1, float> quant53_scale = { 1, { 0.41611874103546143, } };
const TfArray<1, int> quant53_zero = { 1, { 4 } };
const TfLiteAffineQuantization quant53 = { (TfLiteFloatArray*)&quant53_scale, (TfLiteIntArray*)&quant53_zero, 0 };
const TfArray<4, int> tensor_dimension54 = { 4, { 1,24,24,8 } };
const TfArray<1, float> quant54_scale = { 1, { 0.60751998424530029, } };
const TfArray<1, int> quant54_zero = { 1, { -8 } };
const TfLiteAffineQuantization quant54 = { (TfLiteFloatArray*)&quant54_scale, (TfLiteIntArray*)&quant54_zero, 0 };
const TfArray<4, int> tensor_dimension55 = { 4, { 1,24,24,48 } };
const TfArray<1, float> quant55_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant55_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant55 = { (TfLiteFloatArray*)&quant55_scale, (TfLiteIntArray*)&quant55_zero, 0 };
const TfArray<4, int> tensor_dimension56 = { 4, { 1,25,25,48 } };
const TfArray<1, float> quant56_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant56_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant56 = { (TfLiteFloatArray*)&quant56_scale, (TfLiteIntArray*)&quant56_zero, 0 };
const TfArray<4, int> tensor_dimension57 = { 4, { 1,12,12,48 } };
const TfArray<1, float> quant57_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant57_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant57 = { (TfLiteFloatArray*)&quant57_scale, (TfLiteIntArray*)&quant57_zero, 0 };
const TfArray<4, int> tensor_dimension58 = { 4, { 1,12,12,16 } };
const TfArray<1, float> quant58_scale = { 1, { 0.30237498879432678, } };
const TfArray<1, int> quant58_zero = { 1, { 19 } };
const TfLiteAffineQuantization quant58 = { (TfLiteFloatArray*)&quant58_scale, (TfLiteIntArray*)&quant58_zero, 0 };
const TfArray<4, int> tensor_dimension59 = { 4, { 1,12,12,96 } };
const TfArray<1, float> quant59_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant59_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant59 = { (TfLiteFloatArray*)&quant59_scale, (TfLiteIntArray*)&quant59_zero, 0 };
const TfArray<4, int> tensor_dimension60 = { 4, { 1,12,12,96 } };
const TfArray<1, float> quant60_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant60_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant60 = { (TfLiteFloatArray*)&quant60_scale, (TfLiteIntArray*)&quant60_zero, 0 };
const TfArray<4, int> tensor_dimension61 = { 4, { 1,12,12,16 } };
const TfArray<1, float> quant61_scale = { 1, { 0.21634350717067719, } };
const TfArray<1, int> quant61_zero = { 1, { 6 } };
const TfLiteAffineQuantization quant61 = { (TfLiteFloatArray*)&quant61_scale, (TfLiteIntArray*)&quant61_zero, 0 };
const TfArray<4, int> tensor_dimension62 = { 4, { 1,12,12,16 } };
const TfArray<1, float> quant62_scale = { 1, { 0.35596859455108643, } };
const TfArray<1, int> quant62_zero = { 1, { 13 } };
const TfLiteAffineQuantization quant62 = { (TfLiteFloatArray*)&quant62_scale, (TfLiteIntArray*)&quant62_zero, 0 };
const TfArray<4, int> tensor_dimension63 = { 4, { 1,12,12,96 } };
const TfArray<1, float> quant63_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant63_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant63 = { (TfLiteFloatArray*)&quant63_scale, (TfLiteIntArray*)&quant63_zero, 0 };
const TfArray<4, int> tensor_dimension64 = { 4, { 1,12,12,96 } };
const TfArray<1, float> quant64_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant64_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant64 = { (TfLiteFloatArray*)&quant64_scale, (TfLiteIntArray*)&quant64_zero, 0 };
const TfArray<4, int> tensor_dimension65 = { 4, { 1,12,12,16 } };
const TfArray<1, float> quant65_scale = { 1, { 0.24898000061511993, } };
const TfArray<1, int> quant65_zero = { 1, { 8 } };
const TfLiteAffineQuantization quant65 = { (TfLiteFloatArray*)&quant65_scale, (TfLiteIntArray*)&quant65_zero, 0 };
const TfArray<4, int> tensor_dimension66 = { 4, { 1,12,12,16 } };
const TfArray<1, float> quant66_scale = { 1, { 0.40978723764419556, } };
const TfArray<1, int> quant66_zero = { 1, { 8 } };
const TfLiteAffineQuantization quant66 = { (TfLiteFloatArray*)&quant66_scale, (TfLiteIntArray*)&quant66_zero, 0 };
const TfArray<4, int> tensor_dimension67 = { 4, { 1,12,12,96 } };
const TfArray<1, float> quant67_scale = { 1, { 0.023529412224888802, } };
const TfArray<1, int> quant67_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant67 = { (TfLiteFloatArray*)&quant67_scale, (TfLiteIntArray*)&quant67_zero, 0 };
const TfArray<4, int> tensor_dimension68 = { 4, { 1,12,12,32 } };
const TfArray<1, float> quant68_scale = { 1, { 0.054397460073232651, } };
const TfArray<1, int> quant68_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant68 = { (TfLiteFloatArray*)&quant68_scale, (TfLiteIntArray*)&quant68_zero, 0 };
const TfArray<4, int> tensor_dimension69 = { 4, { 1,12,12,3 } };
const TfArray<1, float> quant69_scale = { 1, { 0.11296643316745758, } };
const TfArray<1, int> quant69_zero = { 1, { -11 } };
const TfLiteAffineQuantization quant69 = { (TfLiteFloatArray*)&quant69_scale, (TfLiteIntArray*)&quant69_zero, 0 };
const TfArray<4, int> tensor_dimension70 = { 4, { 1,12,12,3 } };
const TfArray<1, float> quant70_scale = { 1, { 0.00390625, } };
const TfArray<1, int> quant70_zero = { 1, { -128 } };
const TfLiteAffineQuantization quant70 = { (TfLiteFloatArray*)&quant70_scale, (TfLiteIntArray*)&quant70_zero, 0 };
const TfLiteConvParams opdata0 = { kTfLitePaddingSame, 2,2, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs0 = { 3, { 0,43,42 } };
const TfArray<1, int> outputs0 = { 1, { 44 } };
const TfLiteDepthwiseConvParams opdata1 = { kTfLitePaddingSame, 1,1, 1, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs1 = { 3, { 44,41,40 } };
const TfArray<1, int> outputs1 = { 1, { 45 } };
const TfLiteConvParams opdata2 = { kTfLitePaddingSame, 1,1, kTfLiteActNone, 1,1 };
const TfArray<3, int> inputs2 = { 3, { 45,39,38 } };
const TfArray<1, int> outputs2 = { 1, { 46 } };
const TfLiteConvParams opdata3 = { kTfLitePaddingSame, 1,1, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs3 = { 3, { 46,37,36 } };
const TfArray<1, int> outputs3 = { 1, { 47 } };
const TfArray<2, int> inputs4 = { 2, { 47,1 } };
const TfArray<1, int> outputs4 = { 1, { 48 } };
const TfLiteDepthwiseConvParams opdata5 = { kTfLitePaddingValid, 2,2, 1, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs5 = { 3, { 48,35,34 } };
const TfArray<1, int> outputs5 = { 1, { 49 } };
const TfLiteConvParams opdata6 = { kTfLitePaddingSame, 1,1, kTfLiteActNone, 1,1 };
const TfArray<3, int> inputs6 = { 3, { 49,33,32 } };
const TfArray<1, int> outputs6 = { 1, { 50 } };
const TfLiteConvParams opdata7 = { kTfLitePaddingSame, 1,1, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs7 = { 3, { 50,31,30 } };
const TfArray<1, int> outputs7 = { 1, { 51 } };
const TfLiteDepthwiseConvParams opdata8 = { kTfLitePaddingSame, 1,1, 1, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs8 = { 3, { 51,29,28 } };
const TfArray<1, int> outputs8 = { 1, { 52 } };
const TfLiteConvParams opdata9 = { kTfLitePaddingSame, 1,1, kTfLiteActNone, 1,1 };
const TfArray<3, int> inputs9 = { 3, { 52,27,26 } };
const TfArray<1, int> outputs9 = { 1, { 53 } };
const TfLiteAddParams opdata10 = { kTfLiteActNone };
const TfArray<2, int> inputs10 = { 2, { 50,53 } };
const TfArray<1, int> outputs10 = { 1, { 54 } };
const TfLiteConvParams opdata11 = { kTfLitePaddingSame, 1,1, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs11 = { 3, { 54,25,24 } };
const TfArray<1, int> outputs11 = { 1, { 55 } };
const TfArray<2, int> inputs12 = { 2, { 55,1 } };
const TfArray<1, int> outputs12 = { 1, { 56 } };
const TfLiteDepthwiseConvParams opdata13 = { kTfLitePaddingValid, 2,2, 1, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs13 = { 3, { 56,23,22 } };
const TfArray<1, int> outputs13 = { 1, { 57 } };
const TfLiteConvParams opdata14 = { kTfLitePaddingSame, 1,1, kTfLiteActNone, 1,1 };
const TfArray<3, int> inputs14 = { 3, { 57,21,20 } };
const TfArray<1, int> outputs14 = { 1, { 58 } };
const TfLiteConvParams opdata15 = { kTfLitePaddingSame, 1,1, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs15 = { 3, { 58,19,18 } };
const TfArray<1, int> outputs15 = { 1, { 59 } };
const TfLiteDepthwiseConvParams opdata16 = { kTfLitePaddingSame, 1,1, 1, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs16 = { 3, { 59,17,16 } };
const TfArray<1, int> outputs16 = { 1, { 60 } };
const TfLiteConvParams opdata17 = { kTfLitePaddingSame, 1,1, kTfLiteActNone, 1,1 };
const TfArray<3, int> inputs17 = { 3, { 60,15,14 } };
const TfArray<1, int> outputs17 = { 1, { 61 } };
const TfLiteAddParams opdata18 = { kTfLiteActNone };
const TfArray<2, int> inputs18 = { 2, { 58,61 } };
const TfArray<1, int> outputs18 = { 1, { 62 } };
const TfLiteConvParams opdata19 = { kTfLitePaddingSame, 1,1, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs19 = { 3, { 62,13,12 } };
const TfArray<1, int> outputs19 = { 1, { 63 } };
const TfLiteDepthwiseConvParams opdata20 = { kTfLitePaddingSame, 1,1, 1, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs20 = { 3, { 63,11,10 } };
const TfArray<1, int> outputs20 = { 1, { 64 } };
const TfLiteConvParams opdata21 = { kTfLitePaddingSame, 1,1, kTfLiteActNone, 1,1 };
const TfArray<3, int> inputs21 = { 3, { 64,9,8 } };
const TfArray<1, int> outputs21 = { 1, { 65 } };
const TfLiteAddParams opdata22 = { kTfLiteActNone };
const TfArray<2, int> inputs22 = { 2, { 62,65 } };
const TfArray<1, int> outputs22 = { 1, { 66 } };
const TfLiteConvParams opdata23 = { kTfLitePaddingSame, 1,1, kTfLiteActRelu6, 1,1 };
const TfArray<3, int> inputs23 = { 3, { 66,7,6 } };
const TfArray<1, int> outputs23 = { 1, { 67 } };
const TfLiteConvParams opdata24 = { kTfLitePaddingValid, 1,1, kTfLiteActRelu, 1,1 };
const TfArray<3, int> inputs24 = { 3, { 67,5,4 } };
const TfArray<1, int> outputs24 = { 1, { 68 } };
const TfLiteConvParams opdata25 = { kTfLitePaddingValid, 1,1, kTfLiteActNone, 1,1 };
const TfArray<3, int> inputs25 = { 3, { 68,3,2 } };
const TfArray<1, int> outputs25 = { 1, { 69 } };
const TfLiteSoftmaxParams opdata26 = { 1 };
const TfArray<1, int> inputs26 = { 1, { 69 } };
const TfArray<1, int> outputs26 = { 1, { 70 } };
};

TensorInfo_t tensorData[] = {
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 0), (TfLiteIntArray*)&g0::tensor_dimension0, 27648, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant0))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data1, (TfLiteIntArray*)&g0::tensor_dimension1, 32, {kTfLiteNoQuantization, nullptr}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data2, (TfLiteIntArray*)&g0::tensor_dimension2, 12, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant2))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data3, (TfLiteIntArray*)&g0::tensor_dimension3, 96, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant3))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data4, (TfLiteIntArray*)&g0::tensor_dimension4, 128, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant4))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data5, (TfLiteIntArray*)&g0::tensor_dimension5, 3072, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant5))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data6, (TfLiteIntArray*)&g0::tensor_dimension6, 384, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant6))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data7, (TfLiteIntArray*)&g0::tensor_dimension7, 1536, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant7))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data8, (TfLiteIntArray*)&g0::tensor_dimension8, 64, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant8))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data9, (TfLiteIntArray*)&g0::tensor_dimension9, 1536, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant9))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data10, (TfLiteIntArray*)&g0::tensor_dimension10, 384, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant10))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data11, (TfLiteIntArray*)&g0::tensor_dimension11, 864, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant11))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data12, (TfLiteIntArray*)&g0::tensor_dimension12, 384, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant12))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data13, (TfLiteIntArray*)&g0::tensor_dimension13, 1536, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant13))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data14, (TfLiteIntArray*)&g0::tensor_dimension14, 64, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant14))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data15, (TfLiteIntArray*)&g0::tensor_dimension15, 1536, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant15))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data16, (TfLiteIntArray*)&g0::tensor_dimension16, 384, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant16))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data17, (TfLiteIntArray*)&g0::tensor_dimension17, 864, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant17))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data18, (TfLiteIntArray*)&g0::tensor_dimension18, 384, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant18))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data19, (TfLiteIntArray*)&g0::tensor_dimension19, 1536, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant19))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data20, (TfLiteIntArray*)&g0::tensor_dimension20, 64, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant20))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data21, (TfLiteIntArray*)&g0::tensor_dimension21, 768, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant21))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data22, (TfLiteIntArray*)&g0::tensor_dimension22, 192, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant22))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data23, (TfLiteIntArray*)&g0::tensor_dimension23, 432, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant23))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data24, (TfLiteIntArray*)&g0::tensor_dimension24, 192, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant24))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data25, (TfLiteIntArray*)&g0::tensor_dimension25, 384, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant25))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data26, (TfLiteIntArray*)&g0::tensor_dimension26, 32, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant26))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data27, (TfLiteIntArray*)&g0::tensor_dimension27, 384, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant27))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data28, (TfLiteIntArray*)&g0::tensor_dimension28, 192, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant28))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data29, (TfLiteIntArray*)&g0::tensor_dimension29, 432, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant29))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data30, (TfLiteIntArray*)&g0::tensor_dimension30, 192, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant30))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data31, (TfLiteIntArray*)&g0::tensor_dimension31, 384, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant31))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data32, (TfLiteIntArray*)&g0::tensor_dimension32, 32, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant32))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data33, (TfLiteIntArray*)&g0::tensor_dimension33, 384, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant33))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data34, (TfLiteIntArray*)&g0::tensor_dimension34, 192, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant34))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data35, (TfLiteIntArray*)&g0::tensor_dimension35, 432, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant35))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data36, (TfLiteIntArray*)&g0::tensor_dimension36, 192, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant36))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data37, (TfLiteIntArray*)&g0::tensor_dimension37, 384, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant37))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data38, (TfLiteIntArray*)&g0::tensor_dimension38, 32, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant38))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data39, (TfLiteIntArray*)&g0::tensor_dimension39, 128, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant39))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data40, (TfLiteIntArray*)&g0::tensor_dimension40, 64, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant40))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data41, (TfLiteIntArray*)&g0::tensor_dimension41, 144, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant41))}, },
{ kTfLiteMmapRo, kTfLiteInt32, (int32_t*)g0::tensor_data42, (TfLiteIntArray*)&g0::tensor_dimension42, 64, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant42))}, },
{ kTfLiteMmapRo, kTfLiteInt8, (int32_t*)g0::tensor_data43, (TfLiteIntArray*)&g0::tensor_dimension43, 432, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant43))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 36864), (TfLiteIntArray*)&g0::tensor_dimension44, 36864, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant44))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 0), (TfLiteIntArray*)&g0::tensor_dimension45, 36864, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant45))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 36864), (TfLiteIntArray*)&g0::tensor_dimension46, 18432, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant46))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 115248), (TfLiteIntArray*)&g0::tensor_dimension47, 110592, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant47))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 0), (TfLiteIntArray*)&g0::tensor_dimension48, 115248, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant48))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 115248), (TfLiteIntArray*)&g0::tensor_dimension49, 27648, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant49))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 55296), (TfLiteIntArray*)&g0::tensor_dimension50, 4608, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant50))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 27648), (TfLiteIntArray*)&g0::tensor_dimension51, 27648, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant51))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 0), (TfLiteIntArray*)&g0::tensor_dimension52, 27648, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant52))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 27648), (TfLiteIntArray*)&g0::tensor_dimension53, 4608, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant53))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 0), (TfLiteIntArray*)&g0::tensor_dimension54, 4608, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant54))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 30000), (TfLiteIntArray*)&g0::tensor_dimension55, 27648, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant55))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 0), (TfLiteIntArray*)&g0::tensor_dimension56, 30000, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant56))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 30000), (TfLiteIntArray*)&g0::tensor_dimension57, 6912, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant57))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 36912), (TfLiteIntArray*)&g0::tensor_dimension58, 2304, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant58))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 13824), (TfLiteIntArray*)&g0::tensor_dimension59, 13824, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant59))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 0), (TfLiteIntArray*)&g0::tensor_dimension60, 13824, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant60))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 13824), (TfLiteIntArray*)&g0::tensor_dimension61, 2304, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant61))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 27648), (TfLiteIntArray*)&g0::tensor_dimension62, 2304, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant62))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 13824), (TfLiteIntArray*)&g0::tensor_dimension63, 13824, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant63))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 0), (TfLiteIntArray*)&g0::tensor_dimension64, 13824, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant64))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 16128), (TfLiteIntArray*)&g0::tensor_dimension65, 2304, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant65))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 13824), (TfLiteIntArray*)&g0::tensor_dimension66, 2304, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant66))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 0), (TfLiteIntArray*)&g0::tensor_dimension67, 13824, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant67))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 13824), (TfLiteIntArray*)&g0::tensor_dimension68, 4608, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant68))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 432), (TfLiteIntArray*)&g0::tensor_dimension69, 432, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant69))}, },
{ kTfLiteArenaRw, kTfLiteInt8, (int32_t*)(tensor_arena + 0), (TfLiteIntArray*)&g0::tensor_dimension70, 432, {kTfLiteAffineQuantization, const_cast<void*>(static_cast<const void*>(&g0::quant70))}, },
};

#ifndef TF_LITE_STATIC_MEMORY
TfLiteNode tflNodes[27] = {
{ (TfLiteIntArray*)&g0::inputs0, (TfLiteIntArray*)&g0::outputs0, (TfLiteIntArray*)&g0::inputs0, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata0)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs1, (TfLiteIntArray*)&g0::outputs1, (TfLiteIntArray*)&g0::inputs1, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata1)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs2, (TfLiteIntArray*)&g0::outputs2, (TfLiteIntArray*)&g0::inputs2, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata2)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs3, (TfLiteIntArray*)&g0::outputs3, (TfLiteIntArray*)&g0::inputs3, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata3)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs4, (TfLiteIntArray*)&g0::outputs4, (TfLiteIntArray*)&g0::inputs4, nullptr, nullptr, nullptr, nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs5, (TfLiteIntArray*)&g0::outputs5, (TfLiteIntArray*)&g0::inputs5, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata5)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs6, (TfLiteIntArray*)&g0::outputs6, (TfLiteIntArray*)&g0::inputs6, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata6)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs7, (TfLiteIntArray*)&g0::outputs7, (TfLiteIntArray*)&g0::inputs7, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata7)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs8, (TfLiteIntArray*)&g0::outputs8, (TfLiteIntArray*)&g0::inputs8, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata8)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs9, (TfLiteIntArray*)&g0::outputs9, (TfLiteIntArray*)&g0::inputs9, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata9)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs10, (TfLiteIntArray*)&g0::outputs10, (TfLiteIntArray*)&g0::inputs10, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata10)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs11, (TfLiteIntArray*)&g0::outputs11, (TfLiteIntArray*)&g0::inputs11, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata11)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs12, (TfLiteIntArray*)&g0::outputs12, (TfLiteIntArray*)&g0::inputs12, nullptr, nullptr, nullptr, nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs13, (TfLiteIntArray*)&g0::outputs13, (TfLiteIntArray*)&g0::inputs13, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata13)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs14, (TfLiteIntArray*)&g0::outputs14, (TfLiteIntArray*)&g0::inputs14, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata14)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs15, (TfLiteIntArray*)&g0::outputs15, (TfLiteIntArray*)&g0::inputs15, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata15)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs16, (TfLiteIntArray*)&g0::outputs16, (TfLiteIntArray*)&g0::inputs16, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata16)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs17, (TfLiteIntArray*)&g0::outputs17, (TfLiteIntArray*)&g0::inputs17, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata17)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs18, (TfLiteIntArray*)&g0::outputs18, (TfLiteIntArray*)&g0::inputs18, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata18)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs19, (TfLiteIntArray*)&g0::outputs19, (TfLiteIntArray*)&g0::inputs19, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata19)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs20, (TfLiteIntArray*)&g0::outputs20, (TfLiteIntArray*)&g0::inputs20, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata20)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs21, (TfLiteIntArray*)&g0::outputs21, (TfLiteIntArray*)&g0::inputs21, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata21)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs22, (TfLiteIntArray*)&g0::outputs22, (TfLiteIntArray*)&g0::inputs22, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata22)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs23, (TfLiteIntArray*)&g0::outputs23, (TfLiteIntArray*)&g0::inputs23, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata23)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs24, (TfLiteIntArray*)&g0::outputs24, (TfLiteIntArray*)&g0::inputs24, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata24)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs25, (TfLiteIntArray*)&g0::outputs25, (TfLiteIntArray*)&g0::inputs25, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata25)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs26, (TfLiteIntArray*)&g0::outputs26, (TfLiteIntArray*)&g0::inputs26, nullptr, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata26)), nullptr, 0, },
};
#else
TfLiteNode tflNodes[27] = {
{ (TfLiteIntArray*)&g0::inputs0, (TfLiteIntArray*)&g0::outputs0, (TfLiteIntArray*)&g0::inputs0, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata0)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs1, (TfLiteIntArray*)&g0::outputs1, (TfLiteIntArray*)&g0::inputs1, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata1)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs2, (TfLiteIntArray*)&g0::outputs2, (TfLiteIntArray*)&g0::inputs2, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata2)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs3, (TfLiteIntArray*)&g0::outputs3, (TfLiteIntArray*)&g0::inputs3, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata3)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs4, (TfLiteIntArray*)&g0::outputs4, (TfLiteIntArray*)&g0::inputs4, nullptr, nullptr, nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs5, (TfLiteIntArray*)&g0::outputs5, (TfLiteIntArray*)&g0::inputs5, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata5)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs6, (TfLiteIntArray*)&g0::outputs6, (TfLiteIntArray*)&g0::inputs6, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata6)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs7, (TfLiteIntArray*)&g0::outputs7, (TfLiteIntArray*)&g0::inputs7, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata7)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs8, (TfLiteIntArray*)&g0::outputs8, (TfLiteIntArray*)&g0::inputs8, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata8)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs9, (TfLiteIntArray*)&g0::outputs9, (TfLiteIntArray*)&g0::inputs9, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata9)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs10, (TfLiteIntArray*)&g0::outputs10, (TfLiteIntArray*)&g0::inputs10, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata10)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs11, (TfLiteIntArray*)&g0::outputs11, (TfLiteIntArray*)&g0::inputs11, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata11)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs12, (TfLiteIntArray*)&g0::outputs12, (TfLiteIntArray*)&g0::inputs12, nullptr, nullptr, nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs13, (TfLiteIntArray*)&g0::outputs13, (TfLiteIntArray*)&g0::inputs13, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata13)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs14, (TfLiteIntArray*)&g0::outputs14, (TfLiteIntArray*)&g0::inputs14, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata14)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs15, (TfLiteIntArray*)&g0::outputs15, (TfLiteIntArray*)&g0::inputs15, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata15)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs16, (TfLiteIntArray*)&g0::outputs16, (TfLiteIntArray*)&g0::inputs16, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata16)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs17, (TfLiteIntArray*)&g0::outputs17, (TfLiteIntArray*)&g0::inputs17, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata17)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs18, (TfLiteIntArray*)&g0::outputs18, (TfLiteIntArray*)&g0::inputs18, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata18)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs19, (TfLiteIntArray*)&g0::outputs19, (TfLiteIntArray*)&g0::inputs19, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata19)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs20, (TfLiteIntArray*)&g0::outputs20, (TfLiteIntArray*)&g0::inputs20, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata20)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs21, (TfLiteIntArray*)&g0::outputs21, (TfLiteIntArray*)&g0::inputs21, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata21)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs22, (TfLiteIntArray*)&g0::outputs22, (TfLiteIntArray*)&g0::inputs22, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata22)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs23, (TfLiteIntArray*)&g0::outputs23, (TfLiteIntArray*)&g0::inputs23, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata23)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs24, (TfLiteIntArray*)&g0::outputs24, (TfLiteIntArray*)&g0::inputs24, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata24)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs25, (TfLiteIntArray*)&g0::outputs25, (TfLiteIntArray*)&g0::inputs25, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata25)), nullptr, 0, },
{ (TfLiteIntArray*)&g0::inputs26, (TfLiteIntArray*)&g0::outputs26, (TfLiteIntArray*)&g0::inputs26, nullptr, const_cast<void*>(static_cast<const void*>(&g0::opdata26)), nullptr, 0, },
};
#endif

used_operators_e used_ops[] =
{OP_CONV_2D, OP_DEPTHWISE_CONV_2D, OP_CONV_2D, OP_CONV_2D, OP_PAD, OP_DEPTHWISE_CONV_2D, OP_CONV_2D, OP_CONV_2D, OP_DEPTHWISE_CONV_2D, OP_CONV_2D, OP_ADD, OP_CONV_2D, OP_PAD, OP_DEPTHWISE_CONV_2D, OP_CONV_2D, OP_CONV_2D, OP_DEPTHWISE_CONV_2D, OP_CONV_2D, OP_ADD, OP_CONV_2D, OP_DEPTHWISE_CONV_2D, OP_CONV_2D, OP_ADD, OP_CONV_2D, OP_CONV_2D, OP_CONV_2D, OP_SOFTMAX, };


// Indices into tflTensors and tflNodes for subgraphs
const size_t tflTensors_subgraph_index[] = {0, 71, };
const size_t tflNodes_subgraph_index[] = {0, 27, };

// Input/output tensors
static const int in_tensor_indices[] = {
  0, 
};

static const int out_tensor_indices[] = {
  70, 
};


size_t current_subgraph_index = 0;

static void init_tflite_tensor(size_t i, TfLiteTensor *tensor) {
  tensor->type = tensorData[i].type;
  tensor->is_variable = false;

#if defined(EI_CLASSIFIER_ALLOCATION_HEAP)
  tensor->allocation_type = tensorData[i].allocation_type;
#else
  tensor->allocation_type = (tensor_arena <= tensorData[i].data && tensorData[i].data < tensor_arena + kTensorArenaSize) ? kTfLiteArenaRw : kTfLiteMmapRo;
#endif
  tensor->bytes = tensorData[i].bytes;
  tensor->dims = tensorData[i].dims;

#if defined(EI_CLASSIFIER_ALLOCATION_HEAP)
  if(tensor->allocation_type == kTfLiteArenaRw){
    uint8_t* start = (uint8_t*) ((uintptr_t)tensorData[i].data + (uintptr_t) tensor_arena);

    tensor->data.data =  start;
  }
  else {
      tensor->data.data = tensorData[i].data;
  }
#else
  tensor->data.data = tensorData[i].data;
#endif // EI_CLASSIFIER_ALLOCATION_HEAP
  tensor->quantization = tensorData[i].quantization;
  if (tensor->quantization.type == kTfLiteAffineQuantization) {
    TfLiteAffineQuantization const* quant = ((TfLiteAffineQuantization const*)(tensorData[i].quantization.params));
    tensor->params.scale = quant->scale->data[0];
    tensor->params.zero_point = quant->zero_point->data[0];
  }

}

static void init_tflite_eval_tensor(int i, TfLiteEvalTensor *tensor) {

  tensor->type = tensorData[i].type;

  tensor->dims = tensorData[i].dims;

#if defined(EI_CLASSIFIER_ALLOCATION_HEAP)
  auto allocation_type = tensorData[i].allocation_type;
  if(allocation_type == kTfLiteArenaRw) {
    uint8_t* start = (uint8_t*) ((uintptr_t)tensorData[i].data + (uintptr_t) tensor_arena);

    tensor->data.data =  start;
  }
  else {
    tensor->data.data = tensorData[i].data;
  }
#else
  tensor->data.data = tensorData[i].data;
#endif // EI_CLASSIFIER_ALLOCATION_HEAP
}

static void* overflow_buffers[EI_MAX_OVERFLOW_BUFFER_COUNT];
static size_t overflow_buffers_ix = 0;
static void * AllocatePersistentBufferImpl(struct TfLiteContext* ctx,
                                       size_t bytes) {
  void *ptr;
  uint32_t align_bytes = (bytes % 16) ? 16 - (bytes % 16) : 0;

  if (current_location - (bytes + align_bytes) < tensor_boundary) {
    if (overflow_buffers_ix > EI_MAX_OVERFLOW_BUFFER_COUNT - 1) {
      ei_printf("ERR: Failed to allocate persistent buffer of size %d, does not fit in tensor arena and reached EI_MAX_OVERFLOW_BUFFER_COUNT\n",
        (int)bytes);
      return NULL;
    }

    // OK, this will look super weird, but.... we have CMSIS-NN buffers which
    // we cannot calculate beforehand easily.
    ptr = ei_calloc(bytes, 1);
    if (ptr == NULL) {
      ei_printf("ERR: Failed to allocate persistent buffer of size %d\n", (int)bytes);
      return NULL;
    }
    overflow_buffers[overflow_buffers_ix++] = ptr;
    return ptr;
  }

  current_location -= bytes;

  // align to the left aligned boundary of 16 bytes
  current_location -= 15; // for alignment
  current_location += 16 - ((uintptr_t)(current_location) & 15);

  ptr = current_location;
  memset(ptr, 0, bytes);

  return ptr;
}

typedef struct {
  size_t bytes;
  void *ptr;
} scratch_buffer_t;

static scratch_buffer_t scratch_buffers[EI_MAX_SCRATCH_BUFFER_COUNT];
static size_t scratch_buffers_ix = 0;

static TfLiteStatus RequestScratchBufferInArenaImpl(struct TfLiteContext* ctx, size_t bytes,
                                                int* buffer_idx) {
  if (scratch_buffers_ix > EI_MAX_SCRATCH_BUFFER_COUNT - 1) {
    ei_printf("ERR: Failed to allocate scratch buffer of size %d, reached EI_MAX_SCRATCH_BUFFER_COUNT\n",
      (int)bytes);
    return kTfLiteError;
  }

  scratch_buffer_t b;
  b.bytes = bytes;

  b.ptr = AllocatePersistentBufferImpl(ctx, b.bytes);
  if (!b.ptr) {
    ei_printf("ERR: Failed to allocate scratch buffer of size %d\n",
      (int)bytes);
    return kTfLiteError;
  }

  scratch_buffers[scratch_buffers_ix] = b;
  *buffer_idx = scratch_buffers_ix;

  scratch_buffers_ix++;

  return kTfLiteOk;
}

static void* GetScratchBufferImpl(struct TfLiteContext* ctx, int buffer_idx) {
  if (buffer_idx > (int)scratch_buffers_ix) {
    return NULL;
  }
  return scratch_buffers[buffer_idx].ptr;
}

static const uint16_t TENSOR_IX_UNUSED = 0x7FFF;

static void ResetTensors() {
  for (size_t ix = 0; ix < MAX_TFL_TENSOR_COUNT; ix++) {
    tflTensors[ix].index = TENSOR_IX_UNUSED;
  }
  for (size_t ix = 0; ix < MAX_TFL_EVAL_COUNT; ix++) {
    tflEvalTensors[ix].index = TENSOR_IX_UNUSED;
  }
}

static TfLiteTensor* GetTensorImpl(const struct TfLiteContext* context,
                               int tensor_idx) {

  tensor_idx = tflTensors_subgraph_index[current_subgraph_index] + tensor_idx;

  for (size_t ix = 0; ix < MAX_TFL_TENSOR_COUNT; ix++) {
    // already used? OK!
    if (tflTensors[ix].index == tensor_idx) {
      return &tflTensors[ix].tensor;
    }
    // passed all the ones we've used, so end of the list?
    if (tflTensors[ix].index == TENSOR_IX_UNUSED) {
      // init the tensor
      init_tflite_tensor(tensor_idx, &tflTensors[ix].tensor);
      tflTensors[ix].index = tensor_idx;
      return &tflTensors[ix].tensor;
    }
  }

  ei_printf("ERR: GetTensor called beyond MAX_TFL_TENSOR_COUNT (%d)\n", MAX_TFL_TENSOR_COUNT);
  return nullptr;
}

static TfLiteEvalTensor* GetEvalTensorImpl(const struct TfLiteContext* context,
                                       int tensor_idx) {

  tensor_idx = tflTensors_subgraph_index[current_subgraph_index] + tensor_idx;

  for (size_t ix = 0; ix < MAX_TFL_EVAL_COUNT; ix++) {
    // already used? OK!
    if (tflEvalTensors[ix].index == tensor_idx) {
      return &tflEvalTensors[ix].tensor;
    }
    // passed all the ones we've used, so end of the list?
    if (tflEvalTensors[ix].index == TENSOR_IX_UNUSED) {
      // init the tensor
      init_tflite_eval_tensor(tensor_idx, &tflEvalTensors[ix].tensor);
      tflEvalTensors[ix].index = tensor_idx;
      return &tflEvalTensors[ix].tensor;
    }
  }

  ei_printf("ERR: GetTensor called beyond MAX_TFL_EVAL_COUNT (%d)\n", (int)MAX_TFL_EVAL_COUNT);
  return nullptr;
}

class EonMicroContext : public MicroContext {
 public:
 
  EonMicroContext(): MicroContext(nullptr, nullptr, nullptr) { }

  void* AllocatePersistentBuffer(size_t bytes) {
    return AllocatePersistentBufferImpl(nullptr, bytes);
  }

  TfLiteStatus RequestScratchBufferInArena(size_t bytes,
                                           int* buffer_index) {
  return RequestScratchBufferInArenaImpl(nullptr, bytes, buffer_index);
  }

  void* GetScratchBuffer(int buffer_index) {
    return GetScratchBufferImpl(nullptr, buffer_index);
  }
 
  TfLiteTensor* AllocateTempTfLiteTensor(int tensor_index) {
    return GetTensorImpl(nullptr, tensor_index);
  }

  void DeallocateTempTfLiteTensor(TfLiteTensor* tensor) {
    return;
  }

  bool IsAllTempTfLiteTensorDeallocated() {
    return true;
  }

  TfLiteEvalTensor* GetEvalTensor(int tensor_index) {
    return GetEvalTensorImpl(nullptr, tensor_index);
  }

};


} // namespace

TfLiteStatus tflite_learn_3_init( void*(*alloc_fnc)(size_t,size_t) ) {
#ifdef EI_CLASSIFIER_ALLOCATION_HEAP
  tensor_arena = (uint8_t*) alloc_fnc(16, kTensorArenaSize);
  if (!tensor_arena) {
    ei_printf("ERR: failed to allocate tensor arena\n");
    return kTfLiteError;
  }
#else
  memset(tensor_arena, 0, kTensorArenaSize);
#endif
  tensor_boundary = tensor_arena;
  current_location = tensor_arena + kTensorArenaSize;

  EonMicroContext micro_context_;
  
  // Set microcontext as the context ptr
  ctx.impl_ = static_cast<void*>(&micro_context_);
  // Setup tflitecontext functions
  ctx.AllocatePersistentBuffer = &AllocatePersistentBufferImpl;
  ctx.RequestScratchBufferInArena = &RequestScratchBufferInArenaImpl;
  ctx.GetScratchBuffer = &GetScratchBufferImpl;
  ctx.GetTensor = &GetTensorImpl;
  ctx.GetEvalTensor = &GetEvalTensorImpl;
  ctx.ReportError = &MicroContextReportOpError;

  ctx.tensors_size = 71;
  for (size_t i = 0; i < 71; ++i) {
    TfLiteTensor tensor;
    init_tflite_tensor(i, &tensor);
    if (tensor.allocation_type == kTfLiteArenaRw) {
      auto data_end_ptr = (uint8_t*)tensor.data.data + tensorData[i].bytes;
      if (data_end_ptr > tensor_boundary) {
        tensor_boundary = data_end_ptr;
      }
    }
  }

  if (tensor_boundary > current_location /* end of arena size */) {
    ei_printf("ERR: tensor arena is too small, does not fit model - even without scratch buffers\n");
    return kTfLiteError;
  }

  registrations[OP_CONV_2D] = Register_CONV_2D();
  registrations[OP_DEPTHWISE_CONV_2D] = Register_DEPTHWISE_CONV_2D();
  registrations[OP_PAD] = Register_PAD();
  registrations[OP_ADD] = Register_ADD();
  registrations[OP_SOFTMAX] = Register_SOFTMAX();

  for (size_t g = 0; g < 1; ++g) {
    current_subgraph_index = g;
    for(size_t i = tflNodes_subgraph_index[g]; i < tflNodes_subgraph_index[g+1]; ++i) {
      if (registrations[used_ops[i]].init) {
        tflNodes[i].user_data = registrations[used_ops[i]].init(&ctx, (const char*)tflNodes[i].builtin_data, 0);
      }
    }
  }
  current_subgraph_index = 0;

  for(size_t g = 0; g < 1; ++g) {
    current_subgraph_index = g;
    for(size_t i = tflNodes_subgraph_index[g]; i < tflNodes_subgraph_index[g+1]; ++i) {
      if (registrations[used_ops[i]].prepare) {
        ResetTensors();
        TfLiteStatus status = registrations[used_ops[i]].prepare(&ctx, &tflNodes[i]);
        if (status != kTfLiteOk) {
          return status;
        }
      }
    }
  }
  current_subgraph_index = 0;

  return kTfLiteOk;
}

TfLiteStatus tflite_learn_3_input(int index, TfLiteTensor *tensor) {
  init_tflite_tensor(in_tensor_indices[index], tensor);
  return kTfLiteOk;
}

TfLiteStatus tflite_learn_3_output(int index, TfLiteTensor *tensor) {
  init_tflite_tensor(out_tensor_indices[index], tensor);
  return kTfLiteOk;
}

TfLiteStatus tflite_learn_3_invoke() {
  for (size_t i = 0; i < 27; ++i) {
    ResetTensors();

    TfLiteStatus status = registrations[used_ops[i]].invoke(&ctx, &tflNodes[i]);

#if EI_CLASSIFIER_PRINT_STATE
    ei_printf("layer %lu\n", i);
    ei_printf("    inputs:\n");
    for (size_t ix = 0; ix < tflNodes[i].inputs->size; ix++) {
      auto d = tensorData[tflNodes[i].inputs->data[ix]];

      size_t data_ptr = (size_t)d.data;

      if (d.allocation_type == kTfLiteArenaRw) {
        data_ptr = (size_t)tensor_arena + data_ptr;
      }

      if (d.type == TfLiteType::kTfLiteInt8) {
        int8_t* data = (int8_t*)data_ptr;
        ei_printf("        %lu (%zu bytes, ptr=%p, alloc_type=%d, type=%d): ", ix, d.bytes, data, (int)d.allocation_type, (int)d.type);
        for (size_t jx = 0; jx < d.bytes; jx++) {
          ei_printf("%d ", data[jx]);
        }
      }
      else {
        float* data = (float*)data_ptr;
        ei_printf("        %lu (%zu bytes, ptr=%p, alloc_type=%d, type=%d): ", ix, d.bytes, data, (int)d.allocation_type, (int)d.type);
        for (size_t jx = 0; jx < d.bytes / 4; jx++) {
          ei_printf("%f ", data[jx]);
        }
      }
      ei_printf("\n");
    }
    ei_printf("\n");

    ei_printf("    outputs:\n");
    for (size_t ix = 0; ix < tflNodes[i].outputs->size; ix++) {
      auto d = tensorData[tflNodes[i].outputs->data[ix]];

      size_t data_ptr = (size_t)d.data;

      if (d.allocation_type == kTfLiteArenaRw) {
        data_ptr = (size_t)tensor_arena + data_ptr;
      }

      if (d.type == TfLiteType::kTfLiteInt8) {
        int8_t* data = (int8_t*)data_ptr;
        ei_printf("        %lu (%zu bytes, ptr=%p, alloc_type=%d, type=%d): ", ix, d.bytes, data, (int)d.allocation_type, (int)d.type);
        for (size_t jx = 0; jx < d.bytes; jx++) {
          ei_printf("%d ", data[jx]);
        }
      }
      else {
        float* data = (float*)data_ptr;
        ei_printf("        %lu (%zu bytes, ptr=%p, alloc_type=%d, type=%d): ", ix, d.bytes, data, (int)d.allocation_type, (int)d.type);
        for (size_t jx = 0; jx < d.bytes / 4; jx++) {
          ei_printf("%f ", data[jx]);
        }
      }
      ei_printf("\n");
    }
    ei_printf("\n");
#endif // EI_CLASSIFIER_PRINT_STATE

    if (status != kTfLiteOk) {
      return status;
    }
  }
  return kTfLiteOk;
}

TfLiteStatus tflite_learn_3_reset( void (*free_fnc)(void* ptr) ) {
#ifdef EI_CLASSIFIER_ALLOCATION_HEAP
  free_fnc(tensor_arena);
#endif

  // scratch buffers are allocated within the arena, so just reset the counter so memory can be reused
  scratch_buffers_ix = 0;

  // overflow buffers are on the heap, so free them first
  for (size_t ix = 0; ix < overflow_buffers_ix; ix++) {
    ei_free(overflow_buffers[ix]);
  }
  overflow_buffers_ix = 0;
  return kTfLiteOk;
}
